*> \brief \b DLARFG
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLARFG + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlarfg.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlarfg.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlarfg.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLARFG( N, ALPHA, X, INCX, TAU )
* 
*       .. Scalar Arguments ..
*       INTEGER            INCX, N
*       DOUBLE PRECISION   ALPHA, TAU
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   X( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLARFG generates a real elementary reflector H of order n, such
*> that
*>
*>       H * ( alpha ) = ( beta ),   H**T * H = I.
*>           (   x   )   (   0  )
*>
*> where alpha and beta are scalars, and x is an (n-1)-element real
*> vector. H is represented in the form
*>
*>       H = I - tau * ( 1 ) * ( 1 v**T ) ,
*>                     ( v )
*>
*> where tau is a real scalar and v is a real (n-1)-element
*> vector.
*>
*> If the elements of x are all zero, then tau = 0 and H is taken to be
*> the unit matrix.
*>
*> Otherwise  1 <= tau <= 2.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The order of the elementary reflector.
*> \endverbatim
*>
*> \param[in,out] ALPHA
*> \verbatim
*>          ALPHA is DOUBLE PRECISION
*>          On entry, the value alpha.
*>          On exit, it is overwritten with the value beta.
*> \endverbatim
*>
*> \param[in,out] X
*> \verbatim
*>          X is DOUBLE PRECISION array, dimension
*>                         (1+(N-2)*abs(INCX))
*>          On entry, the vector x.
*>          On exit, it is overwritten with the vector v.
*> \endverbatim
*>
*> \param[in] INCX
*> \verbatim
*>          INCX is INTEGER
*>          The increment between elements of X. INCX > 0.
*> \endverbatim
*>
*> \param[out] TAU
*> \verbatim
*>          TAU is DOUBLE PRECISION
*>          The value tau.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup doubleOTHERauxiliary
*
*  =====================================================================
!!      SUBROUTINE DLARFG( N, ALPHA, X, INCX, TAU )
      SUBROUTINE dDLARFG( N, ALPHAh, ALPHAl, Xh, Xl, INCX, TAUh, TAUl )
*
*  -- LAPACK auxiliary routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      INTEGER            INCX, N
!!      DOUBLE PRECISION   ALPHA, TAU
      double precision ALPHAh, ALPHAl
      double precision TAUh, TAUl
*     ..
*     .. Array Arguments ..
!!      DOUBLE PRECISION   X( * )
      double precision Xh(*), Xl(*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!!      DOUBLE PRECISION   ONE, ZERO
!!      PARAMETER          ( ONE = 1.0D+0, ZERO = 0.0D+0 )
      double precision ONEh, ONEl
      double precision ZEROh, ZEROl
*     ..
*     .. Local Scalars ..
      INTEGER            J, KNT
!!      DOUBLE PRECISION   BETA, RSAFMN, SAFMIN, XNORM
      double precision BETAh, BETAl
      double precision RSAFMNh, RSAFMNl
      double precision SAFMINh, SAFMINl
      double precision XNORMh, XNORMl
*     ..
*     .. External Functions ..
!!      DOUBLE PRECISION   DLAMCH, DLAPY2, DNRM2
!!      EXTERNAL           DLAMCH, DLAPY2, DNRM2
      EXTERNAL           dDLAMCH, dDLAPY2, dDNRM2
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, SIGN
*     ..
*     .. External Subroutines ..
!!      EXTERNAL           DSCAL
      EXTERNAL           dDSCAL
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
*     ..
*     .. Executable Statements ..
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
*
      IF( N.LE.1 ) THEN
!!         TAU = ZERO
      TAUh = ZEROh
      TAUl = ZEROl
         RETURN
      END IF
*
!!      XNORM = DNRM2( N-1, X, INCX )
      CALL dDNRM2( N-1, Xh, Xl, INCX, XNORMh, XNORMl )
*
!!      IF( XNORM.EQ.ZERO ) THEN
      IF(
!       (XNORM .eq. 0.0)
     + ((XNORMh .eq. 0.0d+0) .and.
     +  (XNORMl .eq. 0.0d+0))
     $   ) THEN
*
*        H  =  I
*
!!         TAU = ZERO
      TAUh = ZEROh
      TAUl = ZEROl
      ELSE
*
*        general case
*
!!         BETA = -SIGN( DLAPY2( ALPHA, XNORM ), ALPHA )
         CALL dDLAPY2( ALPHAh, ALPHAl, XNORMh, XNORMl, tmp1h, tmp1l )
!<<<  tmp2 = abs(tmp1)
      if (tmp1h .lt. 0.0d+0) then
        tmp2h = -tmp1h
        tmp2l = -tmp1l
      elseif (
     +  (tmp1h .eq. 0.0d+0).and.
     +  (tmp1l .lt. 0.0d+0)) then
        tmp2h = tmp1h
        tmp2l = -tmp1l
      else
        tmp2h = tmp1h
        tmp2l = tmp1l
      endif
!>>>  tmp2 = abs(tmp1)
         IF(
!       (ALPHA .ge. 0.0)
     + ((ALPHAh .gt. 0.0d+0) .or.
     +  ((ALPHAh .eq. 0.0d+0) .and.
     +   (ALPHAl .ge. 0.0d+0)))
     $      ) THEN
      tmp3h = tmp2h
      tmp3l = tmp2l
         ELSE
!<<<  tmp3 = -tmp2
      tmp3h = -tmp2h
      tmp3l = -tmp2l
!>>>  tmp3 = -tmp2
         END IF
!<<<  BETA = -tmp3
      BETAh = -tmp3h
      BETAl = -tmp3l
!>>>  BETA = -tmp3
!!         SAFMIN = DLAMCH( 'S' ) / DLAMCH( 'E' )
         CALL dDLAMCH( 'S', tmp1h, tmp1l )
         CALL dDLAMCH( 'E', tmp2h, tmp2l )
!<<<  SAFMIN = tmp1 / tmp2
      T_q1 = tmp1h / tmp2h
      T_ch = tmp2h * T_q1
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp2l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = tmp1h - T_ch
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp2h
      SAFMINh = T_q1 + T_q2
      SAFMINl = T_q2
     +      - (SAFMINh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SAFMIN = tmp1 / tmp2
         KNT = 0
!!         IF( ABS( BETA ).LT.SAFMIN ) THEN
!<<<  tmp1 = abs(BETA)
      if (BETAh .lt. 0.0d+0) then
        tmp1h = -BETAh
        tmp1l = -BETAl
      elseif (
     +  (BETAh .eq. 0.0d+0).and.
     +  (BETAl .lt. 0.0d+0)) then
        tmp1h = BETAh
        tmp1l = -BETAl
      else
        tmp1h = BETAh
        tmp1l = BETAl
      endif
!>>>  tmp1 = abs(BETA)
         IF(
!       (tmp1 .lt. SAFMIN)
     + ((tmp1h .lt. SAFMINh) .or.
     +  ((tmp1h .eq. SAFMINh) .and.
     +   (tmp1l .lt. SAFMINl)))
     $      ) THEN
*
*           XNORM, BETA may be inaccurate; scale X and recompute them
*
!!            RSAFMN = ONE / SAFMIN
!<<<  RSAFMN = ONE / SAFMIN
      T_q1 = ONEh / SAFMINh
      T_ch = SAFMINh * T_q1
      T_u1 = 134217729.0 * SAFMINh
      T_p1 = T_u1
     +         - (T_u1 - SAFMINh)
      T_p2 = SAFMINh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (SAFMINl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = ONEh - T_ch
      T_r1 = T_s1 - ONEh
      T_s2 = (ONEh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + ONEl
      T_q2 = (T_s1 + T_s2)
     +         / SAFMINh
      RSAFMNh = T_q1 + T_q2
      RSAFMNl = T_q2
     +      - (RSAFMNh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  RSAFMN = ONE / SAFMIN
   10       CONTINUE
            KNT = KNT + 1
!!            CALL DSCAL( N-1, RSAFMN, X, INCX )
            CALL dDSCAL( N-1, RSAFMNh, RSAFMNl, Xh, Xl, INCX )
!!            BETA = BETA*RSAFMN
!<<<  tmp1 = BETA * RSAFMN
      T_ch = BETAh * RSAFMNh
      T_u1 = 134217729.0 * BETAh
      T_p1 = T_u1
     +         - (T_u1 - BETAh)
      T_p2 = BETAh - T_p1
      T_u1 = 134217729.0 * RSAFMNh
      T_r1 = T_u1
     +         - (T_u1 - RSAFMNh)
      T_r2 = RSAFMNh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (BETAh*RSAFMNl
     +         +  BETAl*RSAFMNh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = BETA * RSAFMN
      BETAh = tmp1h
      BETAl = tmp1l
!!            ALPHA = ALPHA*RSAFMN
!<<<  tmp1 = ALPHA * RSAFMN
      T_ch = ALPHAh * RSAFMNh
      T_u1 = 134217729.0 * ALPHAh
      T_p1 = T_u1
     +         - (T_u1 - ALPHAh)
      T_p2 = ALPHAh - T_p1
      T_u1 = 134217729.0 * RSAFMNh
      T_r1 = T_u1
     +         - (T_u1 - RSAFMNh)
      T_r2 = RSAFMNh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (ALPHAh*RSAFMNl
     +         +  ALPHAl*RSAFMNh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = ALPHA * RSAFMN
      ALPHAh = tmp1h
      ALPHAl = tmp1l
!!            IF( ABS( BETA ).LT.SAFMIN )
!<<<  tmp1 = abs(BETA)
      if (BETAh .lt. 0.0d+0) then
        tmp1h = -BETAh
        tmp1l = -BETAl
      elseif (
     +  (BETAh .eq. 0.0d+0).and.
     +  (BETAl .lt. 0.0d+0)) then
        tmp1h = BETAh
        tmp1l = -BETAl
      else
        tmp1h = BETAh
        tmp1l = BETAl
      endif
!>>>  tmp1 = abs(BETA)
            IF(
!       (tmp1 .lt. SAFMIN)
     + ((tmp1h .lt. SAFMINh) .or.
     +  ((tmp1h .eq. SAFMINh) .and.
     +   (tmp1l .lt. SAFMINl)))
     $         )
     $         GO TO 10
*
*           New BETA is at most 1, at least SAFMIN
*
!!            XNORM = DNRM2( N-1, X, INCX )
            CALL dDNRM2( N-1, Xh, Xl, INCX, XNORMh, XNORMl )
!!            BETA = -SIGN( DLAPY2( ALPHA, XNORM ), ALPHA )
            CALL dDLAPY2( ALPHAh, ALPHAl, XNORMh, XNORMl,
     $                   tmp1h, tmp1l )
!<<<  tmp2 = abs(tmp1)
      if (tmp1h .lt. 0.0d+0) then
        tmp2h = -tmp1h
        tmp2l = -tmp1l
      elseif (
     +  (tmp1h .eq. 0.0d+0).and.
     +  (tmp1l .lt. 0.0d+0)) then
        tmp2h = tmp1h
        tmp2l = -tmp1l
      else
        tmp2h = tmp1h
        tmp2l = tmp1l
      endif
!>>>  tmp2 = abs(tmp1)
            IF(
!       (ALPHA .ge. 0.0)
     + ((ALPHAh .gt. 0.0d+0) .or.
     +  ((ALPHAh .eq. 0.0d+0) .and.
     +   (ALPHAl .ge. 0.0d+0)))
     $         ) THEN
      tmp3h = tmp2h
      tmp3l = tmp2l
            ELSE
!<<<  tmp3 = -tmp2
      tmp3h = -tmp2h
      tmp3l = -tmp2l
!>>>  tmp3 = -tmp2
            ENDIF
!<<<  BETA = -tmp3
      BETAh = -tmp3h
      BETAl = -tmp3l
!>>>  BETA = -tmp3
         END IF
!!         TAU = ( BETA-ALPHA ) / BETA
!<<<  tmp1 = BETA - ALPHA
      T_p1 = BETAh - ALPHAh
      T_p2 = T_p1 - BETAh
      T_ch = T_p1
      T_cl = (BETAh - (T_p1 - T_p2))
     +         - (ALPHAh + T_p2)
      T_cl = T_cl + BETAl - ALPHAl
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp1 = BETA - ALPHA
!<<<  TAU = tmp1 / BETA
      T_q1 = tmp1h / BETAh
      T_ch = BETAh * T_q1
      T_u1 = 134217729.0 * BETAh
      T_p1 = T_u1
     +         - (T_u1 - BETAh)
      T_p2 = BETAh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (BETAl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = tmp1h - T_ch
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / BETAh
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp1 / BETA
!!         CALL DSCAL( N-1, ONE / ( ALPHA-BETA ), X, INCX )
!<<<  tmp1 = ALPHA - BETA
      T_p1 = ALPHAh - BETAh
      T_p2 = T_p1 - ALPHAh
      T_ch = T_p1
      T_cl = (ALPHAh - (T_p1 - T_p2))
     +         - (BETAh + T_p2)
      T_cl = T_cl + ALPHAl - BETAl
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp1 = ALPHA - BETA
!<<<  tmp2 = ONE / tmp1
      T_q1 = ONEh / tmp1h
      T_ch = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp1l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = ONEh - T_ch
      T_r1 = T_s1 - ONEh
      T_s2 = (ONEh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + ONEl
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = ONE / tmp1
         CALL dDSCAL( N-1, tmp2h, tmp2l, Xh, Xl, INCX )
*
*        If ALPHA is subnormal, it may lose relative accuracy
*
         DO 20 J = 1, KNT
!!            BETA = BETA*SAFMIN
!<<<  tmp1 = BETA * SAFMIN
      T_ch = BETAh * SAFMINh
      T_u1 = 134217729.0 * BETAh
      T_p1 = T_u1
     +         - (T_u1 - BETAh)
      T_p2 = BETAh - T_p1
      T_u1 = 134217729.0 * SAFMINh
      T_r1 = T_u1
     +         - (T_u1 - SAFMINh)
      T_r2 = SAFMINh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (BETAh*SAFMINl
     +         +  BETAl*SAFMINh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = BETA * SAFMIN
      BETAh = tmp1h
      BETAl = tmp1l
 20      CONTINUE
!!         ALPHA = BETA
      ALPHAh = BETAh
      ALPHAl = BETAl
      END IF
*
      RETURN
*
*     End of DLARFG
*
      END
