*> \brief \b DLARTG
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLARTG + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlartg.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlartg.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlartg.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLARTG( F, G, CS, SN, R )
* 
*       .. Scalar Arguments ..
*       DOUBLE PRECISION   CS, F, G, R, SN
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLARTG generate a plane rotation so that
*>
*>    [  CS  SN  ]  .  [ F ]  =  [ R ]   where CS**2 + SN**2 = 1.
*>    [ -SN  CS  ]     [ G ]     [ 0 ]
*>
*> This is a slower, more accurate version of the BLAS1 routine DROTG,
*> with the following other differences:
*>    F and G are unchanged on return.
*>    If G=0, then CS=1 and SN=0.
*>    If F=0 and (G .ne. 0), then CS=0 and SN=1 without doing any
*>       floating point operations (saves work in DBDSQR when
*>       there are zeros on the diagonal).
*>
*> If F exceeds G in magnitude, CS will be positive.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] F
*> \verbatim
*>          F is DOUBLE PRECISION
*>          The first component of vector to be rotated.
*> \endverbatim
*>
*> \param[in] G
*> \verbatim
*>          G is DOUBLE PRECISION
*>          The second component of vector to be rotated.
*> \endverbatim
*>
*> \param[out] CS
*> \verbatim
*>          CS is DOUBLE PRECISION
*>          The cosine of the rotation.
*> \endverbatim
*>
*> \param[out] SN
*> \verbatim
*>          SN is DOUBLE PRECISION
*>          The sine of the rotation.
*> \endverbatim
*>
*> \param[out] R
*> \verbatim
*>          R is DOUBLE PRECISION
*>          The nonzero component of the rotated vector.
*>
*>  This version has a few statements commented out for thread safety
*>  (machine parameters are computed on each entry). 10 feb 03, SJH.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERauxiliary
*
*  =====================================================================
!      SUBROUTINE DLARTG( F, G, CS, SN, R )
      SUBROUTINE DDLARTG( Fh, Fl, Gh, Gl, CSh, CSl, SNh, SNl, Rh, Rl )
*
*  -- LAPACK auxiliary routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
!      DOUBLE PRECISION   CS, F, G, R, SN
      double precision CSh, CSl
      double precision Fh, Fl
      double precision Gh, Gl
      double precision Rh, Rl
      double precision SNh, SNl
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   ZERO
!      PARAMETER          ( ZERO = 0.0D0 )
!      DOUBLE PRECISION   ONE
!      PARAMETER          ( ONE = 1.0D0 )
!      DOUBLE PRECISION   TWO
!      PARAMETER          ( TWO = 2.0D0 )
      double precision ZEROh, ZEROl
      double precision ONEh, ONEl
      double precision TWOh, TWOl

*     ..
*     .. Local Scalars ..
*     LOGICAL            FIRST
      INTEGER            COUNT, I, itmp
      integer j, kk, kn, l1, mn, n, na1, na2, nn
!      DOUBLE PRECISION   EPS, F1, G1, SAFMIN, SAFMN2, SAFMX2, SCALE
      double precision EPSh, EPSl
      double precision F1h, F1l
      double precision G1h, G1l
      double precision SAFMINh, SAFMINl
      double precision SAFMN2h, SAFMN2l
      double precision SAFMX2h, SAFMX2l
      double precision SCALEh, SCALEl
      double precision Bh, Bl
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
      double precision tmp7h, tmp7l
      double precision tmp8h, tmp8l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
*     ..
*     .. External Functions ..
!      DOUBLE PRECISION   DLAMCH
!      EXTERNAL           DLAMCH
*      External subroutine
      EXTERNAL           DDLAMCH
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, INT, LOG, MAX, SQRT
*     ..
*     .. Save statement ..
*     SAVE               FIRST, SAFMX2, SAFMIN, SAFMN2
*     ..
*     .. Data statements ..
*     DATA               FIRST / .TRUE. /
*     ..
*     .. Executable Statements ..
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      TWOh = 2.0d0
      TWOl = 0.0d+0
*
*     IF( FIRST ) THEN
!     SAFMIN = DLAMCH( 'S' )
!     EPS = DLAMCH( 'E' )
      Call DDLAMCH('S',SAFMINh,SAFMINl)
      Call DDLAMCH('E', EPSh, EPSl)
      Call DDLAMCH('B', Bh, Bl)
!     SAFMN2 = DLAMCH( 'B' )**INT( LOG( SAFMIN / EPS ) /
!   $          LOG( DLAMCH( 'B' ) ) / TWO )
!     SAFMX2 = ONE / SAFMN2
!......... ...
      SAFMN2h = Bh**INT( LOG(SAFMINh/EPSh) / LOG(Bh)/TWOh )
      SAFMN2l = 0.0d0
      SAFMX2h = ONEh / SAFMN2h
      SAFMX2l = 0.0d0
!.............
*        FIRST = .FALSE.
*     END IF
      IF(
!         G.EQ.ZERO 
!       (G .eq. ZERO)
     + ((Gh .eq. ZEROh) .and.
     +  (Gl .eq. ZEROl))
     &   ) THEN
!         CS = ONE
      CSh = ONEh
      CSl = ONEl
!         SN = ZERO
      SNh = ZEROh
      SNl = ZEROl
!         R = F
      Rh = Fh
      Rl = Fl
      ELSE IF( 
!              F.EQ.ZERO
!       (F .eq. ZERO)
     + ((Fh .eq. ZEROh) .and.
     +  (Fl .eq. ZEROl))
     &        ) THEN
!         CS = ZERO
      CSh = ZEROh
      CSl = ZEROl
!         SN = ONE
      SNh = ONEh
      SNl = ONEl
!         R = G
      Rh = Gh
      Rl = Gl
      ELSE
!         F1 = F
      F1h = Fh
      F1l = Fl
!         G1 = G
      G1h = Gh
      G1l = Gl
!         SCALE = MAX( ABS( F1 ), ABS( G1 ) )
!<<<  tmp1 = abs(F1)
      if (F1h .lt. 0.0d+0) then
        tmp1h = -F1h
        tmp1l = -F1l
      elseif (
     +  (F1h .eq. 0.0d+0).and.
     +  (F1l .lt. 0.0d+0)) then
        tmp1h = F1h
        tmp1l = -F1l
      else
        tmp1h = F1h
        tmp1l = F1l
      endif
!>>>  tmp1 = abs(F1)
!<<<  tmp2 = abs(G1)
      if (G1h .lt. 0.0d+0) then
        tmp2h = -G1h
        tmp2l = -G1l
      elseif (
     +  (G1h .eq. 0.0d+0).and.
     +  (G1l .lt. 0.0d+0)) then
        tmp2h = G1h
        tmp2l = -G1l
      else
        tmp2h = G1h
        tmp2l = G1l
      endif
!>>>  tmp2 = abs(G1)
          if (
!       (tmp1 .ge. tmp2)
     + ((tmp1h .gt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .ge. tmp2l)))
     &        )then
      SCALEh = tmp1h
      SCALEl = tmp1l
          else
      SCALEh = tmp2h
      SCALEl = tmp2l
          endif
         IF( 
!            SCALE.GE.SAFMX2
!       (SCALE .ge. SAFMX2)
     + ((SCALEh .gt. SAFMX2h) .or.
     +  ((SCALEh .eq. SAFMX2h) .and.
     +   (SCALEl .ge. SAFMX2l)))
     &       ) THEN
            COUNT = 0
   10       CONTINUE
            COUNT = COUNT + 1
!            F1 = F1*SAFMN2
!<<<  F1 = F1 * SAFMN2
      T_ch = F1h * SAFMN2h
      T_u1 = 134217729.0 * F1h
      T_p1 = T_u1
     +         - (T_u1 - F1h)
      T_p2 = F1h - T_p1
      T_u1 = 134217729.0 * SAFMN2h
      T_r1 = T_u1
     +         - (T_u1 - SAFMN2h)
      T_r2 = SAFMN2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (F1h*SAFMN2l
     +         +  F1l*SAFMN2h)
      T_s1 = T_ch + T_cl
      F1l = T_cl - (T_s1 - T_ch)
      F1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  F1 = F1 * SAFMN2
!            G1 = G1*SAFMN2
!<<<  G1 = G1 * SAFMN2
      T_ch = G1h * SAFMN2h
      T_u1 = 134217729.0 * G1h
      T_p1 = T_u1
     +         - (T_u1 - G1h)
      T_p2 = G1h - T_p1
      T_u1 = 134217729.0 * SAFMN2h
      T_r1 = T_u1
     +         - (T_u1 - SAFMN2h)
      T_r2 = SAFMN2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (G1h*SAFMN2l
     +         +  G1l*SAFMN2h)
      T_s1 = T_ch + T_cl
      G1l = T_cl - (T_s1 - T_ch)
      G1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  G1 = G1 * SAFMN2
!            SCALE = MAX( ABS( F1 ), ABS( G1 ) )
!<<<  tmp1 = abs(F1)
      if (F1h .lt. 0.0d+0) then
        tmp1h = -F1h
        tmp1l = -F1l
      elseif (
     +  (F1h .eq. 0.0d+0).and.
     +  (F1l .lt. 0.0d+0)) then
        tmp1h = F1h
        tmp1l = -F1l
      else
        tmp1h = F1h
        tmp1l = F1l
      endif
!>>>  tmp1 = abs(F1)
!<<<  tmp2 = abs(G1)
      if (G1h .lt. 0.0d+0) then
        tmp2h = -G1h
        tmp2l = -G1l
      elseif (
     +  (G1h .eq. 0.0d+0).and.
     +  (G1l .lt. 0.0d+0)) then
        tmp2h = G1h
        tmp2l = -G1l
      else
        tmp2h = G1h
        tmp2l = G1l
      endif
!>>>  tmp2 = abs(G1)
          if (
!       (tmp1 .ge. tmp2)
     + ((tmp1h .gt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .ge. tmp2l)))
     &        )then
      SCALEh = tmp1h
      SCALEl = tmp1l
          else
      SCALEh = tmp2h
      SCALEl = tmp2l
          endif
            IF( 
!               SCALE.GE.SAFMX2
!       (SCALE .ge. SAFMX2)
     + ((SCALEh .gt. SAFMX2h) .or.
     +  ((SCALEh .eq. SAFMX2h) .and.
     +   (SCALEl .ge. SAFMX2l)))
     &          )
     $         GO TO 10
!            R = SQRT( F1**2+G1**2 )
!<<<  tmp1 = F1 * F1
      tmp1h = F1h * F1h
      T_u1 = 134217729.0 * F1h
      T_p1 = T_u1
     +         - (T_u1 - F1h)
      T_p2 = F1h - T_p1
      T_u1 = 134217729.0 * F1h
      T_r1 = T_u1
     +         - (T_u1 - F1h)
      T_r2 = F1h - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (F1h*F1l
     +      + F1l*F1h)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = F1 * F1
!<<<  tmp2 = G1 * G1
      tmp2h = G1h * G1h
      T_u1 = 134217729.0 * G1h
      T_p1 = T_u1
     +         - (T_u1 - G1h)
      T_p2 = G1h - T_p1
      T_u1 = 134217729.0 * G1h
      T_r1 = T_u1
     +         - (T_u1 - G1h)
      T_r2 = G1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (G1h*G1l
     +      + G1l*G1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = G1 * G1
!<<<  tmp3 = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      tmp3h = T_p1
      tmp3l = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp3l = tmp3l + tmp1l + tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = tmp1 + tmp2
!<<<  R = sqrt(tmp3)
      if (tmp3h.eq.0.0D+0.and.tmp3l.eq.0.0D+0) then
        Rh = 0.0d+0
        Rl = 0.0d+0
      endif
      if (tmp3h.lt.0.0d+0.or.
     +   (tmp3h.eq.0.0d+0.and.tmp3l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        Rh = 0.0d+0
        Rl = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp3h)
      T_ax = tmp3h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp3h,tmp3l,T_z1,T_z2)
          T_p1 = tmp3h - T_z1
          T_p2 = T_p1 - tmp3h
          T_t2l = (tmp3h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp3l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          Rh = T_t1h
          Rl = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  R = sqrt(tmp3)
!            CS = F1 / R
!<<<  CS = F1 / R
      T_q1 = F1h / Rh
      CSh = Rh * T_q1
      T_u1 = 134217729.0 * Rh
      T_p1 = T_u1
     +         - (T_u1 - Rh)
      T_p2 = Rh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      CSl = ((T_p1*T_r1
     +       - CSh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      CSl = CSl + (Rl*T_q1)
      T_s1 = CSh + CSl
      CSl = CSl - (T_s1 - CSh)
      CSh = T_s1
      T_s1 = F1h - CSh
      T_r1 = T_s1 - F1h
      T_s2 = (F1h 
     +         - (T_s1 - T_r1))
     +         - (CSh + T_r1)
      T_s2 = T_s2 - CSl
      T_s2 = T_s2 + F1l
      T_q2 = (T_s1 + T_s2)
     +         / Rh
      CSh = T_q1 + T_q2
      CSl = T_q2
     +      - (CSh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  CS = F1 / R
!            SN = G1 / R
!<<<  SN = G1 / R
      T_q1 = G1h / Rh
      SNh = Rh * T_q1
      T_u1 = 134217729.0 * Rh
      T_p1 = T_u1
     +         - (T_u1 - Rh)
      T_p2 = Rh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      SNl = ((T_p1*T_r1
     +       - SNh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      SNl = SNl + (Rl*T_q1)
      T_s1 = SNh + SNl
      SNl = SNl - (T_s1 - SNh)
      SNh = T_s1
      T_s1 = G1h - SNh
      T_r1 = T_s1 - G1h
      T_s2 = (G1h 
     +         - (T_s1 - T_r1))
     +         - (SNh + T_r1)
      T_s2 = T_s2 - SNl
      T_s2 = T_s2 + G1l
      T_q2 = (T_s1 + T_s2)
     +         / Rh
      SNh = T_q1 + T_q2
      SNl = T_q2
     +      - (SNh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SN = G1 / R
            DO 20 I = 1, COUNT
!               R = R*SAFMX2
!<<<  R = R * SAFMX2
      T_ch = Rh * SAFMX2h
      T_u1 = 134217729.0 * Rh
      T_p1 = T_u1
     +         - (T_u1 - Rh)
      T_p2 = Rh - T_p1
      T_u1 = 134217729.0 * SAFMX2h
      T_r1 = T_u1
     +         - (T_u1 - SAFMX2h)
      T_r2 = SAFMX2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Rh*SAFMX2l
     +         +  Rl*SAFMX2h)
      T_s1 = T_ch + T_cl
      Rl = T_cl - (T_s1 - T_ch)
      Rh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  R = R * SAFMX2
   20       CONTINUE
         ELSE IF( 
!                SCALE.LE.SAFMN2
!       (SCALE .le. SAFMN2)
     + ((SCALEh .lt. SAFMN2h) .or.
     +  ((SCALEh .eq. SAFMN2h) .and.
     +   (SCALEl .le. SAFMN2l)))
     & ) THEN
            COUNT = 0
   30       CONTINUE
            COUNT = COUNT + 1
!            F1 = F1*SAFMX2
!<<<  F1 = F1 * SAFMX2
      T_ch = F1h * SAFMX2h
      T_u1 = 134217729.0 * F1h
      T_p1 = T_u1
     +         - (T_u1 - F1h)
      T_p2 = F1h - T_p1
      T_u1 = 134217729.0 * SAFMX2h
      T_r1 = T_u1
     +         - (T_u1 - SAFMX2h)
      T_r2 = SAFMX2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (F1h*SAFMX2l
     +         +  F1l*SAFMX2h)
      T_s1 = T_ch + T_cl
      F1l = T_cl - (T_s1 - T_ch)
      F1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  F1 = F1 * SAFMX2
!            G1 = G1*SAFMX2
!<<<  G1 = G1 * SAFMX2
      T_ch = G1h * SAFMX2h
      T_u1 = 134217729.0 * G1h
      T_p1 = T_u1
     +         - (T_u1 - G1h)
      T_p2 = G1h - T_p1
      T_u1 = 134217729.0 * SAFMX2h
      T_r1 = T_u1
     +         - (T_u1 - SAFMX2h)
      T_r2 = SAFMX2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (G1h*SAFMX2l
     +         +  G1l*SAFMX2h)
      T_s1 = T_ch + T_cl
      G1l = T_cl - (T_s1 - T_ch)
      G1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  G1 = G1 * SAFMX2
!            SCALE = MAX( ABS( F1 ), ABS( G1 ) )
!<<<  tmp1 = abs(F1)
      if (F1h .lt. 0.0d+0) then
        tmp1h = -F1h
        tmp1l = -F1l
      elseif (
     +  (F1h .eq. 0.0d+0).and.
     +  (F1l .lt. 0.0d+0)) then
        tmp1h = F1h
        tmp1l = -F1l
      else
        tmp1h = F1h
        tmp1l = F1l
      endif
!>>>  tmp1 = abs(F1)
!<<<  tmp2 = abs(G1)
      if (G1h .lt. 0.0d+0) then
        tmp2h = -G1h
        tmp2l = -G1l
      elseif (
     +  (G1h .eq. 0.0d+0).and.
     +  (G1l .lt. 0.0d+0)) then
        tmp2h = G1h
        tmp2l = -G1l
      else
        tmp2h = G1h
        tmp2l = G1l
      endif
!>>>  tmp2 = abs(G1)
          if (
!       (tmp1 .ge. tmp2)
     + ((tmp1h .gt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .ge. tmp2l)))
     &        )then
      SCALEh = tmp1h
      SCALEl = tmp1l
          else
      SCALEh = tmp2h
      SCALEl = tmp2l
          endif
            IF( 
!               SCALE.LE.SAFMN2
!       (SCALE .le. SAFMN2)
     + ((SCALEh .lt. SAFMN2h) .or.
     +  ((SCALEh .eq. SAFMN2h) .and.
     +   (SCALEl .le. SAFMN2l)))
     &          )
     $         GO TO 30
!            R = SQRT( F1**2+G1**2 )
!<<<  tmp1 = F1 * F1
      tmp1h = F1h * F1h
      T_u1 = 134217729.0 * F1h
      T_p1 = T_u1
     +         - (T_u1 - F1h)
      T_p2 = F1h - T_p1
      T_u1 = 134217729.0 * F1h
      T_r1 = T_u1
     +         - (T_u1 - F1h)
      T_r2 = F1h - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (F1h*F1l
     +      + F1l*F1h)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = F1 * F1
!<<<  tmp2 = G1 * G1
      tmp2h = G1h * G1h
      T_u1 = 134217729.0 * G1h
      T_p1 = T_u1
     +         - (T_u1 - G1h)
      T_p2 = G1h - T_p1
      T_u1 = 134217729.0 * G1h
      T_r1 = T_u1
     +         - (T_u1 - G1h)
      T_r2 = G1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (G1h*G1l
     +      + G1l*G1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = G1 * G1
!<<<  tmp3 = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      tmp3h = T_p1
      tmp3l = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp3l = tmp3l + tmp1l + tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = tmp1 + tmp2
!<<<  R = sqrt(tmp3)
      if (tmp3h.eq.0.0D+0.and.tmp3l.eq.0.0D+0) then
        Rh = 0.0d+0
        Rl = 0.0d+0
      endif
      if (tmp3h.lt.0.0d+0.or.
     +   (tmp3h.eq.0.0d+0.and.tmp3l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        Rh = 0.0d+0
        Rl = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp3h)
      T_ax = tmp3h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp3h,tmp3l,T_z1,T_z2)
          T_p1 = tmp3h - T_z1
          T_p2 = T_p1 - tmp3h
          T_t2l = (tmp3h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp3l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          Rh = T_t1h
          Rl = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  R = sqrt(tmp3)
!            CS = F1 / R
!<<<  CS = F1 / R
      T_q1 = F1h / Rh
      CSh = Rh * T_q1
      T_u1 = 134217729.0 * Rh
      T_p1 = T_u1
     +         - (T_u1 - Rh)
      T_p2 = Rh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      CSl = ((T_p1*T_r1
     +       - CSh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      CSl = CSl + (Rl*T_q1)
      T_s1 = CSh + CSl
      CSl = CSl - (T_s1 - CSh)
      CSh = T_s1
      T_s1 = F1h - CSh
      T_r1 = T_s1 - F1h
      T_s2 = (F1h 
     +         - (T_s1 - T_r1))
     +         - (CSh + T_r1)
      T_s2 = T_s2 - CSl
      T_s2 = T_s2 + F1l
      T_q2 = (T_s1 + T_s2)
     +         / Rh
      CSh = T_q1 + T_q2
      CSl = T_q2
     +      - (CSh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  CS = F1 / R
!            SN = G1 / R
!<<<  SN = G1 / R
      T_q1 = G1h / Rh
      SNh = Rh * T_q1
      T_u1 = 134217729.0 * Rh
      T_p1 = T_u1
     +         - (T_u1 - Rh)
      T_p2 = Rh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      SNl = ((T_p1*T_r1
     +       - SNh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      SNl = SNl + (Rl*T_q1)
      T_s1 = SNh + SNl
      SNl = SNl - (T_s1 - SNh)
      SNh = T_s1
      T_s1 = G1h - SNh
      T_r1 = T_s1 - G1h
      T_s2 = (G1h 
     +         - (T_s1 - T_r1))
     +         - (SNh + T_r1)
      T_s2 = T_s2 - SNl
      T_s2 = T_s2 + G1l
      T_q2 = (T_s1 + T_s2)
     +         / Rh
      SNh = T_q1 + T_q2
      SNl = T_q2
     +      - (SNh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SN = G1 / R
            DO 40 I = 1, COUNT
!               R = R*SAFMN2
!<<<  R = R * SAFMN2
      T_ch = Rh * SAFMN2h
      T_u1 = 134217729.0 * Rh
      T_p1 = T_u1
     +         - (T_u1 - Rh)
      T_p2 = Rh - T_p1
      T_u1 = 134217729.0 * SAFMN2h
      T_r1 = T_u1
     +         - (T_u1 - SAFMN2h)
      T_r2 = SAFMN2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Rh*SAFMN2l
     +         +  Rl*SAFMN2h)
      T_s1 = T_ch + T_cl
      Rl = T_cl - (T_s1 - T_ch)
      Rh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  R = R * SAFMN2
   40       CONTINUE
         ELSE
!            R = SQRT( F1**2+G1**2 )
!<<<  tmp1 = F1 * F1
      tmp1h = F1h * F1h
      T_u1 = 134217729.0 * F1h
      T_p1 = T_u1
     +         - (T_u1 - F1h)
      T_p2 = F1h - T_p1
      T_u1 = 134217729.0 * F1h
      T_r1 = T_u1
     +         - (T_u1 - F1h)
      T_r2 = F1h - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (F1h*F1l
     +      + F1l*F1h)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = F1 * F1
!<<<  tmp2 = G1 * G1
      tmp2h = G1h * G1h
      T_u1 = 134217729.0 * G1h
      T_p1 = T_u1
     +         - (T_u1 - G1h)
      T_p2 = G1h - T_p1
      T_u1 = 134217729.0 * G1h
      T_r1 = T_u1
     +         - (T_u1 - G1h)
      T_r2 = G1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (G1h*G1l
     +      + G1l*G1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = G1 * G1
!<<<  tmp3 = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      tmp3h = T_p1
      tmp3l = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp3l = tmp3l + tmp1l + tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = tmp1 + tmp2
!<<<  R = sqrt(tmp3)
      if (tmp3h.eq.0.0D+0.and.tmp3l.eq.0.0D+0) then
        Rh = 0.0d+0
        Rl = 0.0d+0
      endif
      if (tmp3h.lt.0.0d+0.or.
     +   (tmp3h.eq.0.0d+0.and.tmp3l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        Rh = 0.0d+0
        Rl = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp3h)
      T_ax = tmp3h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp3h,tmp3l,T_z1,T_z2)
          T_p1 = tmp3h - T_z1
          T_p2 = T_p1 - tmp3h
          T_t2l = (tmp3h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp3l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          Rh = T_t1h
          Rl = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  R = sqrt(tmp3)
!            CS = F1 / R
!<<<  CS = F1 / R
      T_q1 = F1h / Rh
      CSh = Rh * T_q1
      T_u1 = 134217729.0 * Rh
      T_p1 = T_u1
     +         - (T_u1 - Rh)
      T_p2 = Rh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      CSl = ((T_p1*T_r1
     +       - CSh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      CSl = CSl + (Rl*T_q1)
      T_s1 = CSh + CSl
      CSl = CSl - (T_s1 - CSh)
      CSh = T_s1
      T_s1 = F1h - CSh
      T_r1 = T_s1 - F1h
      T_s2 = (F1h 
     +         - (T_s1 - T_r1))
     +         - (CSh + T_r1)
      T_s2 = T_s2 - CSl
      T_s2 = T_s2 + F1l
      T_q2 = (T_s1 + T_s2)
     +         / Rh
      CSh = T_q1 + T_q2
      CSl = T_q2
     +      - (CSh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  CS = F1 / R
!            SN = G1 / R
!<<<  SN = G1 / R
      T_q1 = G1h / Rh
      SNh = Rh * T_q1
      T_u1 = 134217729.0 * Rh
      T_p1 = T_u1
     +         - (T_u1 - Rh)
      T_p2 = Rh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      SNl = ((T_p1*T_r1
     +       - SNh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      SNl = SNl + (Rl*T_q1)
      T_s1 = SNh + SNl
      SNl = SNl - (T_s1 - SNh)
      SNh = T_s1
      T_s1 = G1h - SNh
      T_r1 = T_s1 - G1h
      T_s2 = (G1h 
     +         - (T_s1 - T_r1))
     +         - (SNh + T_r1)
      T_s2 = T_s2 - SNl
      T_s2 = T_s2 + G1l
      T_q2 = (T_s1 + T_s2)
     +         / Rh
      SNh = T_q1 + T_q2
      SNl = T_q2
     +      - (SNh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SN = G1 / R
         END IF
!<<<  tmp1 = abs(F)
      if (Fh .lt. 0.0d+0) then
        tmp1h = -Fh
        tmp1l = -Fl
      elseif (
     +  (Fh .eq. 0.0d+0).and.
     +  (Fl .lt. 0.0d+0)) then
        tmp1h = Fh
        tmp1l = -Fl
      else
        tmp1h = Fh
        tmp1l = Fl
      endif
!>>>  tmp1 = abs(F)
!<<<  tmp2 = abs(G)
      if (Gh .lt. 0.0d+0) then
        tmp2h = -Gh
        tmp2l = -Gl
      elseif (
     +  (Gh .eq. 0.0d+0).and.
     +  (Gl .lt. 0.0d+0)) then
        tmp2h = Gh
        tmp2l = -Gl
      else
        tmp2h = Gh
        tmp2l = Gl
      endif
!>>>  tmp2 = abs(G)
         IF((
!            ABS( F ).GT.ABS( G ) 
!       (tmp1 .ge. tmp2)
     + ((tmp1h .gt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .ge. tmp2l)))
     &       ).AND.( 
!            CS.LT.ZERO
!       (CS .lt. ZERO)
     + ((CSh .lt. ZEROh) .or.
     +  ((CSh .eq. ZEROh) .and.
     +   (CSl .lt. ZEROl)))
     &      )) THEN
!            CS = -CS
!<<<  CS = -CS
      CSh = -CSh
      CSl = -CSl
!>>>  CS = -CS
!            SN = -SN
!<<<  SN = -SN
      SNh = -SNh
      SNl = -SNl
!>>>  SN = -SN
!            R = -R
!<<<  R = -R
      Rh = -Rh
      Rl = -Rl
!>>>  R = -R
         END IF
      END IF
      RETURN
*
*     End of DLARTG
*
      END
