*> \brief \b DLASQ1
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLASQ1 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlasq1.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlasq1.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlasq1.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLASQ1( N, D, E, WORK, INFO )
* 
*       .. Scalar Arguments ..
*       INTEGER            INFO, N
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   D( * ), E( * ), WORK( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLASQ1 computes the singular values of a real N-by-N bidiagonal
*> matrix with diagonal D and off-diagonal E. The singular values
*> are computed to high relative accuracy, in the absence of
*> denormalization, underflow and overflow. The algorithm was first
*> presented in
*>
*> "Accurate singular values and differential qd algorithms" by K. V.
*> Fernando and B. N. Parlett, Numer. Math., Vol-67, No. 2, pp. 191-230,
*> 1994,
*>
*> and the present implementation is described in "An implementation of
*> the dqds Algorithm (Positive Case)", LAPACK Working Note.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>        The number of rows and columns in the matrix. N >= 0.
*> \endverbatim
*>
*> \param[in,out] D
*> \verbatim
*>          D is DOUBLE PRECISION array, dimension (N)
*>        On entry, D contains the diagonal elements of the
*>        bidiagonal matrix whose SVD is desired. On normal exit,
*>        D contains the singular values in decreasing order.
*> \endverbatim
*>
*> \param[in,out] E
*> \verbatim
*>          E is DOUBLE PRECISION array, dimension (N)
*>        On entry, elements E(1:N-1) contain the off-diagonal elements
*>        of the bidiagonal matrix whose SVD is desired.
*>        On exit, E is overwritten.
*> \endverbatim
*>
*> \param[out] WORK
*> \verbatim
*>          WORK is DOUBLE PRECISION array, dimension (4*N)
*> \endverbatim
*>
*> \param[out] INFO
*> \verbatim
*>          INFO is INTEGER
*>        = 0: successful exit
*>        < 0: if INFO = -i, the i-th argument had an illegal value
*>        > 0: the algorithm failed
*>             = 1, a split was marked by a positive value in E
*>             = 2, current block of Z not diagonalized after 100*N
*>                  iterations (in inner while loop)  On exit D and E
*>                  represent a matrix with the same singular values
*>                  which the calling subroutine could use to finish the
*>                  computation, or even feed back into DLASQ1
*>             = 3, termination criterion of outer while loop not met 
*>                  (program created more than N unreduced blocks)
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERcomputational
*
*  =====================================================================
!!      SUBROUTINE DLASQ1( N, D, E, WORK, INFO )
      SUBROUTINE dDLASQ1( N, Dh, Dl, Eh, El, WORKh, WORKl, INFO )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      INTEGER            INFO, N
*     ..
*     .. Array Arguments ..
!!      DOUBLE PRECISION   D( * ), E( * ), WORK( * )
      double precision Dh(*), Dl(*)
      double precision Eh(*), El(*)
      double precision WORKh(*), WORKl(*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!!      DOUBLE PRECISION   ZERO
!!      PARAMETER          ( ZERO = 0.0D0 )
      double precision ZEROh, ZEROl
*     ..
*     .. Local Scalars ..
      INTEGER            I, IINFO
!!      DOUBLE PRECISION   EPS, SCALE, SAFMIN, SIGMN, SIGMX
      double precision EPSh, EPSl
      double precision SCALEh, SCALEl
      double precision SAFMINh, SAFMINl
      double precision SIGMNh, SIGMNl
      double precision SIGMXh, SIGMXl
*     ..
*     .. External Subroutines ..
!!      EXTERNAL           DCOPY, DLAS2, DLASCL, DLASQ2, DLASRT, XERBLA
      EXTERNAL           dDCOPY, dDLAS2, dDLASCL, dDLASQ2, dDLASRT,
     $                   XERBLA, dDLAMCH
*     ..
*     .. External Functions ..
!!      DOUBLE PRECISION   DLAMCH
!!      EXTERNAL           DLAMCH
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX, SQRT
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
      double precision tmp1h, tmp1l
*     ..
*     .. Executable Statements ..
      ZEROh = 0.0D0
      ZEROl = 0.0d+0
*
      INFO = 0
      IF( N.LT.0 ) THEN
         INFO = -2
         CALL XERBLA( 'DLASQ1', -INFO )
         RETURN
      ELSE IF( N.EQ.0 ) THEN
         RETURN
      ELSE IF( N.EQ.1 ) THEN
!!         D( 1 ) = ABS( D( 1 ) )
!<<<  D(1) = abs(D(1))
      if (Dh(1) .lt. 0.0d+0) then
        Dh(1) = -Dh(1)
        Dl(1) = -Dl(1)
      elseif (
     +  (Dh(1) .eq. 0.0d+0).and.
     +  (Dl(1) .lt. 0.0d+0)) then
        Dh(1) = Dh(1)
        Dl(1) = -Dl(1)
      else
        Dh(1) = Dh(1)
        Dl(1) = Dl(1)
      endif
!>>>  D(1) = abs(D(1))
         RETURN
      ELSE IF( N.EQ.2 ) THEN
!!         CALL DLAS2( D( 1 ), E( 1 ), D( 2 ), SIGMN, SIGMX )
         CALL dDLAS2( Dh( 1 ), Dl( 1 ), Eh( 1 ), El( 1 ),
     $                Dh( 2 ), Dl( 2 ), SIGMNh, SIGMNl, SIGMXh, SIGMXl )
!!         D( 1 ) = SIGMX
      Dh(1) = SIGMXh
      Dl(1) = SIGMXl
!!         D( 2 ) = SIGMN
      Dh(2) = SIGMNh
      Dl(2) = SIGMNl
         RETURN
      END IF
*
*     Estimate the largest singular value.
*
!!      SIGMX = ZERO
      SIGMXh = ZEROh
      SIGMXl = ZEROl
      DO 10 I = 1, N - 1
!!         D( I ) = ABS( D( I ) )
!<<<  D(I) = abs(D(I))
      if (Dh(I) .lt. 0.0d+0) then
        Dh(I) = -Dh(I)
        Dl(I) = -Dl(I)
      elseif (
     +  (Dh(I) .eq. 0.0d+0).and.
     +  (Dl(I) .lt. 0.0d+0)) then
        Dh(I) = Dh(I)
        Dl(I) = -Dl(I)
      else
        Dh(I) = Dh(I)
        Dl(I) = Dl(I)
      endif
!>>>  D(I) = abs(D(I))
!!         SIGMX = MAX( SIGMX, ABS( E( I ) ) )
!<<<  tmp1 = abs(E(I))
      if (Eh(I) .lt. 0.0d+0) then
        tmp1h = -Eh(I)
        tmp1l = -El(I)
      elseif (
     +  (Eh(I) .eq. 0.0d+0).and.
     +  (El(I) .lt. 0.0d+0)) then
        tmp1h = Eh(I)
        tmp1l = -El(I)
      else
        tmp1h = Eh(I)
        tmp1l = El(I)
      endif
!>>>  tmp1 = abs(E(I))
         IF(
!       (SIGMX .lt. tmp1)
     + ((SIGMXh .lt. tmp1h) .or.
     +  ((SIGMXh .eq. tmp1h) .and.
     +   (SIGMXl .lt. tmp1l)))
     $     ) THEN
      SIGMXh = tmp1h
      SIGMXl = tmp1l
         END IF
   10 CONTINUE
!!      D( N ) = ABS( D( N ) )
!<<<  D(N) = abs(D(N))
      if (Dh(N) .lt. 0.0d+0) then
        Dh(N) = -Dh(N)
        Dl(N) = -Dl(N)
      elseif (
     +  (Dh(N) .eq. 0.0d+0).and.
     +  (Dl(N) .lt. 0.0d+0)) then
        Dh(N) = Dh(N)
        Dl(N) = -Dl(N)
      else
        Dh(N) = Dh(N)
        Dl(N) = Dl(N)
      endif
!>>>  D(N) = abs(D(N))
*
*     Early return if SIGMX is zero (matrix is already diagonal).
*
!!      IF( SIGMX.EQ.ZERO ) THEN
      IF(
!       (SIGMX .eq. ZERO)
     + ((SIGMXh .eq. ZEROh) .and.
     +  (SIGMXl .eq. ZEROl))
     $  ) THEN
!!         CALL DLASRT( 'D', N, D, IINFO )
         CALL dDLASRT( 'D', N, Dh, Dl, IINFO )
         RETURN
      END IF
*
      DO 20 I = 1, N
!!         SIGMX = MAX( SIGMX, D( I ) )
         IF(
!       (SIGMX .lt. D(I))
     + ((SIGMXh .lt. Dh(I)) .or.
     +  ((SIGMXh .eq. Dh(I)) .and.
     +   (SIGMXl .lt. Dl(I))))
     &     ) THEN
      SIGMXh = Dh(I)
      SIGMXl = Dl(I)
         END IF
   20 CONTINUE
*
*     Copy D and E into WORK (in the Z format) and scale (squaring the
*     input data makes scaling by a power of the radix pointless).
*
!!      EPS = DLAMCH( 'Precision' )
      CALL dDLAMCH( 'Precision', EPSh, EPSl )
!!      SAFMIN = DLAMCH( 'Safe minimum' )
      CALL dDLAMCH( 'Safe minimum', SAFMINh, SAFMINl )
!!      SCALE = SQRT( EPS / SAFMIN )
!<<<  tmp1 = EPS / SAFMIN
      T_q1 = EPSh / SAFMINh
      T_ch = SAFMINh * T_q1
      T_u1 = 134217729.0 * SAFMINh
      T_p1 = T_u1
     +         - (T_u1 - SAFMINh)
      T_p2 = SAFMINh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (SAFMINl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = EPSh - T_ch
      T_r1 = T_s1 - EPSh
      T_s2 = (EPSh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + EPSl
      T_q2 = (T_s1 + T_s2)
     +         / SAFMINh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = EPS / SAFMIN
!<<<  SCALE = sqrt(tmp1)
      if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
        SCALEh = 0.0d+0
        SCALEl = 0.0d+0
      else
      if (tmp1h.lt.0.0d+0.or.
     +   (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        SCALEh = 0.0d+0
        SCALEl = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp1h)
      T_ax = tmp1h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp1h,tmp1l,T_z1,T_z2)
          T_p1 = tmp1h - T_z1
          T_p2 = T_p1 - tmp1h
          T_t2l = (tmp1h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp1l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          SCALEh = T_t1h
          SCALEl = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  SCALE = sqrt(tmp1)
!!      CALL DCOPY( N, D, 1, WORK( 1 ), 2 )
      CALL dDCOPY( N, Dh, Dl, 1, WORKh( 1 ), WORKl( 1 ), 2 )
!!      CALL DCOPY( N-1, E, 1, WORK( 2 ), 2 )
      CALL dDCOPY( N-1, Eh, El, 1, WORKh( 2 ), WORKl( 2 ), 2 )
!!      CALL DLASCL( 'G', 0, 0, SIGMX, SCALE, 2*N-1, 1, WORK, 2*N-1,
!!     $             IINFO )
      CALL dDLASCL( 'G', 0, 0, SIGMXh, SIGMXl, SCALEh, SCALEl, 2*N-1,
     $              1, WORKh, WORKl, 2*N-1,
     $             IINFO )
*         
*     Compute the q's and e's.
*
      DO 30 I = 1, 2*N - 1
!!         WORK( I ) = WORK( I )**2
!<<<  WORK(I) = WORK(I) * WORK(I)
      T_ch = WORKh(I) * WORKh(I)
      T_u1 = 134217729.0 * WORKh(I)
      T_p1 = T_u1
     +         - (T_u1 - WORKh(I))
      T_p2 = WORKh(I) - T_p1
      T_u1 = 134217729.0 * WORKh(I)
      T_r1 = T_u1
     +         - (T_u1 - WORKh(I))
      T_r2 = WORKh(I) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (WORKh(I)*WORKl(I)
     +         +  WORKl(I)*WORKh(I))
      T_s1 = T_ch + T_cl
      WORKl(I) = T_cl - (T_s1 - T_ch)
      WORKh(I) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  WORK(I) = WORK(I) * WORK(I)
   30 CONTINUE
!!      WORK( 2*N ) = ZERO
      WORKh(2*N) = ZEROh
      WORKl(2*N) = ZEROl
*
!!      CALL DLASQ2( N, WORK, INFO )
      CALL dDLASQ2( N, WORKh, WORKl, INFO )
*
      IF( INFO.EQ.0 ) THEN
         DO 40 I = 1, N
!!            D( I ) = SQRT( WORK( I ) )
!<<<  tmp1 = sqrt(WORK(I))
      if (WORKh(I).eq.0.0D+0.and.WORKl(I).eq.0.0D+0) then
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
      else
      if (WORKh(I).lt.0.0d+0.or.
     +   (WORKh(I).eq.0.0d+0.and.WORKl(I).lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(WORKh(I))
      T_ax = WORKh(I) * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           WORKh(I),WORKl(I),T_z1,T_z2)
          T_p1 = WORKh(I) - T_z1
          T_p2 = T_p1 - WORKh(I)
          T_t2l = (WORKh(I)
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + WORKl(I) - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp1h = T_t1h
          tmp1l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp1 = sqrt(WORK(I))
      Dh(I) = tmp1h
      Dl(I) = tmp1l
   40    CONTINUE
!!         CALL DLASCL( 'G', 0, 0, SCALE, SIGMX, N, 1, D, N, IINFO )
         CALL dDLASCL( 'G', 0, 0, SCALEh, SCALEl, SIGMXh, SIGMXl,
     $                 N, 1, Dh, Dl, N, IINFO )
      ELSE IF( INFO.EQ.2 ) THEN
*
*     Maximum number of iterations exceeded.  Move data from WORK
*     into D and E so the calling subroutine can try to finish
*
         DO I = 1, N
!!            D( I ) = SQRT( WORK( 2*I-1 ) )
!<<<  D(I) = sqrt(WORK(2*I-1))
      if (WORKh(2*I-1).eq.0.0D+0.and.WORKl(2*I-1).eq.0.0D+0) then
        Dh(I) = 0.0d+0
        Dl(I) = 0.0d+0
      else
      if (WORKh(2*I-1).lt.0.0d+0.or.
     +   (WORKh(2*I-1).eq.0.0d+0.and.WORKl(2*I-1).lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        Dh(I) = 0.0d+0
        Dl(I) = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(WORKh(2*I-1))
      T_ax = WORKh(2*I-1) * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           WORKh(2*I-1),WORKl(2*I-1),T_z1,T_z2)
          T_p1 = WORKh(2*I-1) - T_z1
          T_p2 = T_p1 - WORKh(2*I-1)
          T_t2l = (WORKh(2*I-1)
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + WORKl(2*I-1) - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          Dh(I) = T_t1h
          Dl(I) = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  D(I) = sqrt(WORK(2*I-1))
!!            E( I ) = SQRT( WORK( 2*I ) )
!<<<  E(I) = sqrt(WORK(2*I))
      if (WORKh(2*I).eq.0.0D+0.and.WORKl(2*I).eq.0.0D+0) then
        Eh(I) = 0.0d+0
        El(I) = 0.0d+0
      else
      if (WORKh(2*I).lt.0.0d+0.or.
     +   (WORKh(2*I).eq.0.0d+0.and.WORKl(2*I).lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        Eh(I) = 0.0d+0
        El(I) = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(WORKh(2*I))
      T_ax = WORKh(2*I) * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           WORKh(2*I),WORKl(2*I),T_z1,T_z2)
          T_p1 = WORKh(2*I) - T_z1
          T_p2 = T_p1 - WORKh(2*I)
          T_t2l = (WORKh(2*I)
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + WORKl(2*I) - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          Eh(I) = T_t1h
          El(I) = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  E(I) = sqrt(WORK(2*I))
         END DO
!!         CALL DLASCL( 'G', 0, 0, SCALE, SIGMX, N, 1, D, N, IINFO )
         CALL dDLASCL( 'G', 0, 0, SCALEh, SCALEl, SIGMXh, SIGMXl,
     $                 N, 1, Dh, Dl, N, IINFO )
!!         CALL DLASCL( 'G', 0, 0, SCALE, SIGMX, N, 1, E, N, IINFO )
         CALL dDLASCL( 'G', 0, 0, SCALEh, SCALEl, SIGMXh, SIGMXl,
     $                 N, 1, Eh, El, N, IINFO )
      END IF
*
      RETURN
*
*     End of DLASQ1
*
      END
