* -*- fortran -*-
*> \brief \b DLASQ2
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLASQ2 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlasq2.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlasq2.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlasq2.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLASQ2( N, Z, INFO )
* 
*       .. Scalar Arguments ..
*       INTEGER            INFO, N
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   Z( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLASQ2 computes all the eigenvalues of the symmetric positive 
*> definite tridiagonal matrix associated with the qd array Z to high
*> relative accuracy are computed to high relative accuracy, in the
*> absence of denormalization, underflow and overflow.
*>
*> To see the relation of Z to the tridiagonal matrix, let L be a
*> unit lower bidiagonal matrix with subdiagonals Z(2,4,6,,..) and
*> let U be an upper bidiagonal matrix with 1's above and diagonal
*> Z(1,3,5,,..). The tridiagonal is L*U or, if you prefer, the
*> symmetric tridiagonal to which it is similar.
*>
*> Note : DLASQ2 defines a logical variable, IEEE, which is true
*> on machines which follow ieee-754 floating-point standard in their
*> handling of infinities and NaNs, and false otherwise. This variable
*> is passed to DLASQ3.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>        The number of rows and columns in the matrix. N >= 0.
*> \endverbatim
*>
*> \param[in,out] Z
*> \verbatim
*>          Z is DOUBLE PRECISION array, dimension ( 4*N )
*>        On entry Z holds the qd array. On exit, entries 1 to N hold
*>        the eigenvalues in decreasing order, Z( 2*N+1 ) holds the
*>        trace, and Z( 2*N+2 ) holds the sum of the eigenvalues. If
*>        N > 2, then Z( 2*N+3 ) holds the iteration count, Z( 2*N+4 )
*>        holds NDIVS/NIN^2, and Z( 2*N+5 ) holds the percentage of
*>        shifts that failed.
*> \endverbatim
*>
*> \param[out] INFO
*> \verbatim
*>          INFO is INTEGER
*>        = 0: successful exit
*>        < 0: if the i-th argument is a scalar and had an illegal
*>             value, then INFO = -i, if the i-th argument is an
*>             array and the j-entry had an illegal value, then
*>             INFO = -(i*100+j)
*>        > 0: the algorithm failed
*>              = 1, a split was marked by a positive value in E
*>              = 2, current block of Z not diagonalized after 100*N
*>                   iterations (in inner while loop).  On exit Z holds
*>                   a qd array with the same eigenvalues as the given Z.
*>              = 3, termination criterion of outer while loop not met 
*>                   (program created more than N unreduced blocks)
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERcomputational
*
*> \par Further Details:
*  =====================
*>
*> \verbatim
*>
*>  Local Variables: I0:N0 defines a current unreduced segment of Z.
*>  The shifts are accumulated in SIGMA. Iteration count is in ITER.
*>  Ping-pong is controlled by PP (alternates between 0 and 1).
*> \endverbatim
*>
*  =====================================================================
!      SUBROUTINE DLASQ2( N, Z, INFO )
      SUBROUTINE DDLASQ2( N, Zh, Zl, INFO )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      INTEGER            INFO, N
*     ..
*     .. Array Arguments ..
!      DOUBLE PRECISION   Z( * )
      DOUBLE PRECISION   Zh( * ), Zl( * )
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   CBIAS
!      PARAMETER          ( CBIAS = 1.50D0 )
!      DOUBLE PRECISION   ZERO, HALF, ONE, TWO, FOUR, HUNDRD
!      PARAMETER          ( ZERO = 0.0D0, HALF = 0.5D0, ONE = 1.0D0,
!     $                     TWO = 2.0D0, FOUR = 4.0D0, HUNDRD = 100.0D0 )
      double precision cbiash, cbiasl
      double precision zeroh, zerol
      double precision halfh, halfl
      double precision oneh, onel
      double precision twoh, twol
      double precision fourh, fourl
      double precision HUNDRDh, HUNDRDl
*     ..
*     .. Local Scalars ..
      LOGICAL            IEEE
      INTEGER            I0, I1, I4, IINFO, IPN4, ITER, IWHILA, IWHILB,
     $                   K, KMIN, N0, N1, NBIG, NDIV, NFAIL, PP, SPLT, 
     $                   TTYPE
!      DOUBLE PRECISION   D, DEE, DEEMIN, DESIG, DMIN, DMIN1, DMIN2, DN,
!     $                   DN1, DN2, E, EMAX, EMIN, EPS, G, OLDEMN, QMAX,
!     $                   QMIN, S, SAFMIN, SIGMA, T, TAU, TEMP, TOL,
!     $                   TOL2, TRACE, ZMAX, TEMPE, TEMPQ
      double precision Dh, Dl
      double precision DEEh, DEEl
      double precision DEEMINh, DEEMINl
      double precision DESIGh, DESIGl
      double precision DMINh, DMINl
      double precision DMIN1h, DMIN1l
      double precision DMIN2h, DMIN2l
      double precision DNh, DNl
      double precision DN1h, DN1l
      double precision DN2h, DN2l
      double precision Eh, El
      double precision EMAXh, EMAXl
      double precision EMINh, EMINl
      double precision EPSh, EPSl
      double precision Gh, Gl
      double precision OLDEMNh, OLDEMNl
      double precision QMAXh, QMAXl
      double precision QMINh, QMINl
      double precision Sh, Sl
      double precision SAFMINh, SAFMINl
      double precision SIGMAh, SIGMAl
      double precision Th, Tl
      double precision TAUh, TAUl
      double precision TEMPh, TEMPl
      double precision TOLh, TOLl
      double precision TOL2h, TOL2l
      double precision TRACEh, TRACEl
      double precision ZMAXh, ZMAXl
      double precision TEMPEh, TEMPEl
      double precision TEMPQh, TEMPQl
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
!     tmp vars
      double precision t_ch, t_cl
      double precision t_p1, t_p2
      double precision t_q1, t_q2
      double precision t_r1, t_r2
      double precision t_s1, t_s2
      double precision t_t1, t_t2
      double precision t_u1, t_u2
      double precision t_x1, t_x2
      double precision t_y1, t_y2
      double precision t_z1, t_z2
      double precision t_x, t_ax
      double precision t_t1h, t_t1l
      double precision t_t2h, t_t2l
*     ..
*     .. External Subroutines ..
!      EXTERNAL           DLASQ3, DLASRT, XERBLA
      EXTERNAL           DDLASQ3, DDLASRT, XERBLA
*     ..
*     .. External Functions ..
      INTEGER            ILAENV
!      DOUBLE PRECISION   DLAMCH
!      EXTERNAL           DLAMCH, ILAENV
      EXTERNAL           ILAENV
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, DBLE, MAX, MIN, SQRT
*     ..
*     .. Executable Statements ..
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      CBIASh = 1.50D0
      CBIASl = 0.0d+0
      HALFh = 0.5D0
      HALFl = 0.0d+0
      TWOh = 2.0D0
      TWOl = 0.0d+0
      FOURh = 4.0D0
      FOURl = 0.0d+0
      HUNDRDh = 100.0D0
      HUNDRDl = 0.0d+0
*      
*     Test the input arguments.
*     (in case DLASQ2 is not called by DLASQ1)
*
      INFO = 0
!      EPS = DLAMCH( 'Precision' )
!      SAFMIN = DLAMCH( 'Safe minimum' )
      CALL DDLAMCH( 'Precision', EPSh, EPSl )
      CALL DDLAMCH( 'Safe minimum', SAFMINh, SAFMINl )
!      TOL = EPS*HUNDRD
!      TOL2 = TOL**2
!<<<  TOL = EPS * HUNDRD
      t_ch = EPSh * HUNDRDh
      t_u1 = 134217729.0 * EPSh
      t_p1 = t_u1
     +         - (t_u1 - EPSh)
      t_p2 = EPSh - t_p1
      t_u1 = 134217729.0 * HUNDRDh
      t_r1 = t_u1
     +         - (t_u1 - HUNDRDh)
      t_r2 = HUNDRDh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (EPSh*HUNDRDl
     +         +  EPSl*HUNDRDh)
      t_s1 = t_ch + t_cl
      TOLl = t_cl - (t_s1 - t_ch)
      TOLh = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TOL = EPS * HUNDRD
!<<<  TOL2 = TOL * TOL
      t_ch = TOLh * TOLh
      t_u1 = 134217729.0 * TOLh
      t_p1 = t_u1
     +         - (t_u1 - TOLh)
      t_p2 = TOLh - t_p1
      t_u1 = 134217729.0 * TOLh
      t_r1 = t_u1
     +         - (t_u1 - TOLh)
      t_r2 = TOLh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (TOLh*TOLl
     +         +  TOLl*TOLh)
      t_s1 = t_ch + t_cl
      TOL2l = t_cl - (t_s1 - t_ch)
      TOL2h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TOL2 = TOL * TOL
*
      IF( N.LT.0 ) THEN
         INFO = -1
         CALL XERBLA( 'DLASQ2', 1 )
         RETURN
      ELSE IF( N.EQ.0 ) THEN
         RETURN
      ELSE IF( N.EQ.1 ) THEN
*
*        1-by-1 case.
*
!         IF( Z( 1 ).LT.ZERO ) THEN
         IF(
!          (Z(1) .lt. ZERO)
     +    ((Zh(1) .lt. ZEROh) .or.
     +     ((Zh(1) .eq. ZEROh) .and.
     +      (Zl(1) .lt. ZEROl)))
     &   ) THEN
            INFO = -201
            CALL XERBLA( 'DLASQ2', 2 )
         END IF
         RETURN
      ELSE IF( N.EQ.2 ) THEN
*
*        2-by-2 case.
*
!         IF( Z( 2 ).LT.ZERO .OR. Z( 3 ).LT.ZERO ) THEN
         IF((
!          (Z(2) .lt. ZERO)
     +    ((Zh(2) .lt. ZEROh) .or.
     +     ((Zh(2) .eq. ZEROh) .and.
     +      (Zl(2) .lt. ZEROl)))
     &     ).OR.(
!          (Z(3) .lt. ZERO)
     +    ((Zh(3) .lt. ZEROh) .or.
     +     ((Zh(3) .eq. ZEROh) .and.
     +      (Zl(3) .lt. ZEROl)))
     &   )) THEN
            INFO = -2
            CALL XERBLA( 'DLASQ2', 2 )
            RETURN
!         ELSE IF( Z( 3 ).GT.Z( 1 ) ) THEN
         ELSE IF(
!          (Z(3) .gt. Z(1))
     +    ((Zh(3) .gt. Zh(1)) .or.
     +     ((Zh(3) .eq. Zh(1)) .and.
     +      (Zl(3) .gt. Zl(1))))
     &   ) THEN
!            D = Z( 3 )
!            Z( 3 ) = Z( 1 )
!            Z( 1 ) = D
            Dh = Zh(3)
            Dl = Zl(3)
            Zh(3) = Zh(1)
            Zl(3) = Zl(1)
            Zh(1) = Dh
            Zl(1) = Dl
         END IF
!         Z( 5 ) = Z( 1 ) + Z( 2 ) + Z( 3 )
!<<<     Z(5) = Z(1) + Z(2)
         t_p1 = Zh(1) + Zh(2)
         t_p2 = t_p1 - Zh(1)
         t_q1 = t_p1
         t_q2 = (Zh(1) - (t_p1 - t_p2))
     +            + (Zh(2) - t_p2)
         t_q2 = t_q2 + Zl(1) + Zl(2)
         Zh(5) = t_q1 + t_q2
         Zl(5) = t_q2 - (Zh(5) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>     Z(5) = Z(1) + Z(2)
!<<<     Z(5) = Z(5) + Z(3)
         t_p1 = Zh(5) + Zh(3)
         t_p2 = t_p1 - Zh(5)
         t_q1 = t_p1
         t_q2 = (Zh(5) - (t_p1 - t_p2))
     +            + (Zh(3) - t_p2)
         t_q2 = t_q2 + Zl(5) + Zl(3)
         Zh(5) = t_q1 + t_q2
         Zl(5) = t_q2 - (Zh(5) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>     Z(5) = Z(5) + Z(3)
!         IF( Z( 2 ).GT.Z( 3 )*TOL2 ) THEN
!<<<     tmp1 = Z(3) * TOL2
         t_ch = Zh(3) * TOL2h
         t_u1 = 134217729.0 * Zh(3)
         t_p1 = t_u1
     +            - (t_u1 - Zh(3))
         t_p2 = Zh(3) - t_p1
         t_u1 = 134217729.0 * TOL2h
         t_r1 = t_u1
     +            - (t_u1 - TOL2h)
         t_r2 = TOL2h - t_r1
         t_cl = ((t_p1*t_r1
     +            - t_ch)
     +           + t_p1 * t_r2
     +           + t_p2 * t_r1)
     +           + t_p2 * t_r2
         t_cl = t_cl
     +            + (Zh(3)*TOL2l
     +            +  Zl(3)*TOL2h)
         t_s1 = t_ch + t_cl
         tmp1l = t_cl - (t_s1 - t_ch)
         tmp1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>     tmp1 = Z(3) * TOL2
         IF(
!          (Z(2) .gt. tmp1)
     +    ((Zh(2) .gt. tmp1h) .or.
     +     ((Zh(2) .eq. tmp1h) .and.
     +      (Zl(2) .gt. tmp1l)))
     &   ) THEN
!            T = HALF*( ( Z( 1 )-Z( 3 ) )+Z( 2 ) ) 
!<<<        tmp1 = Z(1) - Z(3)
            t_p1 = Zh(1) - Zh(3)
            t_p2 = t_p1 - Zh(1)
            t_ch = t_p1
            t_cl = (Zh(1) - (t_p1 - t_p2))
     +               - (Zh(3) + t_p2)
            t_cl = t_cl + Zl(1) - Zl(3)
            t_s1 = t_ch + t_cl
            tmp1l = t_cl - (t_s1 - t_ch)
            tmp1h = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>        tmp1 = Z(1) - Z(3)
!<<<        tmp1 = tmp1 + Z(2)
            t_p1 = tmp1h + Zh(2)
            t_p2 = t_p1 - tmp1h
            t_q1 = t_p1
            t_q2 = (tmp1h - (t_p1 - t_p2))
     +               + (Zh(2) - t_p2)
            t_q2 = t_q2 + tmp1l + Zl(2)
            tmp1h = t_q1 + t_q2
            tmp1l = t_q2 - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>        tmp1 = tmp1 + Z(2)
!<<<        T = HALF * tmp1
            t_ch = HALFh * tmp1h
            t_u1 = 134217729.0 * HALFh
            t_p1 = t_u1
     +               - (t_u1 - HALFh)
            t_p2 = HALFh - t_p1
            t_u1 = 134217729.0 * tmp1h
            t_r1 = t_u1
     +               - (t_u1 - tmp1h)
            t_r2 = tmp1h - t_r1
            t_cl = ((t_p1*t_r1
     +               - t_ch)
     +              + t_p1 * t_r2
     +              + t_p2 * t_r1)
     +              + t_p2 * t_r2
            t_cl = t_cl
     +               + (HALFh*tmp1l
     +               +  HALFl*tmp1h)
            t_s1 = t_ch + t_cl
            Tl = t_cl - (t_s1 - t_ch)
            Th = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>        T = HALF * tmp1
!            S = Z( 3 )*( Z( 2 ) / T )
!<<<        S = Z(2) / T
            t_q1 = Zh(2) / Th
            t_ch = Th * t_q1
            t_u1 = 134217729.0 * Th
            t_p1 = t_u1
     +               - (t_u1 - Th)
            t_p2 = Th - t_p1
            t_u1 = 134217729.0 * t_q1
            t_r1 = t_u1
     +               - (t_u1 - t_q1)
            t_r2 = t_q1 - t_r1
            t_cl = ((t_p1*t_r1
     +               - t_ch)
     +               + t_p1*t_r2
     +               + t_p2*t_r1)
     +               + t_p2*t_r2
            t_cl = t_cl
     +               + (Tl * t_q1)
            t_s1 = t_ch + t_cl
            t_cl = t_cl
     +               - (t_s1 - t_ch)
            t_ch = t_s1
            t_s1 = Zh(2) - t_ch
            t_r1 = t_s1 - Zh(2)
            t_s2 = (Zh(2)
     +               - (t_s1 - t_r1))
     +               - (t_ch + t_r1)
            t_s2 = t_s2 - t_cl
            t_s2 = t_s2 + Zl(2)
            t_q2 = (t_s1 + t_s2)
     +               / Th
            Sh = t_q1 + t_q2
            Sl = t_q2
     +            - (Sh - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>        S = Z(2) / T
!<<<        S = Z(3) * S
            t_ch = Zh(3) * Sh
            t_u1 = 134217729.0 * Zh(3)
            t_p1 = t_u1
     +               - (t_u1 - Zh(3))
            t_p2 = Zh(3) - t_p1
            t_u1 = 134217729.0 * Sh
            t_r1 = t_u1
     +               - (t_u1 - Sh)
            t_r2 = Sh - t_r1
            t_cl = ((t_p1*t_r1
     +               - t_ch)
     +              + t_p1 * t_r2
     +              + t_p2 * t_r1)
     +              + t_p2 * t_r2
            t_cl = t_cl
     +               + (Zh(3)*Sl
     +               +  Zl(3)*Sh)
            t_s1 = t_ch + t_cl
            Sl = t_cl - (t_s1 - t_ch)
            Sh = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>        S = Z(3) * S
!            IF( S.LE.T ) THEN
            IF(
!             (S .le. T)
     +       ((Sh .lt. Th) .or.
     +        ((Sh .eq. Th) .and.
     +         (Sl .le. Tl)))
     &      ) THEN
!               S = Z( 3 )*( Z( 2 ) / ( T*( ONE+SQRT( ONE+S / T ) ) ) )
!<<<           tmp1 = S / T
               t_q1 = Sh / Th
               t_ch = Th * t_q1
               t_u1 = 134217729.0 * Th
               t_p1 = t_u1
     +                  - (t_u1 - Th)
               t_p2 = Th - t_p1
               t_u1 = 134217729.0 * t_q1
               t_r1 = t_u1
     +                  - (t_u1 - t_q1)
               t_r2 = t_q1 - t_r1
               t_cl = ((t_p1*t_r1
     +                  - t_ch)
     +                  + t_p1*t_r2
     +                  + t_p2*t_r1)
     +                  + t_p2*t_r2
               t_cl = t_cl
     +                  + (Tl * t_q1)
               t_s1 = t_ch + t_cl
               t_cl = t_cl
     +                  - (t_s1 - t_ch)
               t_ch = t_s1
               t_s1 = Sh - t_ch
               t_r1 = t_s1 - Sh
               t_s2 = (Sh
     +                  - (t_s1 - t_r1))
     +                  - (t_ch + t_r1)
               t_s2 = t_s2 - t_cl
               t_s2 = t_s2 + Sl
               t_q2 = (t_s1 + t_s2)
     +                  / Th
               tmp1h = t_q1 + t_q2
               tmp1l = t_q2
     +               - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>           tmp1 = S / T
!<<<           tmp1 = one + tmp1
               t_p1 = oneh + tmp1h
               t_p2 = t_p1 - oneh
               t_q1 = t_p1
               t_q2 = (oneh - (t_p1 - t_p2))
     +                  + (tmp1h - t_p2)
               t_q2 = t_q2 + onel + tmp1l
               tmp1h = t_q1 + t_q2
               tmp1l = t_q2 - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>           tmp1 = one + tmp1
!<<<           tmp1 = sqrt(tmp1)
               if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
                 tmp1h = 0.0d+0
                 tmp1l = 0.0d+0
               else
               if (tmp1h.lt.0.0d+0.or.
     +            (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
                 write(6,'(a)')
     +              'SQRT CALLED WITH NEGATIVE ARGUMENT'
                 tmp1h = 0.0d+0
                 tmp1l = 0.0d+0
                 return
               endif
               t_x = 1.0D+0/sqrt(tmp1h)
               t_ax = tmp1h * t_x
!                  call dd_mul(t_y1,t_y2,t_x,
!                              0.0,0.5,0.0)
                   t_y1 = t_x * 0.5d+0
                   t_u1 = 134217729.0 * t_x
                   t_p1 = t_u1
     +                      - (t_u1 - t_x)
                   t_p2 = t_x - t_p1
                   t_u1 = 134217729.0 * 0.5d+0
                   t_r1 = t_u1
     +                      - (t_u1 - 0.5d+0)
                   t_r2 = 0.5d+0 - t_r1
                   t_y2 = ((t_p1*t_r1
     +                      - t_y1)
     +                      + t_p1*t_r2
     +                      + t_p2 * t_r1)
     +                      + t_p2 * t_r2
                   t_s1 = t_y1 + t_y2
                   t_y2 = t_y2
     +                      - (t_s1 - t_y1)
                   t_y1 = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                  call dd_mul(t_z1,t_z2,t_ax,
!                              0.0,t_ax,0.0)
                   t_z1 = t_ax * t_ax
                   t_u1 = 134217729.0 * t_ax
                   t_p1 = t_u1
     +                      - (t_u1 - t_ax)
                   t_p2 = t_ax - t_p1
                   t_u1 = 134217729.0 * t_ax
                   t_r1 = t_u1
     +                      - (t_u1 - t_ax)
                   t_r2 = t_ax - t_r1
                   t_z2 = ((t_p1 * t_r1
     +                      - t_z1)
     +                      + t_p1 * t_r2
     +                      + t_p2 * t_r1)
     +                      + t_p2*t_r2
                   t_s1 = t_z1 + t_z2
                   t_z2 = t_z2
     +                      - (t_s1 - t_z1)
                   t_z1 = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                  call dd_sub(t_t2h,t_t2l,
!                           tmp1h,tmp1l,t_z1,t_z2)
                   t_p1 = tmp1h - t_z1
                   t_p2 = t_p1 - tmp1h
                   t_t2l = (tmp1h
     +                      -(t_p1 - t_p2))
     +                      -(t_z1 + t_p2)
                   t_t2h = t_p1
                   t_t2l = t_t2l
     +                      + tmp1l - t_z2
                   t_s1 = t_t2h + t_t2l
                   t_t2l = t_t2l
     +                      - (t_s1 - t_t2h)
                   t_t2h = t_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!                  call dd_mul(t_t1h,t_t1l,
!                              t_t2h,t_t2l,
!                              t_y1,t_y2)
                   t_t1h = t_t2h * t_y1
                   t_u1 = 134217729.0 * t_t2h
                   t_p1 = t_u1
     +                      - (t_u1 - t_t2h)
                   t_p2 = t_t2h - t_p1
                   t_u1 = 134217729.0 * t_y1
                   t_r1 = t_u1
     +                      - (t_u1 - t_y1)
                   t_r2 = t_y1 - t_r1
                   t_t1l = ((t_p1*t_r1
     +                      - t_t1h)
     +                      + t_p1*t_r2
     +                      + t_p2*t_r1)
     +                      + t_p2*t_r2
                   t_t1l = t_t1l
     +                      + (t_t2h * t_y2
     +                      +  t_t2l * t_y1)
                   t_s1 = t_t1h + t_t1l
                   t_t1l = t_t1l - (t_s1
     +                       - t_t1h)
                   t_t1h = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                  call dd_selfadd(t_t1h,t_t1l,
!                              t_ax,0.0)
                   t_p1 = t_t1h + t_ax
                   t_p2 = t_p1 - t_t1h
                   t_q2 = (t_t1h
     +                      - (t_p1 - t_p2))
     +                      + (t_ax - t_p2)
                   t_q1 = t_p1
                   t_q2 = t_q2 + t_t1l
                   t_t1h = t_q1 + t_q2
                   t_t1l = t_q2
     +                       - (t_t1h - t_q1)
                   tmp1h = t_t1h
                   tmp1l = t_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
               endif
!>>>           tmp1 = sqrt(tmp1)
!<<<           tmp1 = one + tmp1
               t_p1 = oneh + tmp1h
               t_p2 = t_p1 - oneh
               t_q1 = t_p1
               t_q2 = (oneh - (t_p1 - t_p2))
     +                  + (tmp1h - t_p2)
               t_q2 = t_q2 + onel + tmp1l
               tmp1h = t_q1 + t_q2
               tmp1l = t_q2 - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>           tmp1 = one + tmp1
!<<<           tmp1 = t * tmp1
               t_ch = th * tmp1h
               t_u1 = 134217729.0 * th
               t_p1 = t_u1
     +                  - (t_u1 - th)
               t_p2 = th - t_p1
               t_u1 = 134217729.0 * tmp1h
               t_r1 = t_u1
     +                  - (t_u1 - tmp1h)
               t_r2 = tmp1h - t_r1
               t_cl = ((t_p1*t_r1
     +                  - t_ch)
     +                 + t_p1 * t_r2
     +                 + t_p2 * t_r1)
     +                 + t_p2 * t_r2
               t_cl = t_cl
     +                  + (th*tmp1l
     +                  +  tl*tmp1h)
               t_s1 = t_ch + t_cl
               tmp1l = t_cl - (t_s1 - t_ch)
               tmp1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>           tmp1 = t * tmp1
!<<<           tmp1 = Z(2) / tmp1
               t_q1 = Zh(2) / tmp1h
               t_ch = tmp1h * t_q1
               t_u1 = 134217729.0 * tmp1h
               t_p1 = t_u1
     +                  - (t_u1 - tmp1h)
               t_p2 = tmp1h - t_p1
               t_u1 = 134217729.0 * t_q1
               t_r1 = t_u1
     +                  - (t_u1 - t_q1)
               t_r2 = t_q1 - t_r1
               t_cl = ((t_p1*t_r1
     +                  - t_ch)
     +                  + t_p1*t_r2
     +                  + t_p2*t_r1)
     +                  + t_p2*t_r2
               t_cl = t_cl
     +                  + (tmp1l * t_q1)
               t_s1 = t_ch + t_cl
               t_cl = t_cl
     +                  - (t_s1 - t_ch)
               t_ch = t_s1
               t_s1 = Zh(2) - t_ch
               t_r1 = t_s1 - Zh(2)
               t_s2 = (Zh(2)
     +                  - (t_s1 - t_r1))
     +                  - (t_ch + t_r1)
               t_s2 = t_s2 - t_cl
               t_s2 = t_s2 + Zl(2)
               t_q2 = (t_s1 + t_s2)
     +                  / tmp1h
               tmp1h = t_q1 + t_q2
               tmp1l = t_q2
     +               - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>           tmp1 = Z(2) / tmp1
!<<<           S = Z(3) * tmp1
               t_ch = Zh(3) * tmp1h
               t_u1 = 134217729.0 * Zh(3)
               t_p1 = t_u1
     +                  - (t_u1 - Zh(3))
               t_p2 = Zh(3) - t_p1
               t_u1 = 134217729.0 * tmp1h
               t_r1 = t_u1
     +                  - (t_u1 - tmp1h)
               t_r2 = tmp1h - t_r1
               t_cl = ((t_p1*t_r1
     +                  - t_ch)
     +                 + t_p1 * t_r2
     +                 + t_p2 * t_r1)
     +                 + t_p2 * t_r2
               t_cl = t_cl
     +                  + (Zh(3)*tmp1l
     +                  +  Zl(3)*tmp1h)
               t_s1 = t_ch + t_cl
               Sl = t_cl - (t_s1 - t_ch)
               Sh = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>           S = Z(3) * tmp1
            ELSE
!               S = Z( 3 )*( Z( 2 ) / ( T+SQRT( T )*SQRT( T+S ) ) )
!<<<           tmp1 = sqrt(T)
               if (Th.eq.0.0D+0.and.Tl.eq.0.0D+0) then
                 tmp1h = 0.0d+0
                 tmp1l = 0.0d+0
               else
               if (Th.lt.0.0d+0.or.
     +            (Th.eq.0.0d+0.and.Tl.lt.0.0d+0)) then
                 write(6,'(a)')
     +              'SQRT CALLED WITH NEGATIVE ARGUMENT'
                 tmp1h = 0.0d+0
                 tmp1l = 0.0d+0
                 return
               endif
               t_x = 1.0D+0/sqrt(Th)
               t_ax = Th * t_x
!                  call dd_mul(t_y1,t_y2,t_x,
!                              0.0,0.5,0.0)
                   t_y1 = t_x * 0.5d+0
                   t_u1 = 134217729.0 * t_x
                   t_p1 = t_u1
     +                      - (t_u1 - t_x)
                   t_p2 = t_x - t_p1
                   t_u1 = 134217729.0 * 0.5d+0
                   t_r1 = t_u1
     +                      - (t_u1 - 0.5d+0)
                   t_r2 = 0.5d+0 - t_r1
                   t_y2 = ((t_p1*t_r1
     +                      - t_y1)
     +                      + t_p1*t_r2
     +                      + t_p2 * t_r1)
     +                      + t_p2 * t_r2
                   t_s1 = t_y1 + t_y2
                   t_y2 = t_y2
     +                      - (t_s1 - t_y1)
                   t_y1 = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                  call dd_mul(t_z1,t_z2,t_ax,
!                              0.0,t_ax,0.0)
                   t_z1 = t_ax * t_ax
                   t_u1 = 134217729.0 * t_ax
                   t_p1 = t_u1
     +                      - (t_u1 - t_ax)
                   t_p2 = t_ax - t_p1
                   t_u1 = 134217729.0 * t_ax
                   t_r1 = t_u1
     +                      - (t_u1 - t_ax)
                   t_r2 = t_ax - t_r1
                   t_z2 = ((t_p1 * t_r1
     +                      - t_z1)
     +                      + t_p1 * t_r2
     +                      + t_p2 * t_r1)
     +                      + t_p2*t_r2
                   t_s1 = t_z1 + t_z2
                   t_z2 = t_z2
     +                      - (t_s1 - t_z1)
                   t_z1 = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                  call dd_sub(t_t2h,t_t2l,
!                           Th,Tl,t_z1,t_z2)
                   t_p1 = Th - t_z1
                   t_p2 = t_p1 - Th
                   t_t2l = (Th
     +                      -(t_p1 - t_p2))
     +                      -(t_z1 + t_p2)
                   t_t2h = t_p1
                   t_t2l = t_t2l
     +                      + Tl - t_z2
                   t_s1 = t_t2h + t_t2l
                   t_t2l = t_t2l
     +                      - (t_s1 - t_t2h)
                   t_t2h = t_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!                  call dd_mul(t_t1h,t_t1l,
!                              t_t2h,t_t2l,
!                              t_y1,t_y2)
                   t_t1h = t_t2h * t_y1
                   t_u1 = 134217729.0 * t_t2h
                   t_p1 = t_u1
     +                      - (t_u1 - t_t2h)
                   t_p2 = t_t2h - t_p1
                   t_u1 = 134217729.0 * t_y1
                   t_r1 = t_u1
     +                      - (t_u1 - t_y1)
                   t_r2 = t_y1 - t_r1
                   t_t1l = ((t_p1*t_r1
     +                      - t_t1h)
     +                      + t_p1*t_r2
     +                      + t_p2*t_r1)
     +                      + t_p2*t_r2
                   t_t1l = t_t1l
     +                      + (t_t2h * t_y2
     +                      +  t_t2l * t_y1)
                   t_s1 = t_t1h + t_t1l
                   t_t1l = t_t1l - (t_s1
     +                       - t_t1h)
                   t_t1h = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                  call dd_selfadd(t_t1h,t_t1l,
!                              t_ax,0.0)
                   t_p1 = t_t1h + t_ax
                   t_p2 = t_p1 - t_t1h
                   t_q2 = (t_t1h
     +                      - (t_p1 - t_p2))
     +                      + (t_ax - t_p2)
                   t_q1 = t_p1
                   t_q2 = t_q2 + t_t1l
                   t_t1h = t_q1 + t_q2
                   t_t1l = t_q2
     +                       - (t_t1h - t_q1)
                   tmp1h = t_t1h
                   tmp1l = t_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
               endif
!>>>           tmp1 = sqrt(T)
!<<<           tmp2 = T + S
               t_p1 = Th + Sh
               t_p2 = t_p1 - Th
               t_q1 = t_p1
               t_q2 = (Th - (t_p1 - t_p2))
     +                  + (Sh - t_p2)
               t_q2 = t_q2 + Tl + Sl
               tmp2h = t_q1 + t_q2
               tmp2l = t_q2 - (tmp2h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>           tmp2 = T + S
!<<<           tmp2 = sqrt(tmp2)
               if (tmp2h.eq.0.0D+0.and.tmp2l.eq.0.0D+0) then
                 tmp2h = 0.0d+0
                 tmp2l = 0.0d+0
               else
               if (tmp2h.lt.0.0d+0.or.
     +            (tmp2h.eq.0.0d+0.and.tmp2l.lt.0.0d+0)) then
                 write(6,'(a)')
     +              'SQRT CALLED WITH NEGATIVE ARGUMENT'
                 tmp2h = 0.0d+0
                 tmp2l = 0.0d+0
                 return
               endif
               t_x = 1.0D+0/sqrt(tmp2h)
               t_ax = tmp2h * t_x
!                  call dd_mul(t_y1,t_y2,t_x,
!                              0.0,0.5,0.0)
                   t_y1 = t_x * 0.5d+0
                   t_u1 = 134217729.0 * t_x
                   t_p1 = t_u1
     +                      - (t_u1 - t_x)
                   t_p2 = t_x - t_p1
                   t_u1 = 134217729.0 * 0.5d+0
                   t_r1 = t_u1
     +                      - (t_u1 - 0.5d+0)
                   t_r2 = 0.5d+0 - t_r1
                   t_y2 = ((t_p1*t_r1
     +                      - t_y1)
     +                      + t_p1*t_r2
     +                      + t_p2 * t_r1)
     +                      + t_p2 * t_r2
                   t_s1 = t_y1 + t_y2
                   t_y2 = t_y2
     +                      - (t_s1 - t_y1)
                   t_y1 = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                  call dd_mul(t_z1,t_z2,t_ax,
!                              0.0,t_ax,0.0)
                   t_z1 = t_ax * t_ax
                   t_u1 = 134217729.0 * t_ax
                   t_p1 = t_u1
     +                      - (t_u1 - t_ax)
                   t_p2 = t_ax - t_p1
                   t_u1 = 134217729.0 * t_ax
                   t_r1 = t_u1
     +                      - (t_u1 - t_ax)
                   t_r2 = t_ax - t_r1
                   t_z2 = ((t_p1 * t_r1
     +                      - t_z1)
     +                      + t_p1 * t_r2
     +                      + t_p2 * t_r1)
     +                      + t_p2*t_r2
                   t_s1 = t_z1 + t_z2
                   t_z2 = t_z2
     +                      - (t_s1 - t_z1)
                   t_z1 = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                  call dd_sub(t_t2h,t_t2l,
!                           tmp2h,tmp2l,t_z1,t_z2)
                   t_p1 = tmp2h - t_z1
                   t_p2 = t_p1 - tmp2h
                   t_t2l = (tmp2h
     +                      -(t_p1 - t_p2))
     +                      -(t_z1 + t_p2)
                   t_t2h = t_p1
                   t_t2l = t_t2l
     +                      + tmp2l - t_z2
                   t_s1 = t_t2h + t_t2l
                   t_t2l = t_t2l
     +                      - (t_s1 - t_t2h)
                   t_t2h = t_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!                  call dd_mul(t_t1h,t_t1l,
!                              t_t2h,t_t2l,
!                              t_y1,t_y2)
                   t_t1h = t_t2h * t_y1
                   t_u1 = 134217729.0 * t_t2h
                   t_p1 = t_u1
     +                      - (t_u1 - t_t2h)
                   t_p2 = t_t2h - t_p1
                   t_u1 = 134217729.0 * t_y1
                   t_r1 = t_u1
     +                      - (t_u1 - t_y1)
                   t_r2 = t_y1 - t_r1
                   t_t1l = ((t_p1*t_r1
     +                      - t_t1h)
     +                      + t_p1*t_r2
     +                      + t_p2*t_r1)
     +                      + t_p2*t_r2
                   t_t1l = t_t1l
     +                      + (t_t2h * t_y2
     +                      +  t_t2l * t_y1)
                   t_s1 = t_t1h + t_t1l
                   t_t1l = t_t1l - (t_s1
     +                       - t_t1h)
                   t_t1h = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                  call dd_selfadd(t_t1h,t_t1l,
!                              t_ax,0.0)
                   t_p1 = t_t1h + t_ax
                   t_p2 = t_p1 - t_t1h
                   t_q2 = (t_t1h
     +                      - (t_p1 - t_p2))
     +                      + (t_ax - t_p2)
                   t_q1 = t_p1
                   t_q2 = t_q2 + t_t1l
                   t_t1h = t_q1 + t_q2
                   t_t1l = t_q2
     +                       - (t_t1h - t_q1)
                   tmp2h = t_t1h
                   tmp2l = t_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
               endif
!>>>           tmp2 = sqrt(tmp2)
!<<<           tmp1 = tmp1 * tmp2
               t_ch = tmp1h * tmp2h
               t_u1 = 134217729.0 * tmp1h
               t_p1 = t_u1
     +                  - (t_u1 - tmp1h)
               t_p2 = tmp1h - t_p1
               t_u1 = 134217729.0 * tmp2h
               t_r1 = t_u1
     +                  - (t_u1 - tmp2h)
               t_r2 = tmp2h - t_r1
               t_cl = ((t_p1*t_r1
     +                  - t_ch)
     +                 + t_p1 * t_r2
     +                 + t_p2 * t_r1)
     +                 + t_p2 * t_r2
               t_cl = t_cl
     +                  + (tmp1h*tmp2l
     +                  +  tmp1l*tmp2h)
               t_s1 = t_ch + t_cl
               tmp1l = t_cl - (t_s1 - t_ch)
               tmp1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>           tmp1 = tmp1 * tmp2
!<<<           tmp1 = T + tmp1
               t_p1 = Th + tmp1h
               t_p2 = t_p1 - Th
               t_q1 = t_p1
               t_q2 = (Th - (t_p1 - t_p2))
     +                  + (tmp1h - t_p2)
               t_q2 = t_q2 + Tl + tmp1l
               tmp1h = t_q1 + t_q2
               tmp1l = t_q2 - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>           tmp1 = T + tmp1
!<<<           tmp1 = Z(2) / tmp1
               t_q1 = Zh(2) / tmp1h
               t_ch = tmp1h * t_q1
               t_u1 = 134217729.0 * tmp1h
               t_p1 = t_u1
     +                  - (t_u1 - tmp1h)
               t_p2 = tmp1h - t_p1
               t_u1 = 134217729.0 * t_q1
               t_r1 = t_u1
     +                  - (t_u1 - t_q1)
               t_r2 = t_q1 - t_r1
               t_cl = ((t_p1*t_r1
     +                  - t_ch)
     +                  + t_p1*t_r2
     +                  + t_p2*t_r1)
     +                  + t_p2*t_r2
               t_cl = t_cl
     +                  + (tmp1l * t_q1)
               t_s1 = t_ch + t_cl
               t_cl = t_cl
     +                  - (t_s1 - t_ch)
               t_ch = t_s1
               t_s1 = Zh(2) - t_ch
               t_r1 = t_s1 - Zh(2)
               t_s2 = (Zh(2)
     +                  - (t_s1 - t_r1))
     +                  - (t_ch + t_r1)
               t_s2 = t_s2 - t_cl
               t_s2 = t_s2 + Zl(2)
               t_q2 = (t_s1 + t_s2)
     +                  / tmp1h
               tmp1h = t_q1 + t_q2
               tmp1l = t_q2
     +               - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>           tmp1 = Z(2) / tmp1
!<<<           S = Z(3) * tmp1
               t_ch = Zh(3) * tmp1h
               t_u1 = 134217729.0 * Zh(3)
               t_p1 = t_u1
     +                  - (t_u1 - Zh(3))
               t_p2 = Zh(3) - t_p1
               t_u1 = 134217729.0 * tmp1h
               t_r1 = t_u1
     +                  - (t_u1 - tmp1h)
               t_r2 = tmp1h - t_r1
               t_cl = ((t_p1*t_r1
     +                  - t_ch)
     +                 + t_p1 * t_r2
     +                 + t_p2 * t_r1)
     +                 + t_p2 * t_r2
               t_cl = t_cl
     +                  + (Zh(3)*tmp1l
     +                  +  Zl(3)*tmp1h)
               t_s1 = t_ch + t_cl
               Sl = t_cl - (t_s1 - t_ch)
               Sh = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>           S = Z(3) * tmp1
            END IF
!            T = Z( 1 ) + ( S+Z( 2 ) )
!<<<        tmp1 = S + Z(2)
            t_p1 = Sh + Zh(2)
            t_p2 = t_p1 - Sh
            t_q1 = t_p1
            t_q2 = (Sh - (t_p1 - t_p2))
     +               + (Zh(2) - t_p2)
            t_q2 = t_q2 + Sl + Zl(2)
            tmp1h = t_q1 + t_q2
            tmp1l = t_q2 - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>        tmp1 = S + Z(2)
!<<<        T = Z(1) + tmp1
            t_p1 = Zh(1) + tmp1h
            t_p2 = t_p1 - Zh(1)
            t_q1 = t_p1
            t_q2 = (Zh(1) - (t_p1 - t_p2))
     +               + (tmp1h - t_p2)
            t_q2 = t_q2 + Zl(1) + tmp1l
            Th = t_q1 + t_q2
            Tl = t_q2 - (Th - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>        T = Z(1) + tmp1
!            Z( 3 ) = Z( 3 )*( Z( 1 ) / T )
!<<<        tmp1 = Z(1) / T
            t_q1 = Zh(1) / Th
            t_ch = Th * t_q1
            t_u1 = 134217729.0 * Th
            t_p1 = t_u1
     +               - (t_u1 - Th)
            t_p2 = Th - t_p1
            t_u1 = 134217729.0 * t_q1
            t_r1 = t_u1
     +               - (t_u1 - t_q1)
            t_r2 = t_q1 - t_r1
            t_cl = ((t_p1*t_r1
     +               - t_ch)
     +               + t_p1*t_r2
     +               + t_p2*t_r1)
     +               + t_p2*t_r2
            t_cl = t_cl
     +               + (Tl * t_q1)
            t_s1 = t_ch + t_cl
            t_cl = t_cl
     +               - (t_s1 - t_ch)
            t_ch = t_s1
            t_s1 = Zh(1) - t_ch
            t_r1 = t_s1 - Zh(1)
            t_s2 = (Zh(1)
     +               - (t_s1 - t_r1))
     +               - (t_ch + t_r1)
            t_s2 = t_s2 - t_cl
            t_s2 = t_s2 + Zl(1)
            t_q2 = (t_s1 + t_s2)
     +               / Th
            tmp1h = t_q1 + t_q2
            tmp1l = t_q2
     +            - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>        tmp1 = Z(1) / T
!<<<        Z(3) = Z(3) * tmp1
            t_ch = Zh(3) * tmp1h
            t_u1 = 134217729.0 * Zh(3)
            t_p1 = t_u1
     +               - (t_u1 - Zh(3))
            t_p2 = Zh(3) - t_p1
            t_u1 = 134217729.0 * tmp1h
            t_r1 = t_u1
     +               - (t_u1 - tmp1h)
            t_r2 = tmp1h - t_r1
            t_cl = ((t_p1*t_r1
     +               - t_ch)
     +              + t_p1 * t_r2
     +              + t_p2 * t_r1)
     +              + t_p2 * t_r2
            t_cl = t_cl
     +               + (Zh(3)*tmp1l
     +               +  Zl(3)*tmp1h)
            t_s1 = t_ch + t_cl
            Zl(3) = t_cl - (t_s1 - t_ch)
            Zh(3) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>        Z(3) = Z(3) * tmp1
!            Z( 1 ) = T
            Zh(1) = Th
            Zl(1) = Tl
         END IF
!         Z( 2 ) = Z( 3 )
!         Z( 6 ) = Z( 2 ) + Z( 1 )
         Zh(2) = Zh(3)
         Zl(2) = Zl(3)
!<<<     Z(6) = Z(2) + Z(1)
         t_p1 = Zh(2) + Zh(1)
         t_p2 = t_p1 - Zh(2)
         t_q1 = t_p1
         t_q2 = (Zh(2) - (t_p1 - t_p2))
     +            + (Zh(1) - t_p2)
         t_q2 = t_q2 + Zl(2) + Zl(1)
         Zh(6) = t_q1 + t_q2
         Zl(6) = t_q2 - (Zh(6) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>     Z(6) = Z(2) + Z(1)
         RETURN
      END IF
*
*     Check for negative data and compute sums of q's and e's.
*
!      Z( 2*N ) = ZERO
!      EMIN = Z( 2 )
!      QMAX = ZERO
!      ZMAX = ZERO
!      D = ZERO
!      E = ZERO
      Zh(2*N) = ZEROh
      Zl(2*N) = ZEROl
      EMINh = Zh(2)
      EMINl = Zl(2)
      QMAXh = ZEROh
      QMAXl = ZEROl
      ZMAXh = ZEROh
      ZMAXl = ZEROl
      Dh = ZEROh
      Dl = ZEROl
      Eh = ZEROh
      El = ZEROl
*
      DO 10 K = 1, 2*( N-1 ), 2
!         IF( Z( K ).LT.ZERO ) THEN
         IF(
!       (Z(K) .lt. 0.0)
     + ((Zh(K) .lt. 0.0d+0) .or.
     +  ((Zh(K) .eq. 0.0d+0) .and.
     +   (Zl(K) .lt. 0.0d+0)))
     +     ) THEN
            INFO = -( 200+K )
            CALL XERBLA( 'DLASQ2', 2 )
            RETURN
!         ELSE IF( Z( K+1 ).LT.ZERO ) THEN
         ELSE IF(
!       (Z(K+1) .lt. 0.0)
     + ((Zh(K+1) .lt. 0.0d+0) .or.
     +  ((Zh(K+1) .eq. 0.0d+0) .and.
     +   (Zl(K+1) .lt. 0.0d+0)))
     +          ) THEN
            INFO = -( 200+K+1 )
            CALL XERBLA( 'DLASQ2', 2 )
            RETURN
         END IF
!         D = D + Z( K )
!<<<  tmp1 = D + Z(K)
      t_p1 = Dh + Zh(K)
      t_p2 = t_p1 - Dh
      t_q1 = t_p1
      t_q2 = (Dh - (t_p1 - t_p2))
     +         + (Zh(K) - t_p2)
      t_q2 = t_q2 + Dl + Zl(K)
      tmp1h = t_q1 + t_q2
      tmp1l = t_q2 - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = D + Z(K)
      Dh = tmp1h
      Dl = tmp1l
!         E = E + Z( K+1 )
!<<<  tmp1 = E + Z(K+1)
      t_p1 = Eh + Zh(K+1)
      t_p2 = t_p1 - Eh
      t_q1 = t_p1
      t_q2 = (Eh - (t_p1 - t_p2))
     +         + (Zh(K+1) - t_p2)
      t_q2 = t_q2 + El + Zl(K+1)
      tmp1h = t_q1 + t_q2
      tmp1l = t_q2 - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = E + Z(K+1)
      Eh = tmp1h
      El = tmp1l
!         QMAX = MAX( QMAX, Z( K ) )
!<<<         QMAX = MAX( QMAX, Z( K ) )
         IF(
!       (Z(K) .gt. QMAX)
     + ((Zh(K) .gt. QMAXh) .or.
     +  ((Zh(K) .eq. QMAXh) .and.
     +   (Zl(K) .gt. QMAXl)))
     +     ) THEN
      QMAXh = Zh(K)
      QMAXl = Zl(K)
         END IF
!>>>         QMAX = MAX( QMAX, Z( K ) )
!         EMIN = MIN( EMIN, Z( K+1 ) )
!<<<         EMIN = MIN( EMIN, Z( K+1 ) )
         IF(
!       (Z(K+1) .lt. EMIN)
     + ((Zh(K+1) .lt. EMINh) .or.
     +  ((Zh(K+1) .eq. EMINh) .and.
     +   (Zl(K+1) .lt. EMINl)))
     +     ) THEN
      EMINh = Zh(K+1)
      EMINl = Zl(K+1)
         END IF
!>>>         EMIN = MIN( EMIN, Z( K+1 ) )
!         ZMAX = MAX( QMAX, ZMAX, Z( K+1 ) )
!<<<         ZMAX = MAX( QMAX, ZMAX, Z( K+1 ) )
         IF(
!       (QMAX .gt. ZMAX)
     + ((QMAXh .gt. ZMAXh) .or.
     +  ((QMAXh .eq. ZMAXh) .and.
     +   (QMAXl .gt. ZMAXl)))
     +     ) THEN
           IF(
!       (QMAX .gt. Z(K+1))
     + ((QMAXh .gt. Zh(K+1)) .or.
     +  ((QMAXh .eq. Zh(K+1)) .and.
     +   (QMAXl .gt. Zl(K+1))))
     +       ) THEN
      ZMAXh = QMAXh
      ZMAXl = QMAXl
           END IF
         ELSE IF(
!       (Z(K+1) .gt. ZMAX)
     + ((Zh(K+1) .gt. ZMAXh) .or.
     +  ((Zh(K+1) .eq. ZMAXh) .and.
     +   (Zl(K+1) .gt. ZMAXl)))
     +          ) THEN
      ZMAXh = Zh(K+1)
      ZMAXl = Zl(K+1)
         END IF
!>>>         ZMAX = MAX( QMAX, ZMAX, Z( K+1 ) )
   10 CONTINUE
!      IF( Z( 2*N-1 ).LT.ZERO ) THEN
      IF(
!       (Z(2*N-1) .lt. 0.0)
     + ((Zh(2*N-1) .lt. 0.0d+0) .or.
     +  ((Zh(2*N-1) .eq. 0.0d+0) .and.
     +   (Zl(2*N-1) .lt. 0.0d+0)))
     +  ) THEN
         INFO = -( 200+2*N-1 )
         CALL XERBLA( 'DLASQ2', 2 )
         RETURN
      END IF
!      D = D + Z( 2*N-1 )
!<<<  tmp1 = D + Z(2*N-1)
      t_p1 = Dh + Zh(2*N-1)
      t_p2 = t_p1 - Dh
      t_q1 = t_p1
      t_q2 = (Dh - (t_p1 - t_p2))
     +         + (Zh(2*N-1) - t_p2)
      t_q2 = t_q2 + Dl + Zl(2*N-1)
      tmp1h = t_q1 + t_q2
      tmp1l = t_q2 - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = D + Z(2*N-1)
      Dh = tmp1h
      Dl = tmp1l
!      QMAX = MAX( QMAX, Z( 2*N-1 ) )      
!<<<      QMAX = MAX( QMAX, Z( 2*N-1 ) )
      IF(
!       (Z(2*N-1) .gt. QMAX)
     + ((Zh(2*N-1) .gt. QMAXh) .or.
     +  ((Zh(2*N-1) .eq. QMAXh) .and.
     +   (Zl(2*N-1) .gt. QMAXl)))
     +  ) THEN
      QMAXh = Zh(2*N-1)
      QMAXl = Zl(2*N-1)
      END IF
!>>>      QMAX = MAX( QMAX, Z( 2*N-1 ) )      
!      ZMAX = MAX( QMAX, ZMAX )
!<<<      ZMAX = MAX( QMAX, ZMAX )
      IF(
!       (QMAX .gt. ZMAX)
     + ((QMAXh .gt. ZMAXh) .or.
     +  ((QMAXh .eq. ZMAXh) .and.
     +   (QMAXl .gt. ZMAXl)))
     +  ) THEN
      ZMAXh = QMAXh
      ZMAXl = QMAXl
      END IF
!>>>      ZMAX = MAX( QMAX, ZMAX )
*
*     Check for diagonality.
*
!      IF( E.EQ.ZERO ) THEN
      IF(
!       (E .eq. 0.0)
     + ((Eh .eq. 0.0d+0) .and.
     +  (El .eq. 0.0d+0))
     +  ) THEN
         DO 20 K = 2, N
!            Z( K ) = Z( 2*K-1 )
      Zh(K) = Zh(2*K-1)
      Zl(K) = Zl(2*K-1)
   20    CONTINUE
!         CALL DLASRT( 'D', N, Z, IINFO )
         CALL DDLASRT( 'D', N, Zh, Zl, IINFO )
!         Z( 2*N-1 ) = D
      Zh(2*N-1) = Dh
      Zl(2*N-1) = Dl
         RETURN
      END IF
*
!      TRACE = D + E
!<<<  TRACE = D + E
      t_p1 = Dh + Eh
      t_p2 = t_p1 - Dh
      t_q1 = t_p1
      t_q2 = (Dh - (t_p1 - t_p2))
     +         + (Eh - t_p2)
      t_q2 = t_q2 + Dl + El
      TRACEh = t_q1 + t_q2
      TRACEl = t_q2 - (TRACEh - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  TRACE = D + E
*
*     Check for zero data.
*
!      IF( TRACE.EQ.ZERO ) THEN
      IF(
!       (TRACE .eq. 0.0)
     + ((TRACEh .eq. 0.0d+0) .and.
     +  (TRACEl .eq. 0.0d+0))
     +  ) THEN
!         Z( 2*N-1 ) = ZERO
      Zh(2*N-1) = 0.0d+0
      Zl(2*N-1) = 0.0d+0
         RETURN
      END IF
*         
*     Check whether the machine is IEEE conformable.
*         
      IEEE = ILAENV( 10, 'DLASQ2', 'N', 1, 2, 3, 4 ).EQ.1 .AND.
     $       ILAENV( 11, 'DLASQ2', 'N', 1, 2, 3, 4 ).EQ.1      
*         
*     Rearrange data for locality: Z=(q1,qq1,e1,ee1,q2,qq2,e2,ee2,...).
*
      DO 30 K = 2*N, 2, -2
!         Z( 2*K ) = ZERO 
      Zh(2*K) = 0.0d+0
      Zl(2*K) = 0.0d+0
!         Z( 2*K-1 ) = Z( K ) 
      Zh(2*K-1) = Zh(K)
      Zl(2*K-1) = Zl(K)
!         Z( 2*K-2 ) = ZERO 
      Zh(2*K-2) = 0.0d+0
      Zl(2*K-2) = 0.0d+0
!         Z( 2*K-3 ) = Z( K-1 ) 
      Zh(2*K-3) = Zh(K-1)
      Zl(2*K-3) = Zl(K-1)
   30 CONTINUE
*
      I0 = 1
      N0 = N
*
*     Reverse the qd-array, if warranted.
*
!      IF( CBIAS*Z( 4*I0-3 ).LT.Z( 4*N0-3 ) ) THEN
!<<<  tmp1 = CBIAS * Z(4*I0-3)
      t_ch = CBIASh * Zh(4*I0-3)
      t_u1 = 134217729.0 * CBIASh
      t_p1 = t_u1
     +         - (t_u1 - CBIASh)
      t_p2 = CBIASh - t_p1
      t_u1 = 134217729.0 * Zh(4*I0-3)
      t_r1 = t_u1
     +         - (t_u1 - Zh(4*I0-3))
      t_r2 = Zh(4*I0-3) - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (CBIASh*Zl(4*I0-3)
     +         +  CBIASl*Zh(4*I0-3))
      t_s1 = t_ch + t_cl
      tmp1l = t_cl - (t_s1 - t_ch)
      tmp1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = CBIAS * Z(4*I0-3)
      IF(
!       (tmp1 .lt. Z(4*N0-3))
     + ((tmp1h .lt. Zh(4*N0-3)) .or.
     +  ((tmp1h .eq. Zh(4*N0-3)) .and.
     +   (tmp1l .lt. Zl(4*N0-3))))
     +  ) THEN
         IPN4 = 4*( I0+N0 )
         DO 40 I4 = 4*I0, 2*( I0+N0-1 ), 4
!            TEMP = Z( I4-3 )
      TEMPh = Zh(I4-3)
      TEMPl = Zl(I4-3)
!            Z( I4-3 ) = Z( IPN4-I4-3 )
      Zh(I4-3) = Zh(IPN4-I4-3)
      Zl(I4-3) = Zl(IPN4-I4-3)
!            Z( IPN4-I4-3 ) = TEMP
      Zh(IPN4-I4-3) = TEMPh
      Zl(IPN4-I4-3) = TEMPl
!            TEMP = Z( I4-1 )
      TEMPh = Zh(I4-1)
      TEMPl = Zl(I4-1)
!            Z( I4-1 ) = Z( IPN4-I4-5 )
      Zh(I4-1) = Zh(IPN4-I4-5)
      Zl(I4-1) = Zl(IPN4-I4-5)
!            Z( IPN4-I4-5 ) = TEMP
      Zh(IPN4-I4-5) = TEMPh
      Zl(IPN4-I4-5) = TEMPl
   40    CONTINUE
      END IF
*
*     Initial split checking via dqd and Li's test.
*
      PP = 0
*
      DO 80 K = 1, 2
*
!         D = Z( 4*N0+PP-3 )
      Dh = Zh(4*N0+PP-3)
      Dl = Zl(4*N0+PP-3)
         DO 50 I4 = 4*( N0-1 ) + PP, 4*I0 + PP, -4
!            IF( Z( I4-1 ).LE.TOL2*D ) THEN
!<<<  tmp1 = TOL2 * D
      t_ch = TOL2h * Dh
      t_u1 = 134217729.0 * TOL2h
      t_p1 = t_u1
     +         - (t_u1 - TOL2h)
      t_p2 = TOL2h - t_p1
      t_u1 = 134217729.0 * Dh
      t_r1 = t_u1
     +         - (t_u1 - Dh)
      t_r2 = Dh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (TOL2h*Dl
     +         +  TOL2l*Dh)
      t_s1 = t_ch + t_cl
      tmp1l = t_cl - (t_s1 - t_ch)
      tmp1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = TOL2 * D
            IF(
!       (Z(I4-1) .le. tmp1)
     + ((Zh(I4-1) .lt. tmp1h) .or.
     +  ((Zh(I4-1) .eq. tmp1h) .and.
     +   (Zl(I4-1) .le. tmp1l)))
     +        ) THEN
!               Z( I4-1 ) = -ZERO
!<<<  Z(I4-1) = -ZERO
      Zh(I4-1) = -ZEROh
      Zl(I4-1) = -ZEROl
!>>>  Z(I4-1) = -ZERO
!               D = Z( I4-3 )
      Dh = Zh(I4-3)
      Dl = Zl(I4-3)
            ELSE
!               D = Z( I4-3 )*( D / ( D+Z( I4-1 ) ) )
!<<<  tmp1 = D + Z(I4-1)
      t_p1 = Dh + Zh(I4-1)
      t_p2 = t_p1 - Dh
      t_q1 = t_p1
      t_q2 = (Dh - (t_p1 - t_p2))
     +         + (Zh(I4-1) - t_p2)
      t_q2 = t_q2 + Dl + Zl(I4-1)
      tmp1h = t_q1 + t_q2
      tmp1l = t_q2 - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = D + Z(I4-1)
!<<<  tmp2 = D / tmp1
      t_q1 = Dh / tmp1h
      t_ch = tmp1h * t_q1
      t_u1 = 134217729.0 * tmp1h
      t_p1 = t_u1
     +         - (t_u1 - tmp1h)
      t_p2 = tmp1h - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (tmp1l * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = Dh - t_ch
      t_r1 = t_s1 - Dh
      t_s2 = (Dh
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + Dl
      t_q2 = (t_s1 + t_s2)
     +         / tmp1h
      tmp2h = t_q1 + t_q2
      tmp2l = t_q2
     +      - (tmp2h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = D / tmp1
!<<<  tmp1 = Z(I4-3) * tmp2
      t_ch = Zh(I4-3) * tmp2h
      t_u1 = 134217729.0 * Zh(I4-3)
      t_p1 = t_u1
     +         - (t_u1 - Zh(I4-3))
      t_p2 = Zh(I4-3) - t_p1
      t_u1 = 134217729.0 * tmp2h
      t_r1 = t_u1
     +         - (t_u1 - tmp2h)
      t_r2 = tmp2h - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (Zh(I4-3)*tmp2l
     +         +  Zl(I4-3)*tmp2h)
      t_s1 = t_ch + t_cl
      tmp1l = t_cl - (t_s1 - t_ch)
      tmp1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = Z(I4-3) * tmp2
      Dh = tmp1h
      Dl = tmp1l
            END IF
   50    CONTINUE
*
*        dqd maps Z to ZZ plus Li's test.
*
!         EMIN = Z( 4*I0+PP+1 )
      EMINh = Zh(4*I0+PP+1)
      EMINl = Zl(4*I0+PP+1)
!         D = Z( 4*I0+PP-3 )
      Dh = Zh(4*I0+PP-3)
      Dl = Zl(4*I0+PP-3)
         DO 60 I4 = 4*I0 + PP, 4*( N0-1 ) + PP, 4
!            Z( I4-2*PP-2 ) = D + Z( I4-1 )
!<<<  tmp1 = D + Z(I4-1)
      t_p1 = Dh + Zh(I4-1)
      t_p2 = t_p1 - Dh
      t_q1 = t_p1
      t_q2 = (Dh - (t_p1 - t_p2))
     +         + (Zh(I4-1) - t_p2)
      t_q2 = t_q2 + Dl + Zl(I4-1)
      tmp1h = t_q1 + t_q2
      tmp1l = t_q2 - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = D + Z(I4-1)
      Zh(I4-2*PP-2) = tmp1h
      Zl(I4-2*PP-2) = tmp1l
!<<< SAFMIN*Z( I4+1 )
!<<<  tmp2 = SAFMIN * Z(I4+1)
      t_ch = SAFMINh * Zh(I4+1)
      t_u1 = 134217729.0 * SAFMINh
      t_p1 = t_u1
     +         - (t_u1 - SAFMINh)
      t_p2 = SAFMINh - t_p1
      t_u1 = 134217729.0 * Zh(I4+1)
      t_r1 = t_u1
     +         - (t_u1 - Zh(I4+1))
      t_r2 = Zh(I4+1) - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (SAFMINh*Zl(I4+1)
     +         +  SAFMINl*Zh(I4+1))
      t_s1 = t_ch + t_cl
      tmp2l = t_cl - (t_s1 - t_ch)
      tmp2h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = SAFMIN * Z(I4+1)
!>>> SAFMIN*Z( I4+1 )
!<<< SAFMIN*Z( I4-2*PP-2 )
!<<<  tmp3 = SAFMIN * Z(I4-2*PP-2)
      t_ch = SAFMINh * Zh(I4-2*PP-2)
      t_u1 = 134217729.0 * SAFMINh
      t_p1 = t_u1
     +         - (t_u1 - SAFMINh)
      t_p2 = SAFMINh - t_p1
      t_u1 = 134217729.0 * Zh(I4-2*PP-2)
      t_r1 = t_u1
     +         - (t_u1 - Zh(I4-2*PP-2))
      t_r2 = Zh(I4-2*PP-2) - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (SAFMINh*Zl(I4-2*PP-2)
     +         +  SAFMINl*Zh(I4-2*PP-2))
      t_s1 = t_ch + t_cl
      tmp3l = t_cl - (t_s1 - t_ch)
      tmp3h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = SAFMIN * Z(I4-2*PP-2)
!>>> SAFMIN*Z( I4-2*PP-2 )
!            IF( Z( I4-1 ).LE.TOL2*D ) THEN
!<<<  tmp1 = TOL2 * D
      t_ch = TOL2h * Dh
      t_u1 = 134217729.0 * TOL2h
      t_p1 = t_u1
     +         - (t_u1 - TOL2h)
      t_p2 = TOL2h - t_p1
      t_u1 = 134217729.0 * Dh
      t_r1 = t_u1
     +         - (t_u1 - Dh)
      t_r2 = Dh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (TOL2h*Dl
     +         +  TOL2l*Dh)
      t_s1 = t_ch + t_cl
      tmp1l = t_cl - (t_s1 - t_ch)
      tmp1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = TOL2 * D
            IF(
!       (Z(I4-1) .le. tmp1)
     + ((Zh(I4-1) .lt. tmp1h) .or.
     +  ((Zh(I4-1) .eq. tmp1h) .and.
     +   (Zl(I4-1) .le. tmp1l)))
     +        ) THEN
!               Z( I4-1 ) = -ZERO
!<<<  Z(I4-1) = -ZERO
      Zh(I4-1) = -ZEROh
      Zl(I4-1) = -ZEROl
!>>>  Z(I4-1) = -ZERO
!               Z( I4-2*PP-2 ) = D
      Zh(I4-2*PP-2) = Dh
      Zl(I4-2*PP-2) = Dl
!               Z( I4-2*PP ) = ZERO
      Zh(I4-2*PP) = 0.0d+0
      Zl(I4-2*PP) = 0.0d+0
!               D = Z( I4+1 )
      Dh = Zh(I4+1)
      Dl = Zl(I4+1)
!            ELSE IF( SAFMIN*Z( I4+1 ).LT.Z( I4-2*PP-2 ) .AND.
!     $               SAFMIN*Z( I4-2*PP-2 ).LT.Z( I4+1 ) ) THEN
            ELSE IF(
!       (tmp2 .lt. Z(I4-2*PP-2))
     + ((tmp2h .lt. Zh(I4-2*PP-2)) .or.
     +  ((tmp2h .eq. Zh(I4-2*PP-2)) .and.
     +   (tmp2l .lt. Zl(I4-2*PP-2))))
     +              .AND.
!       (tmp3 .lt. Z(I4+1))
     + ((tmp3h .lt. Zh(I4+1)) .or.
     +  ((tmp3h .eq. Zh(I4+1)) .and.
     +   (tmp3l .lt. Zl(I4+1))))
     +             ) THEN
!               TEMP = Z( I4+1 ) / Z( I4-2*PP-2 )
!<<<  TEMP = Z(I4+1) / Z(I4-2*PP-2)
      t_q1 = Zh(I4+1) / Zh(I4-2*PP-2)
      t_ch = Zh(I4-2*PP-2) * t_q1
      t_u1 = 134217729.0 * Zh(I4-2*PP-2)
      t_p1 = t_u1
     +         - (t_u1 - Zh(I4-2*PP-2))
      t_p2 = Zh(I4-2*PP-2) - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (Zl(I4-2*PP-2) * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = Zh(I4+1) - t_ch
      t_r1 = t_s1 - Zh(I4+1)
      t_s2 = (Zh(I4+1)
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + Zl(I4+1)
      t_q2 = (t_s1 + t_s2)
     +         / Zh(I4-2*PP-2)
      TEMPh = t_q1 + t_q2
      TEMPl = t_q2
     +      - (TEMPh - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(I4+1) / Z(I4-2*PP-2)
!               Z( I4-2*PP ) = Z( I4-1 )*TEMP
!<<<  Z(I4-2*PP) = Z(I4-1) * TEMP
      t_ch = Zh(I4-1) * TEMPh
      t_u1 = 134217729.0 * Zh(I4-1)
      t_p1 = t_u1
     +         - (t_u1 - Zh(I4-1))
      t_p2 = Zh(I4-1) - t_p1
      t_u1 = 134217729.0 * TEMPh
      t_r1 = t_u1
     +         - (t_u1 - TEMPh)
      t_r2 = TEMPh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (Zh(I4-1)*TEMPl
     +         +  Zl(I4-1)*TEMPh)
      t_s1 = t_ch + t_cl
      Zl(I4-2*PP) = t_cl - (t_s1 - t_ch)
      Zh(I4-2*PP) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(I4-2*PP) = Z(I4-1) * TEMP
!               D = D*TEMP
!<<<  tmp1 = D * TEMP
      t_ch = Dh * TEMPh
      t_u1 = 134217729.0 * Dh
      t_p1 = t_u1
     +         - (t_u1 - Dh)
      t_p2 = Dh - t_p1
      t_u1 = 134217729.0 * TEMPh
      t_r1 = t_u1
     +         - (t_u1 - TEMPh)
      t_r2 = TEMPh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (Dh*TEMPl
     +         +  Dl*TEMPh)
      t_s1 = t_ch + t_cl
      tmp1l = t_cl - (t_s1 - t_ch)
      tmp1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = D * TEMP
      Dh = tmp1h
      Dl = tmp1l
            ELSE
!               Z( I4-2*PP ) = Z( I4+1 )*( Z( I4-1 ) / Z( I4-2*PP-2 ) )
!<<<  tmp1 = Z(I4-1) / Z(I4-2*PP-2)
      t_q1 = Zh(I4-1) / Zh(I4-2*PP-2)
      t_ch = Zh(I4-2*PP-2) * t_q1
      t_u1 = 134217729.0 * Zh(I4-2*PP-2)
      t_p1 = t_u1
     +         - (t_u1 - Zh(I4-2*PP-2))
      t_p2 = Zh(I4-2*PP-2) - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (Zl(I4-2*PP-2) * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = Zh(I4-1) - t_ch
      t_r1 = t_s1 - Zh(I4-1)
      t_s2 = (Zh(I4-1)
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + Zl(I4-1)
      t_q2 = (t_s1 + t_s2)
     +         / Zh(I4-2*PP-2)
      tmp1h = t_q1 + t_q2
      tmp1l = t_q2
     +      - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = Z(I4-1) / Z(I4-2*PP-2)
!<<<  Z(I4-2*PP) = Z(I4+1) * tmp1
      t_ch = Zh(I4+1) * tmp1h
      t_u1 = 134217729.0 * Zh(I4+1)
      t_p1 = t_u1
     +         - (t_u1 - Zh(I4+1))
      t_p2 = Zh(I4+1) - t_p1
      t_u1 = 134217729.0 * tmp1h
      t_r1 = t_u1
     +         - (t_u1 - tmp1h)
      t_r2 = tmp1h - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (Zh(I4+1)*tmp1l
     +         +  Zl(I4+1)*tmp1h)
      t_s1 = t_ch + t_cl
      Zl(I4-2*PP) = t_cl - (t_s1 - t_ch)
      Zh(I4-2*PP) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(I4-2*PP) = Z(I4+1) * tmp1
!               D = Z( I4+1 )*( D / Z( I4-2*PP-2 ) )
!<<<  tmp1 = D / Z(I4-2*PP-2)
      t_q1 = Dh / Zh(I4-2*PP-2)
      t_ch = Zh(I4-2*PP-2) * t_q1
      t_u1 = 134217729.0 * Zh(I4-2*PP-2)
      t_p1 = t_u1
     +         - (t_u1 - Zh(I4-2*PP-2))
      t_p2 = Zh(I4-2*PP-2) - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (Zl(I4-2*PP-2) * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = Dh - t_ch
      t_r1 = t_s1 - Dh
      t_s2 = (Dh
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + Dl
      t_q2 = (t_s1 + t_s2)
     +         / Zh(I4-2*PP-2)
      tmp1h = t_q1 + t_q2
      tmp1l = t_q2
     +      - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = D / Z(I4-2*PP-2)
!<<<  D = Z(I4+1) * tmp1
      t_ch = Zh(I4+1) * tmp1h
      t_u1 = 134217729.0 * Zh(I4+1)
      t_p1 = t_u1
     +         - (t_u1 - Zh(I4+1))
      t_p2 = Zh(I4+1) - t_p1
      t_u1 = 134217729.0 * tmp1h
      t_r1 = t_u1
     +         - (t_u1 - tmp1h)
      t_r2 = tmp1h - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (Zh(I4+1)*tmp1l
     +         +  Zl(I4+1)*tmp1h)
      t_s1 = t_ch + t_cl
      Dl = t_cl - (t_s1 - t_ch)
      Dh = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  D = Z(I4+1) * tmp1
            END IF
!            EMIN = MIN( EMIN, Z( I4-2*PP ) )
            IF(
!       (Z(I4-2*PP) .lt. EMIN)
     + ((Zh(I4-2*PP) .lt. EMINh) .or.
     +  ((Zh(I4-2*PP) .eq. EMINh) .and.
     +   (Zl(I4-2*PP) .lt. EMINl)))
     +        ) THEN
      EMINh = Zh(I4-2*PP)
      EMINl = Zl(I4-2*PP)
            END IF
   60    CONTINUE 
!         Z( 4*N0-PP-2 ) = D
      Zh(4*N0-PP-2) = Dh
      Zl(4*N0-PP-2) = Dl
*
*        Now find qmax.
*
!         QMAX = Z( 4*I0-PP-2 )
      QMAXh = Zh(4*I0-PP-2)
      QMAXl = Zl(4*I0-PP-2)
         DO 70 I4 = 4*I0 - PP + 2, 4*N0 - PP - 2, 4
!            QMAX = MAX( QMAX, Z( I4 ) )
            IF(
!       (Z(I4) .lt. QMAX)
     + ((Zh(I4) .lt. QMAXh) .or.
     +  ((Zh(I4) .eq. QMAXh) .and.
     +   (Zl(I4) .lt. QMAXl)))
     +        ) THEN
      QMAXh = Zh(I4)
      QMAXl = Zl(I4)
            END IF
   70    CONTINUE
*
*        Prepare for the next iteration on K.
*
         PP = 1 - PP
   80 CONTINUE
*
*     Initialise variables to pass to DLASQ3.
*
      TTYPE = 0
!      DMIN1 = ZERO
      DMIN1h = 0.0d+0
      DMIN1l = 0.0d+0
!      DMIN2 = ZERO
      DMIN2h = 0.0d+0
      DMIN2l = 0.0d+0
!      DN    = ZERO
      DNh = 0.0d+0
      DNl = 0.0d+0
!      DN1   = ZERO
      DN1h = 0.0d+0
      DN1l = 0.0d+0
!      DN2   = ZERO
      DN2h = 0.0d+0
      DN2l = 0.0d+0
!      G     = ZERO
      Gh = 0.0d+0
      Gl = 0.0d+0
!      TAU   = ZERO
      TAUh = 0.0d+0
      TAUl = 0.0d+0
*
      ITER = 2
      NFAIL = 0
      NDIV = 2*( N0-I0 )
*
      DO 160 IWHILA = 1, N + 1
         IF( N0.LT.1 ) 
     $      GO TO 170
*
*        While array unfinished do 
*
*        E(N0) holds the value of SIGMA when submatrix in I0:N0
*        splits from the rest of the array, but is negated.
*      
!         DESIG = ZERO
      DESIGh = 0.0d+0
      DESIGl = 0.0d+0
         IF( N0.EQ.N ) THEN
!            SIGMA = ZERO
      SIGMAh = 0.0d+0
      SIGMAl = 0.0d+0
         ELSE
!            SIGMA = -Z( 4*N0-1 )
!<<<  SIGMA = -Z(4*N0-1)
      SIGMAh = -Zh(4*N0-1)
      SIGMAl = -Zl(4*N0-1)
!>>>  SIGMA = -Z(4*N0-1)
         END IF
!         IF( SIGMA.LT.ZERO ) THEN
         IF(
!       (SIGMA .lt. 0.0)
     + ((SIGMAh .lt. 0.0d+0) .or.
     +  ((SIGMAh .eq. 0.0d+0) .and.
     +   (SIGMAl .lt. 0.0d+0)))
     +     ) THEN
            INFO = 1
            RETURN
         END IF
*
*        Find last unreduced submatrix's top index I0, find QMAX and
*        EMIN. Find Gershgorin-type bound if Q's much greater than E's.
*
!         EMAX = ZERO 
      EMAXh = 0.0d+0
      EMAXl = 0.0d+0
         IF( N0.GT.I0 ) THEN
!            EMIN = ABS( Z( 4*N0-5 ) )
!<<<  EMIN = abs(Z(4*N0-5))
      if (Zh(4*N0-5) .lt. 0.0d+0) then
        EMINh = -Zh(4*N0-5)
        EMINl = -Zl(4*N0-5)
      elseif (
     +  (Zh(4*N0-5) .eq. 0.0d+0).and.
     +  (Zl(4*N0-5) .lt. 0.0d+0)) then
        EMINh = Zh(4*N0-5)
        EMINl = -Zl(4*N0-5)
      else
        EMINh = Zh(4*N0-5)
        EMINl = Zl(4*N0-5)
      endif
!>>>  EMIN = abs(Z(4*N0-5))
         ELSE
!            EMIN = ZERO
      EMINh = 0.0d+0
      EMINl = 0.0d+0
         END IF
!         QMIN = Z( 4*N0-3 )
      QMINh = Zh(4*N0-3)
      QMINl = Zl(4*N0-3)
!         QMAX = QMIN
      QMAXh = QMINh
      QMAXl = QMINl
         DO 90 I4 = 4*N0, 8, -4
!            IF( Z( I4-5 ).LE.ZERO )
            IF(
!       (Z(I4-5) .le. 0.0)
     + ((Zh(I4-5) .lt. 0.0d+0) .or.
     +  ((Zh(I4-5) .eq. 0.0d+0) .and.
     +   (Zl(I4-5) .le. 0.0d+0)))
     +        )
     $         GO TO 100
!            IF( QMIN.GE.FOUR*EMAX ) THEN
!<<<  tmp1 = FOUR * EMAX
      t_ch = FOURh * EMAXh
      t_u1 = 134217729.0 * FOURh
      t_p1 = t_u1
     +         - (t_u1 - FOURh)
      t_p2 = FOURh - t_p1
      t_u1 = 134217729.0 * EMAXh
      t_r1 = t_u1
     +         - (t_u1 - EMAXh)
      t_r2 = EMAXh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (FOURh*EMAXl
     +         +  FOURl*EMAXh)
      t_s1 = t_ch + t_cl
      tmp1l = t_cl - (t_s1 - t_ch)
      tmp1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = FOUR * EMAX
            IF(
!       (QMIN .ge. tmp1)
     + ((QMINh .gt. tmp1h) .or.
     +  ((QMINh .eq. tmp1h) .and.
     +   (QMINl .ge. tmp1l)))
     +        ) THEN
!               QMIN = MIN( QMIN, Z( I4-3 ) )
               IF(
!       (Z(I4-3) .lt. QMIN)
     + ((Zh(I4-3) .lt. QMINh) .or.
     +  ((Zh(I4-3) .eq. QMINh) .and.
     +   (Zl(I4-3) .lt. QMINl)))
     +            ) THEN
      QMINh = Zh(I4-3)
      QMINl = Zl(I4-3)
               END IF
!               EMAX = MAX( EMAX, Z( I4-5 ) )
               IF(
!       (Z(I4-5) .gt. EMAX)
     + ((Zh(I4-5) .gt. EMAXh) .or.
     +  ((Zh(I4-5) .eq. EMAXh) .and.
     +   (Zl(I4-5) .gt. EMAXl)))
     +           ) THEN
      EMAXh = Zh(I4-5)
      EMAXl = Zl(I4-5)
               END IF
            END IF
!            QMAX = MAX( QMAX, Z( I4-7 )+Z( I4-5 ) )
!<<<  tmp1 = Z(I4-7) + Z(I4-5)
      t_p1 = Zh(I4-7) + Zh(I4-5)
      t_p2 = t_p1 - Zh(I4-7)
      t_q1 = t_p1
      t_q2 = (Zh(I4-7) - (t_p1 - t_p2))
     +         + (Zh(I4-5) - t_p2)
      t_q2 = t_q2 + Zl(I4-7) + Zl(I4-5)
      tmp1h = t_q1 + t_q2
      tmp1l = t_q2 - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = Z(I4-7) + Z(I4-5)
            IF(
!       (tmp1 .gt. QMAX)
     + ((tmp1h .gt. QMAXh) .or.
     +  ((tmp1h .eq. QMAXh) .and.
     +   (tmp1l .gt. QMAXl)))
     +        ) THEN
      QMAXh = tmp1h
      QMAXl = tmp1l
            END IF
!            EMIN = MIN( EMIN, Z( I4-5 ) )
            IF(
!       (Z(I4-5) .lt. EMIN)
     + ((Zh(I4-5) .lt. EMINh) .or.
     +  ((Zh(I4-5) .eq. EMINh) .and.
     +   (Zl(I4-5) .lt. EMINl)))
     +        ) THEN
      EMINh = Zh(I4-5)
      EMINl = Zl(I4-5)
            END IF
   90    CONTINUE
         I4 = 4 
*
  100    CONTINUE
         I0 = I4 / 4
         PP = 0
*
         IF( N0-I0.GT.1 ) THEN
!            DEE = Z( 4*I0-3 )
      DEEh = Zh(4*I0-3)
      DEEl = Zl(4*I0-3)
!            DEEMIN = DEE
      DEEMINh = DEEh
      DEEMINl = DEEl
            KMIN = I0
            DO 110 I4 = 4*I0+1, 4*N0-3, 4
!               DEE = Z( I4 )*( DEE /( DEE+Z( I4-2 ) ) )
!<<<  tmp1 = DEE + Z(I4-2)
      t_p1 = DEEh + Zh(I4-2)
      t_p2 = t_p1 - DEEh
      t_q1 = t_p1
      t_q2 = (DEEh - (t_p1 - t_p2))
     +         + (Zh(I4-2) - t_p2)
      t_q2 = t_q2 + DEEl + Zl(I4-2)
      tmp1h = t_q1 + t_q2
      tmp1l = t_q2 - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = DEE + Z(I4-2)
!<<<  tmp2 = DEE / tmp1
      t_q1 = DEEh / tmp1h
      t_ch = tmp1h * t_q1
      t_u1 = 134217729.0 * tmp1h
      t_p1 = t_u1
     +         - (t_u1 - tmp1h)
      t_p2 = tmp1h - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (tmp1l * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = DEEh - t_ch
      t_r1 = t_s1 - DEEh
      t_s2 = (DEEh
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + DEEl
      t_q2 = (t_s1 + t_s2)
     +         / tmp1h
      tmp2h = t_q1 + t_q2
      tmp2l = t_q2
     +      - (tmp2h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = DEE / tmp1
!<<<  DEE = Z(I4) * tmp2
      t_ch = Zh(I4) * tmp2h
      t_u1 = 134217729.0 * Zh(I4)
      t_p1 = t_u1
     +         - (t_u1 - Zh(I4))
      t_p2 = Zh(I4) - t_p1
      t_u1 = 134217729.0 * tmp2h
      t_r1 = t_u1
     +         - (t_u1 - tmp2h)
      t_r2 = tmp2h - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (Zh(I4)*tmp2l
     +         +  Zl(I4)*tmp2h)
      t_s1 = t_ch + t_cl
      DEEl = t_cl - (t_s1 - t_ch)
      DEEh = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  DEE = Z(I4) * tmp2
!               IF( DEE.LE.DEEMIN ) THEN
               IF( 
!       (DEE .le. DEEMIN)
     + ((DEEh .lt. DEEMINh) .or.
     +  ((DEEh .eq. DEEMINh) .and.
     +   (DEEl .le. DEEMINl)))
     +           ) THEN
!                  DEEMIN = DEE
      DEEMINh = DEEh
      DEEMINl = DEEl
                  KMIN = ( I4+3 )/4
               END IF
  110       CONTINUE
!            IF( (KMIN-I0)*2.LT.N0-KMIN .AND. 
!     $         DEEMIN.LE.HALF*Z(4*N0-3) ) THEN
!<<<  tmp1 = HALF * Z(4*N0-3)
      t_ch = HALFh * Zh(4*N0-3)
      t_u1 = 134217729.0 * HALFh
      t_p1 = t_u1
     +         - (t_u1 - HALFh)
      t_p2 = HALFh - t_p1
      t_u1 = 134217729.0 * Zh(4*N0-3)
      t_r1 = t_u1
     +         - (t_u1 - Zh(4*N0-3))
      t_r2 = Zh(4*N0-3) - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (HALFh*Zl(4*N0-3)
     +         +  HALFl*Zh(4*N0-3))
      t_s1 = t_ch + t_cl
      tmp1l = t_cl - (t_s1 - t_ch)
      tmp1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = HALF * Z(4*N0-3)
            IF( (KMIN-I0)*2.LT.N0-KMIN .AND. 
!       (DEEMIN .le. tmp1)
     + ((DEEMINh .lt. tmp1h) .or.
     +  ((DEEMINh .eq. tmp1h) .and.
     +   (DEEMINl .le. tmp1l)))
     +        ) THEN
               IPN4 = 4*( I0+N0 )
               PP = 2
               DO 120 I4 = 4*I0, 2*( I0+N0-1 ), 4
!                  TEMP = Z( I4-3 )
      TEMPh = Zh(I4-3)
      TEMPl = Zl(I4-3)
!                  Z( I4-3 ) = Z( IPN4-I4-3 )
      Zh(I4-3) = Zh(IPN4-I4-3)
      Zl(I4-3) = Zl(IPN4-I4-3)
!                  Z( IPN4-I4-3 ) = TEMP
      Zh(IPN4-I4-3) = TEMPh
      Zl(IPN4-I4-3) = TEMPl
!                  TEMP = Z( I4-2 )
      TEMPh = Zh(I4-2)
      TEMPl = Zl(I4-2)
!                  Z( I4-2 ) = Z( IPN4-I4-2 )
      Zh(I4-2) = Zh(IPN4-I4-2)
      Zl(I4-2) = Zl(IPN4-I4-2)
!                  Z( IPN4-I4-2 ) = TEMP
      Zh(IPN4-I4-2) = TEMPh
      Zl(IPN4-I4-2) = TEMPl
!                  TEMP = Z( I4-1 )
      TEMPh = Zh(I4-1)
      TEMPl = Zl(I4-1)
!                  Z( I4-1 ) = Z( IPN4-I4-5 )
      Zh(I4-1) = Zh(IPN4-I4-5)
      Zl(I4-1) = Zl(IPN4-I4-5)
!                  Z( IPN4-I4-5 ) = TEMP
      Zh(IPN4-I4-5) = TEMPh
      Zl(IPN4-I4-5) = TEMPl
!                  TEMP = Z( I4 )
      TEMPh = Zh(I4)
      TEMPl = Zl(I4)
!                  Z( I4 ) = Z( IPN4-I4-4 )
      Zh(I4) = Zh(IPN4-I4-4)
      Zl(I4) = Zl(IPN4-I4-4)
!                  Z( IPN4-I4-4 ) = TEMP
      Zh(IPN4-I4-4) = TEMPh
      Zl(IPN4-I4-4) = TEMPl
  120          CONTINUE
            END IF
         END IF
*
*        Put -(initial shift) into DMIN.
*
!         DMIN = -MAX( ZERO, QMIN-TWO*SQRT( QMIN )*SQRT( EMAX ) )
!<<<  tmp1 = sqrt(QMIN)
      if (QMINh.eq.0.0D+0.and.QMINl.eq.0.0D+0) then
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
      else
      if (QMINh.lt.0.0d+0.or.
     +   (QMINh.eq.0.0d+0.and.QMINl.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
        return
      endif
      t_x = 1.0D+0/sqrt(QMINh)
      t_ax = QMINh * t_x
!         call dd_mul(t_y1,t_y2,t_x,
!                     0.0,0.5,0.0)
          t_y1 = t_x * 0.5d+0
          t_u1 = 134217729.0 * t_x
          t_p1 = t_u1
     +             - (t_u1 - t_x)
          t_p2 = t_x - t_p1
          t_u1 = 134217729.0 * 0.5d+0
          t_r1 = t_u1
     +             - (t_u1 - 0.5d+0)
          t_r2 = 0.5d+0 - t_r1
          t_y2 = ((t_p1*t_r1
     +             - t_y1)
     +             + t_p1*t_r2
     +             + t_p2 * t_r1)
     +             + t_p2 * t_r2
          t_s1 = t_y1 + t_y2
          t_y2 = t_y2
     +             - (t_s1 - t_y1)
          t_y1 = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(t_z1,t_z2,t_ax,
!                     0.0,t_ax,0.0)
          t_z1 = t_ax * t_ax
          t_u1 = 134217729.0 * t_ax
          t_p1 = t_u1
     +             - (t_u1 - t_ax)
          t_p2 = t_ax - t_p1
          t_u1 = 134217729.0 * t_ax
          t_r1 = t_u1
     +             - (t_u1 - t_ax)
          t_r2 = t_ax - t_r1
          t_z2 = ((t_p1 * t_r1
     +             - t_z1)
     +             + t_p1 * t_r2
     +             + t_p2 * t_r1)
     +             + t_p2*t_r2
          t_s1 = t_z1 + t_z2
          t_z2 = t_z2
     +             - (t_s1 - t_z1)
          t_z1 = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(t_t2h,t_t2l,
!                           QMINh,QMINl,t_z1,t_z2)
          t_p1 = QMINh - t_z1
          t_p2 = t_p1 - QMINh
          t_t2l = (QMINh
     +             -(t_p1 - t_p2))
     +             -(t_z1 + t_p2)
          t_t2h = t_p1
          t_t2l = t_t2l
     +             + QMINl - t_z2
          t_s1 = t_t2h + t_t2l
          t_t2l = t_t2l
     +             - (t_s1 - t_t2h)
          t_t2h = t_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(t_t1h,t_t1l,
!                     t_t2h,t_t2l,
!                     t_y1,t_y2)
          t_t1h = t_t2h * t_y1
          t_u1 = 134217729.0 * t_t2h
          t_p1 = t_u1
     +             - (t_u1 - t_t2h)
          t_p2 = t_t2h - t_p1
          t_u1 = 134217729.0 * t_y1
          t_r1 = t_u1
     +             - (t_u1 - t_y1)
          t_r2 = t_y1 - t_r1
          t_t1l = ((t_p1*t_r1
     +             - t_t1h)
     +             + t_p1*t_r2
     +             + t_p2*t_r1)
     +             + t_p2*t_r2
          t_t1l = t_t1l
     +             + (t_t2h * t_y2
     +             +  t_t2l * t_y1)
          t_s1 = t_t1h + t_t1l
          t_t1l = t_t1l - (t_s1
     +              - t_t1h)
          t_t1h = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(t_t1h,t_t1l,
!                     t_ax,0.0)
          t_p1 = t_t1h + t_ax
          t_p2 = t_p1 - t_t1h
          t_q2 = (t_t1h
     +             - (t_p1 - t_p2))
     +             + (t_ax - t_p2)
          t_q1 = t_p1
          t_q2 = t_q2 + t_t1l
          t_t1h = t_q1 + t_q2
          t_t1l = t_q2
     +              - (t_t1h - t_q1)
          tmp1h = t_t1h
          tmp1l = t_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp1 = sqrt(QMIN)
!<<<  tmp2 = sqrt(EMAX)
      if (EMAXh.eq.0.0D+0.and.EMAXl.eq.0.0D+0) then
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
      else
      if (EMAXh.lt.0.0d+0.or.
     +   (EMAXh.eq.0.0d+0.and.EMAXl.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
        return
      endif
      t_x = 1.0D+0/sqrt(EMAXh)
      t_ax = EMAXh * t_x
!         call dd_mul(t_y1,t_y2,t_x,
!                     0.0,0.5,0.0)
          t_y1 = t_x * 0.5d+0
          t_u1 = 134217729.0 * t_x
          t_p1 = t_u1
     +             - (t_u1 - t_x)
          t_p2 = t_x - t_p1
          t_u1 = 134217729.0 * 0.5d+0
          t_r1 = t_u1
     +             - (t_u1 - 0.5d+0)
          t_r2 = 0.5d+0 - t_r1
          t_y2 = ((t_p1*t_r1
     +             - t_y1)
     +             + t_p1*t_r2
     +             + t_p2 * t_r1)
     +             + t_p2 * t_r2
          t_s1 = t_y1 + t_y2
          t_y2 = t_y2
     +             - (t_s1 - t_y1)
          t_y1 = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(t_z1,t_z2,t_ax,
!                     0.0,t_ax,0.0)
          t_z1 = t_ax * t_ax
          t_u1 = 134217729.0 * t_ax
          t_p1 = t_u1
     +             - (t_u1 - t_ax)
          t_p2 = t_ax - t_p1
          t_u1 = 134217729.0 * t_ax
          t_r1 = t_u1
     +             - (t_u1 - t_ax)
          t_r2 = t_ax - t_r1
          t_z2 = ((t_p1 * t_r1
     +             - t_z1)
     +             + t_p1 * t_r2
     +             + t_p2 * t_r1)
     +             + t_p2*t_r2
          t_s1 = t_z1 + t_z2
          t_z2 = t_z2
     +             - (t_s1 - t_z1)
          t_z1 = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(t_t2h,t_t2l,
!                           EMAXh,EMAXl,t_z1,t_z2)
          t_p1 = EMAXh - t_z1
          t_p2 = t_p1 - EMAXh
          t_t2l = (EMAXh
     +             -(t_p1 - t_p2))
     +             -(t_z1 + t_p2)
          t_t2h = t_p1
          t_t2l = t_t2l
     +             + EMAXl - t_z2
          t_s1 = t_t2h + t_t2l
          t_t2l = t_t2l
     +             - (t_s1 - t_t2h)
          t_t2h = t_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(t_t1h,t_t1l,
!                     t_t2h,t_t2l,
!                     t_y1,t_y2)
          t_t1h = t_t2h * t_y1
          t_u1 = 134217729.0 * t_t2h
          t_p1 = t_u1
     +             - (t_u1 - t_t2h)
          t_p2 = t_t2h - t_p1
          t_u1 = 134217729.0 * t_y1
          t_r1 = t_u1
     +             - (t_u1 - t_y1)
          t_r2 = t_y1 - t_r1
          t_t1l = ((t_p1*t_r1
     +             - t_t1h)
     +             + t_p1*t_r2
     +             + t_p2*t_r1)
     +             + t_p2*t_r2
          t_t1l = t_t1l
     +             + (t_t2h * t_y2
     +             +  t_t2l * t_y1)
          t_s1 = t_t1h + t_t1l
          t_t1l = t_t1l - (t_s1
     +              - t_t1h)
          t_t1h = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(t_t1h,t_t1l,
!                     t_ax,0.0)
          t_p1 = t_t1h + t_ax
          t_p2 = t_p1 - t_t1h
          t_q2 = (t_t1h
     +             - (t_p1 - t_p2))
     +             + (t_ax - t_p2)
          t_q1 = t_p1
          t_q2 = t_q2 + t_t1l
          t_t1h = t_q1 + t_q2
          t_t1l = t_q2
     +              - (t_t1h - t_q1)
          tmp2h = t_t1h
          tmp2l = t_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp2 = sqrt(EMAX)
!<<<  tmp3 = TWO * tmp1
      t_ch = TWOh * tmp1h
      t_u1 = 134217729.0 * TWOh
      t_p1 = t_u1
     +         - (t_u1 - TWOh)
      t_p2 = TWOh - t_p1
      t_u1 = 134217729.0 * tmp1h
      t_r1 = t_u1
     +         - (t_u1 - tmp1h)
      t_r2 = tmp1h - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (TWOh*tmp1l
     +         +  TWOl*tmp1h)
      t_s1 = t_ch + t_cl
      tmp3l = t_cl - (t_s1 - t_ch)
      tmp3h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = TWO * tmp1
!<<<  tmp1 = tmp3 * tmp2
      t_ch = tmp3h * tmp2h
      t_u1 = 134217729.0 * tmp3h
      t_p1 = t_u1
     +         - (t_u1 - tmp3h)
      t_p2 = tmp3h - t_p1
      t_u1 = 134217729.0 * tmp2h
      t_r1 = t_u1
     +         - (t_u1 - tmp2h)
      t_r2 = tmp2h - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (tmp3h*tmp2l
     +         +  tmp3l*tmp2h)
      t_s1 = t_ch + t_cl
      tmp1l = t_cl - (t_s1 - t_ch)
      tmp1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = tmp3 * tmp2
!<<<  tmp2 = QMIN - tmp1
      t_p1 = QMINh - tmp1h
      t_p2 = t_p1 - QMINh
      t_ch = t_p1
      t_cl = (QMINh - (t_p1 - t_p2))
     +         - (tmp1h + t_p2)
      t_cl = t_cl + QMINl - tmp1l
      t_s1 = t_ch + t_cl
      tmp2l = t_cl - (t_s1 - t_ch)
      tmp2h = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp2 = QMIN - tmp1
         IF(
!       (tmp2 .gt. 0.0)
     + ((tmp2h .gt. 0.0d+0) .or.
     +  ((tmp2h .eq. 0.0d+0) .and.
     +   (tmp2l .gt. 0.0d+0)))
     +     ) THEN
!<<<  DMIN = -tmp2
      DMINh = -tmp2h
      DMINl = -tmp2l
!>>>  DMIN = -tmp2
         ELSE
!<<<  DMIN = -ZERO
      DMINh = -ZEROh
      DMINl = -ZEROl
!>>>  DMIN = -ZERO
         END IF
*
*        Now I0:N0 is unreduced. 
*        PP = 0 for ping, PP = 1 for pong.
*        PP = 2 indicates that flipping was applied to the Z array and
*               and that the tests for deflation upon entry in DLASQ3 
*               should not be performed.
*
         NBIG = 100*( N0-I0+1 )
         DO 140 IWHILB = 1, NBIG
            IF( I0.GT.N0 ) 
     $         GO TO 150
*
*           While submatrix unfinished take a good dqds step.
*
!            CALL DLASQ3( I0, N0, Z, PP, DMIN, SIGMA, DESIG, QMAX, NFAIL,
!     $                   ITER, NDIV, IEEE, TTYPE, DMIN1, DMIN2, DN, DN1,
!     $                   DN2, G, TAU )
            CALL DDLASQ3( I0, N0, Zh, Zl, PP, DMINh, DMINl,
     $                    SIGMAh, SIGMAl, DESIGh, DESIGl, QMAXh, QMAXl,
     $                    NFAIL, ITER, NDIV, IEEE, TTYPE,
     $                    DMIN1h, DMIN1l, DMIN2h, DMIN2l, DNh, DNl,
     $                    DN1h, DN1l, DN2h, DN2l, Gh, Gl, TAUh, TAUl )
*
            PP = 1 - PP
*
*           When EMIN is very small check for splits.
*
            IF( PP.EQ.0 .AND. N0-I0.GE.3 ) THEN
!               IF( Z( 4*N0 ).LE.TOL2*QMAX .OR.
!     $             Z( 4*N0-1 ).LE.TOL2*SIGMA ) THEN
!<<<  tmp1 = TOL2 * QMAX
      t_ch = TOL2h * QMAXh
      t_u1 = 134217729.0 * TOL2h
      t_p1 = t_u1
     +         - (t_u1 - TOL2h)
      t_p2 = TOL2h - t_p1
      t_u1 = 134217729.0 * QMAXh
      t_r1 = t_u1
     +         - (t_u1 - QMAXh)
      t_r2 = QMAXh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (TOL2h*QMAXl
     +         +  TOL2l*QMAXh)
      t_s1 = t_ch + t_cl
      tmp1l = t_cl - (t_s1 - t_ch)
      tmp1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = TOL2 * QMAX
!<<<  tmp2 = TOL2 * SIGMA
      t_ch = TOL2h * SIGMAh
      t_u1 = 134217729.0 * TOL2h
      t_p1 = t_u1
     +         - (t_u1 - TOL2h)
      t_p2 = TOL2h - t_p1
      t_u1 = 134217729.0 * SIGMAh
      t_r1 = t_u1
     +         - (t_u1 - SIGMAh)
      t_r2 = SIGMAh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (TOL2h*SIGMAl
     +         +  TOL2l*SIGMAh)
      t_s1 = t_ch + t_cl
      tmp2l = t_cl - (t_s1 - t_ch)
      tmp2h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = TOL2 * SIGMA
               IF(
!       (Z(4*N0) .le. tmp1)
     + ((Zh(4*N0) .lt. tmp1h) .or.
     +  ((Zh(4*N0) .eq. tmp1h) .and.
     +   (Zl(4*N0) .le. tmp1l)))
     +            .OR.
!       (Z(4*N0-1) .le. tmp2)
     + ((Zh(4*N0-1) .lt. tmp2h) .or.
     +  ((Zh(4*N0-1) .eq. tmp2h) .and.
     +   (Zl(4*N0-1) .le. tmp2l)))
     +           ) THEN
                  SPLT = I0 - 1
!                  QMAX = Z( 4*I0-3 )
      QMAXh = Zh(4*I0-3)
      QMAXl = Zl(4*I0-3)
!                  EMIN = Z( 4*I0-1 )
      EMINh = Zh(4*I0-1)
      EMINl = Zl(4*I0-1)
!                  OLDEMN = Z( 4*I0 )
      OLDEMNh = Zh(4*I0)
      OLDEMNl = Zl(4*I0)
                  DO 130 I4 = 4*I0, 4*( N0-3 ), 4
!                     IF( Z( I4 ).LE.TOL2*Z( I4-3 ) .OR.
!     $                   Z( I4-1 ).LE.TOL2*SIGMA ) THEN
!<<<  tmp1 = TOL2 * Z(I4-3)
      t_ch = TOL2h * Zh(I4-3)
      t_u1 = 134217729.0 * TOL2h
      t_p1 = t_u1
     +         - (t_u1 - TOL2h)
      t_p2 = TOL2h - t_p1
      t_u1 = 134217729.0 * Zh(I4-3)
      t_r1 = t_u1
     +         - (t_u1 - Zh(I4-3))
      t_r2 = Zh(I4-3) - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (TOL2h*Zl(I4-3)
     +         +  TOL2l*Zh(I4-3))
      t_s1 = t_ch + t_cl
      tmp1l = t_cl - (t_s1 - t_ch)
      tmp1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = TOL2 * Z(I4-3)
!<<<  tmp2 = TOL2 * SIGMA
      t_ch = TOL2h * SIGMAh
      t_u1 = 134217729.0 * TOL2h
      t_p1 = t_u1
     +         - (t_u1 - TOL2h)
      t_p2 = TOL2h - t_p1
      t_u1 = 134217729.0 * SIGMAh
      t_r1 = t_u1
     +         - (t_u1 - SIGMAh)
      t_r2 = SIGMAh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (TOL2h*SIGMAl
     +         +  TOL2l*SIGMAh)
      t_s1 = t_ch + t_cl
      tmp2l = t_cl - (t_s1 - t_ch)
      tmp2h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = TOL2 * SIGMA
                     IF(
!       (Z(I4) .le. tmp1)
     + ((Zh(I4) .lt. tmp1h) .or.
     +  ((Zh(I4) .eq. tmp1h) .and.
     +   (Zl(I4) .le. tmp1l)))
     +                  .OR.
!       (Z(I4-1) .le. tmp2)
     + ((Zh(I4-1) .lt. tmp2h) .or.
     +  ((Zh(I4-1) .eq. tmp2h) .and.
     +   (Zl(I4-1) .le. tmp2l)))
     +                 ) THEN
!                        Z( I4-1 ) = -SIGMA
!<<<  Z(I4-1) = -SIGMA
      Zh(I4-1) = -SIGMAh
      Zl(I4-1) = -SIGMAl
!>>>  Z(I4-1) = -SIGMA
                        SPLT = I4 / 4
!                        QMAX = ZERO
      QMAXh = 0.0d+0
      QMAXl = 0.0d+0
!                        EMIN = Z( I4+3 )
      EMINh = Zh(I4+3)
      EMINl = Zl(I4+3)
!                        OLDEMN = Z( I4+4 )
      OLDEMNh = Zh(I4+4)
      OLDEMNl = Zl(I4+4)
                     ELSE
!                        QMAX = MAX( QMAX, Z( I4+1 ) )
                        IF(
!       (Z(I4+1) .gt. QMAX)
     + ((Zh(I4+1) .gt. QMAXh) .or.
     +  ((Zh(I4+1) .eq. QMAXh) .and.
     +   (Zl(I4+1) .gt. QMAXl)))
     +                    ) THEN
      QMAXh = Zh(I4+1)
      QMAXl = Zl(I4+1)
                        END IF
!                        EMIN = MIN( EMIN, Z( I4-1 ) )
                        IF(
!       (Z(I4-1) .lt. EMIN)
     + ((Zh(I4-1) .lt. EMINh) .or.
     +  ((Zh(I4-1) .eq. EMINh) .and.
     +   (Zl(I4-1) .lt. EMINl)))
     +                    ) THEN
      EMINh = Zh(I4-1)
      EMINl = Zl(I4-1)
                        END IF
!                        OLDEMN = MIN( OLDEMN, Z( I4 ) )
                        IF(
!       (Z(I4) .lt. OLDEMN)
     + ((Zh(I4) .lt. OLDEMNh) .or.
     +  ((Zh(I4) .eq. OLDEMNh) .and.
     +   (Zl(I4) .lt. OLDEMNl)))
     +                    ) THEN
      OLDEMNh = Zh(I4)
      OLDEMNl = Zl(I4)
                        END IF
                     END IF
  130             CONTINUE
!                  Z( 4*N0-1 ) = EMIN
      Zh(4*N0-1) = EMINh
      Zl(4*N0-1) = EMINl
!                  Z( 4*N0 ) = OLDEMN
      Zh(4*N0) = OLDEMNh
      Zl(4*N0) = OLDEMNl
                  I0 = SPLT + 1
               END IF
            END IF
*
  140    CONTINUE
*
         INFO = 2
*       
*        Maximum number of iterations exceeded, restore the shift 
*        SIGMA and place the new d's and e's in a qd array.
*        This might need to be done for several blocks
*
         I1 = I0
         N1 = N0
 145     CONTINUE
!         TEMPQ = Z( 4*I0-3 )
      TEMPQh = Zh(4*I0-3)
      TEMPQl = Zl(4*I0-3)
!         Z( 4*I0-3 ) = Z( 4*I0-3 ) + SIGMA
!<<<  tmp1 = Z(4*I0-3) + SIGMA
      t_p1 = Zh(4*I0-3) + SIGMAh
      t_p2 = t_p1 - Zh(4*I0-3)
      t_q1 = t_p1
      t_q2 = (Zh(4*I0-3) - (t_p1 - t_p2))
     +         + (SIGMAh - t_p2)
      t_q2 = t_q2 + Zl(4*I0-3) + SIGMAl
      tmp1h = t_q1 + t_q2
      tmp1l = t_q2 - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = Z(4*I0-3) + SIGMA
      Zh(4*I0-3) = tmp1h
      Zl(4*I0-3) = tmp1l
         DO K = I0+1, N0
!            TEMPE = Z( 4*K-5 )
      TEMPEh = Zh(4*K-5)
      TEMPEl = Zl(4*K-5)
!            Z( 4*K-5 ) = Z( 4*K-5 ) * (TEMPQ / Z( 4*K-7 ))
!<<<  tmp1 = TEMPQ / Z(4*K-7)
      t_q1 = TEMPQh / Zh(4*K-7)
      t_ch = Zh(4*K-7) * t_q1
      t_u1 = 134217729.0 * Zh(4*K-7)
      t_p1 = t_u1
     +         - (t_u1 - Zh(4*K-7))
      t_p2 = Zh(4*K-7) - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (Zl(4*K-7) * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = TEMPQh - t_ch
      t_r1 = t_s1 - TEMPQh
      t_s2 = (TEMPQh
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + TEMPQl
      t_q2 = (t_s1 + t_s2)
     +         / Zh(4*K-7)
      tmp1h = t_q1 + t_q2
      tmp1l = t_q2
     +      - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = TEMPQ / Z(4*K-7)
!<<<  tmp2 = Z(4*K-5) * tmp1
      t_ch = Zh(4*K-5) * tmp1h
      t_u1 = 134217729.0 * Zh(4*K-5)
      t_p1 = t_u1
     +         - (t_u1 - Zh(4*K-5))
      t_p2 = Zh(4*K-5) - t_p1
      t_u1 = 134217729.0 * tmp1h
      t_r1 = t_u1
     +         - (t_u1 - tmp1h)
      t_r2 = tmp1h - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (Zh(4*K-5)*tmp1l
     +         +  Zl(4*K-5)*tmp1h)
      t_s1 = t_ch + t_cl
      tmp2l = t_cl - (t_s1 - t_ch)
      tmp2h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = Z(4*K-5) * tmp1
      Zh(4*K-5) = tmp2h
      Zl(4*K-5) = tmp2l
!            TEMPQ = Z( 4*K-3 )
      TEMPQh = Zh(4*K-3)
      TEMPQl = Zl(4*K-3)
!            Z( 4*K-3 ) = Z( 4*K-3 ) + SIGMA + TEMPE - Z( 4*K-5 )
!<<<  tmp1 = Z(4*K-3) + SIGMA
      t_p1 = Zh(4*K-3) + SIGMAh
      t_p2 = t_p1 - Zh(4*K-3)
      t_q1 = t_p1
      t_q2 = (Zh(4*K-3) - (t_p1 - t_p2))
     +         + (SIGMAh - t_p2)
      t_q2 = t_q2 + Zl(4*K-3) + SIGMAl
      tmp1h = t_q1 + t_q2
      tmp1l = t_q2 - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = Z(4*K-3) + SIGMA
!<<<  tmp2 = tmp1 + TEMPE
      t_p1 = tmp1h + TEMPEh
      t_p2 = t_p1 - tmp1h
      t_q1 = t_p1
      t_q2 = (tmp1h - (t_p1 - t_p2))
     +         + (TEMPEh - t_p2)
      t_q2 = t_q2 + tmp1l + TEMPEl
      tmp2h = t_q1 + t_q2
      tmp2l = t_q2 - (tmp2h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = tmp1 + TEMPE
!<<<  tmp1 = tmp2 - Z(4*K-5)
      t_p1 = tmp2h - Zh(4*K-5)
      t_p2 = t_p1 - tmp2h
      t_ch = t_p1
      t_cl = (tmp2h - (t_p1 - t_p2))
     +         - (Zh(4*K-5) + t_p2)
      t_cl = t_cl + tmp2l - Zl(4*K-5)
      t_s1 = t_ch + t_cl
      tmp1l = t_cl - (t_s1 - t_ch)
      tmp1h = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp1 = tmp2 - Z(4*K-5)
      Zh(4*K-3) = tmp1h
      Zl(4*K-3) = tmp1l
         END DO
*
*        Prepare to do this on the previous block if there is one
*
         IF( I1.GT.1 ) THEN
            N1 = I1-1
!            DO WHILE( ( I1.GE.2 ) .AND. ( Z(4*I1-5).GE.ZERO ) )
            DO WHILE( ( I1.GE.2 ) .AND. (
!       (Z(4*I1-5) .ge. 0.0)
     + ((Zh(4*I1-5) .gt. 0.0d+0) .or.
     +  ((Zh(4*I1-5) .eq. 0.0d+0) .and.
     +   (Zl(4*I1-5) .ge. 0.0d+0)))
     +               ) )
               I1 = I1 - 1
            END DO
!            SIGMA = -Z(4*N1-1)
!<<<  SIGMA = -Z(4*N1-1)
      SIGMAh = -Zh(4*N1-1)
      SIGMAl = -Zl(4*N1-1)
!>>>  SIGMA = -Z(4*N1-1)
            GO TO 145
         END IF

         DO K = 1, N
!            Z( 2*K-1 ) = Z( 4*K-3 )
      Zh(2*K-1) = Zh(4*K-3)
      Zl(2*K-1) = Zl(4*K-3)
*
*        Only the block 1..N0 is unfinished.  The rest of the e's
*        must be essentially zero, although sometimes other data
*        has been stored in them.
*
            IF( K.LT.N0 ) THEN
!               Z( 2*K ) = Z( 4*K-1 )
      Zh(2*K) = Zh(4*K-1)
      Zl(2*K) = Zl(4*K-1)
            ELSE
!               Z( 2*K ) = 0
      Zh(2*K) = 0.0d+0
      Zl(2*K) = 0.0d+0
            END IF
         END DO
         RETURN
*
*        end IWHILB
*
  150    CONTINUE
*
  160 CONTINUE
*
      INFO = 3
      RETURN
*
*     end IWHILA   
*
  170 CONTINUE
*      
*     Move q's to the front.
*      
      DO 180 K = 2, N
!         Z( K ) = Z( 4*K-3 )
      Zh(K) = Zh(4*K-3)
      Zl(K) = Zl(4*K-3)
  180 CONTINUE
*      
*     Sort and compute sum of eigenvalues.
*
!      CALL DLASRT( 'D', N, Z, IINFO )
      CALL DDLASRT( 'D', N, Zh, Zl, IINFO )
*
!      E = ZERO
      Eh = 0.0d+0
      El = 0.0d+0
      DO 190 K = N, 1, -1
!         E = E + Z( K )
!<<<  tmp1 = E + Z(K)
      t_p1 = Eh + Zh(K)
      t_p2 = t_p1 - Eh
      t_q1 = t_p1
      t_q2 = (Eh - (t_p1 - t_p2))
     +         + (Zh(K) - t_p2)
      t_q2 = t_q2 + El + Zl(K)
      tmp1h = t_q1 + t_q2
      tmp1l = t_q2 - (tmp1h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = E + Z(K)
      Eh = tmp1h
      El = tmp1l
  190 CONTINUE
*
*     Store trace, sum(eigenvalues) and information on performance.
*
!      Z( 2*N+1 ) = TRACE 
      Zh(2*N+1) = TRACEh
      Zl(2*N+1) = TRACEl
!      Z( 2*N+2 ) = E
      Zh(2*N+2) = Eh
      Zl(2*N+2) = El
!      Z( 2*N+3 ) = DBLE( ITER )
      Zh(2*N+3) = DBLE(ITER)
      Zl(2*N+3) = 0.0d+0
!      Z( 2*N+4 ) = DBLE( NDIV ) / DBLE( N**2 )
      tmp1h = DBLE(NDIV)
      tmp1l = 0.0d+0
      tmp2h = DBLE(N**2)
      tmp2l = 0.0d+0
!<<<  Z(2*N+4) = tmp1 / tmp2
      t_q1 = tmp1h / tmp2h
      t_ch = tmp2h * t_q1
      t_u1 = 134217729.0 * tmp2h
      t_p1 = t_u1
     +         - (t_u1 - tmp2h)
      t_p2 = tmp2h - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (tmp2l * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = tmp1h - t_ch
      t_r1 = t_s1 - tmp1h
      t_s2 = (tmp1h
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + tmp1l
      t_q2 = (t_s1 + t_s2)
     +         / tmp2h
      Zh(2*N+4) = t_q1 + t_q2
      Zl(2*N+4) = t_q2
     +      - (Zh(2*N+4) - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  Z(2*N+4) = tmp1 / tmp2
!      Z( 2*N+5 ) = HUNDRD*NFAIL / DBLE( ITER )
      tmp1h = NFAIL
      tmp1l = 0.0d+0
!<<<  tmp2 = HUNDRD * tmp1
      t_ch = HUNDRDh * tmp1h
      t_u1 = 134217729.0 * HUNDRDh
      t_p1 = t_u1
     +         - (t_u1 - HUNDRDh)
      t_p2 = HUNDRDh - t_p1
      t_u1 = 134217729.0 * tmp1h
      t_r1 = t_u1
     +         - (t_u1 - tmp1h)
      t_r2 = tmp1h - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (HUNDRDh*tmp1l
     +         +  HUNDRDl*tmp1h)
      t_s1 = t_ch + t_cl
      tmp2l = t_cl - (t_s1 - t_ch)
      tmp2h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = HUNDRD * tmp1
      tmp1h = DBLE(ITER)
      tmp1l = 0.0d+0
!<<<  tmp3 = tmp2 / tmp1
      t_q1 = tmp2h / tmp1h
      t_ch = tmp1h * t_q1
      t_u1 = 134217729.0 * tmp1h
      t_p1 = t_u1
     +         - (t_u1 - tmp1h)
      t_p2 = tmp1h - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (tmp1l * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = tmp2h - t_ch
      t_r1 = t_s1 - tmp2h
      t_s2 = (tmp2h
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + tmp2l
      t_q2 = (t_s1 + t_s2)
     +         / tmp1h
      tmp3h = t_q1 + t_q2
      tmp3l = t_q2
     +      - (tmp3h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp3 = tmp2 / tmp1
      Zh(2*N+5) = tmp3h
      Zl(2*N+5) = tmp3l
      RETURN
*
*     End of DLASQ2
*
      END
