*> \brief \b DLASR
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLASR + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlasr.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlasr.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlasr.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLASR( SIDE, PIVOT, DIRECT, M, N, C, S, A, LDA )
* 
*       .. Scalar Arguments ..
*       CHARACTER          DIRECT, PIVOT, SIDE
*       INTEGER            LDA, M, N
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   A( LDA, * ), C( * ), S( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLASR applies a sequence of plane rotations to a real matrix A,
*> from either the left or the right.
*> 
*> When SIDE = 'L', the transformation takes the form
*> 
*>    A := P*A
*> 
*> and when SIDE = 'R', the transformation takes the form
*> 
*>    A := A*P**T
*> 
*> where P is an orthogonal matrix consisting of a sequence of z plane
*> rotations, with z = M when SIDE = 'L' and z = N when SIDE = 'R',
*> and P**T is the transpose of P.
*> 
*> When DIRECT = 'F' (Forward sequence), then
*> 
*>    P = P(z-1) * ... * P(2) * P(1)
*> 
*> and when DIRECT = 'B' (Backward sequence), then
*> 
*>    P = P(1) * P(2) * ... * P(z-1)
*> 
*> where P(k) is a plane rotation matrix defined by the 2-by-2 rotation
*> 
*>    R(k) = (  c(k)  s(k) )
*>         = ( -s(k)  c(k) ).
*> 
*> When PIVOT = 'V' (Variable pivot), the rotation is performed
*> for the plane (k,k+1), i.e., P(k) has the form
*> 
*>    P(k) = (  1                                            )
*>           (       ...                                     )
*>           (              1                                )
*>           (                   c(k)  s(k)                  )
*>           (                  -s(k)  c(k)                  )
*>           (                                1              )
*>           (                                     ...       )
*>           (                                            1  )
*> 
*> where R(k) appears as a rank-2 modification to the identity matrix in
*> rows and columns k and k+1.
*> 
*> When PIVOT = 'T' (Top pivot), the rotation is performed for the
*> plane (1,k+1), so P(k) has the form
*> 
*>    P(k) = (  c(k)                    s(k)                 )
*>           (         1                                     )
*>           (              ...                              )
*>           (                     1                         )
*>           ( -s(k)                    c(k)                 )
*>           (                                 1             )
*>           (                                      ...      )
*>           (                                             1 )
*> 
*> where R(k) appears in rows and columns 1 and k+1.
*> 
*> Similarly, when PIVOT = 'B' (Bottom pivot), the rotation is
*> performed for the plane (k,z), giving P(k) the form
*> 
*>    P(k) = ( 1                                             )
*>           (      ...                                      )
*>           (             1                                 )
*>           (                  c(k)                    s(k) )
*>           (                         1                     )
*>           (                              ...              )
*>           (                                     1         )
*>           (                 -s(k)                    c(k) )
*> 
*> where R(k) appears in rows and columns k and z.  The rotations are
*> performed without ever forming P(k) explicitly.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] SIDE
*> \verbatim
*>          SIDE is CHARACTER*1
*>          Specifies whether the plane rotation matrix P is applied to
*>          A on the left or the right.
*>          = 'L':  Left, compute A := P*A
*>          = 'R':  Right, compute A:= A*P**T
*> \endverbatim
*>
*> \param[in] PIVOT
*> \verbatim
*>          PIVOT is CHARACTER*1
*>          Specifies the plane for which P(k) is a plane rotation
*>          matrix.
*>          = 'V':  Variable pivot, the plane (k,k+1)
*>          = 'T':  Top pivot, the plane (1,k+1)
*>          = 'B':  Bottom pivot, the plane (k,z)
*> \endverbatim
*>
*> \param[in] DIRECT
*> \verbatim
*>          DIRECT is CHARACTER*1
*>          Specifies whether P is a forward or backward sequence of
*>          plane rotations.
*>          = 'F':  Forward, P = P(z-1)*...*P(2)*P(1)
*>          = 'B':  Backward, P = P(1)*P(2)*...*P(z-1)
*> \endverbatim
*>
*> \param[in] M
*> \verbatim
*>          M is INTEGER
*>          The number of rows of the matrix A.  If m <= 1, an immediate
*>          return is effected.
*> \endverbatim
*>
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The number of columns of the matrix A.  If n <= 1, an
*>          immediate return is effected.
*> \endverbatim
*>
*> \param[in] C
*> \verbatim
*>          C is DOUBLE PRECISION array, dimension
*>                  (M-1) if SIDE = 'L'
*>                  (N-1) if SIDE = 'R'
*>          The cosines c(k) of the plane rotations.
*> \endverbatim
*>
*> \param[in] S
*> \verbatim
*>          S is DOUBLE PRECISION array, dimension
*>                  (M-1) if SIDE = 'L'
*>                  (N-1) if SIDE = 'R'
*>          The sines s(k) of the plane rotations.  The 2-by-2 plane
*>          rotation part of the matrix P(k), R(k), has the form
*>          R(k) = (  c(k)  s(k) )
*>                 ( -s(k)  c(k) ).
*> \endverbatim
*>
*> \param[in,out] A
*> \verbatim
*>          A is DOUBLE PRECISION array, dimension (LDA,N)
*>          The M-by-N matrix A.  On exit, A is overwritten by P*A if
*>          SIDE = 'R' or by A*P**T if SIDE = 'L'.
*> \endverbatim
*>
*> \param[in] LDA
*> \verbatim
*>          LDA is INTEGER
*>          The leading dimension of the array A.  LDA >= max(1,M).
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERauxiliary
*
*  =====================================================================
!      SUBROUTINE DLASR( SIDE, PIVOT, DIRECT, M, N, C, S, A, LDA )
      SUBROUTINE DDLASR( SIDE, PIVOT, DIRECT, M, N, 
     &                 Ch,Cl, Sh,Sl, Ah,Al, LDA )
*
*  -- LAPACK auxiliary routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      CHARACTER          DIRECT, PIVOT, SIDE
      INTEGER            LDA, M, N
*     ..
*     .. Array Arguments ..
!      DOUBLE PRECISION   A( LDA, * ), C( * ), S( * )
      double precision Ah(LDA,*), Al(LDA,*)
      double precision Ch(*), Cl(*)
      double precision Sh(*), Sl(*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   ONE, ZERO
!      PARAMETER          ( ONE = 1.0D+0, ZERO = 0.0D+0 )
      double precision ONEh, ONEl
      double precision ZEROh, ZEROl

      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
      double precision tmp7h, tmp7l
      double precision tmp8h, tmp8l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l

*     ..
*     .. Local Scalars ..
      INTEGER            I, INFO, J
!      DOUBLE PRECISION   CTEMP, STEMP, TEMP
      double precision CTEMPh, CTEMPl
      double precision STEMPh, STEMPl
      double precision TEMPh, TEMPl
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      EXTERNAL           LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL           XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX
*     ..
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
*     .. Executable Statements ..
*
*     Test the input parameters
*
      INFO = 0
      IF( .NOT.( LSAME( SIDE, 'L' ) .OR. LSAME( SIDE, 'R' ) ) ) THEN
         INFO = 1
      ELSE IF( .NOT.( LSAME( PIVOT, 'V' ) .OR. LSAME( PIVOT,
     $         'T' ) .OR. LSAME( PIVOT, 'B' ) ) ) THEN
         INFO = 2
      ELSE IF( .NOT.( LSAME( DIRECT, 'F' ) .OR. LSAME( DIRECT, 'B' ) ) )
     $          THEN
         INFO = 3
      ELSE IF( M.LT.0 ) THEN
         INFO = 4
      ELSE IF( N.LT.0 ) THEN
         INFO = 5
      ELSE IF( LDA.LT.MAX( 1, M ) ) THEN
         INFO = 9
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DLASR ', INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( ( M.EQ.0 ) .OR. ( N.EQ.0 ) )
     $   RETURN
      IF( LSAME( SIDE, 'L' ) ) THEN
*
*        Form  P * A
*
         IF( LSAME( PIVOT, 'V' ) ) THEN
            IF( LSAME( DIRECT, 'F' ) ) THEN
               DO 20 J = 1, M - 1
!                  CTEMP = C( J )
!                  STEMP = S( J )
                    CTEMPh = Ch(J)
                    CTEMPl = Cl(J)
                    STEMPh = Sh(J)
                    STEMPl = Sl(J)
                  IF(( 
!                    ( CTEMP.NE.ONE ) 
!                     (CTEMP .ne. ONE)
     +               ((CTEMPh .ne. ONEh) .or.
     +                (CTEMPl .ne. ONEl))
     &               ).OR.( 
!                    ( STEMP.NE.ZERO )
!                     (STEMP .ne. ZERO)
     +               ((STEMPh .ne. ZEROh) .or.
     +                (STEMPl .ne. ZEROl))
     &               )) THEN
                     DO 10 I = 1, N
!                        TEMP = A( J+1, I )
                          TEMPh = Ah(J+1,I)
                          TEMPl = Al(J+1,I)
!                        A( J+1, I ) = CTEMP*TEMP - STEMP*A( J, I )
!<<<                      tmp1 = CTEMP * TEMP
                          tmp1h = CTEMPh * TEMPh
                          T_u1 = 134217729.0 * CTEMPh
                          T_p1 = T_u1
     +                             - (T_u1 - CTEMPh)
                          T_p2 = CTEMPh - T_p1
                          T_u1 = 134217729.0 * TEMPh
                          T_r1 = T_u1
     +                             - (T_u1 - TEMPh)
                          T_r2 = TEMPh - T_r1
                          tmp1l = ((T_p1*T_r1
     +                            - tmp1h)
     +                            + T_p1 * T_r2
     +                            + T_p2 * T_r1)
     +                            + T_p2 * T_r2
                          tmp1l = tmp1l
     +                          + (CTEMPh*TEMPl
     +                          + CTEMPl*TEMPh)
                          T_s1 = tmp1h + tmp1l
                          tmp1l = tmp1l - (T_s1 - tmp1h)
                          tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      tmp1 = CTEMP * TEMP
!<<<                      tmp2 = STEMP * A(J,I)
                          tmp2h = STEMPh * Ah(J,I)
                          T_u1 = 134217729.0 * STEMPh
                          T_p1 = T_u1
     +                             - (T_u1 - STEMPh)
                          T_p2 = STEMPh - T_p1
                          T_u1 = 134217729.0 * Ah(J,I)
                          T_r1 = T_u1
     +                             - (T_u1 - Ah(J,I))
                          T_r2 = Ah(J,I) - T_r1
                          tmp2l = ((T_p1*T_r1
     +                            - tmp2h)
     +                            + T_p1 * T_r2
     +                            + T_p2 * T_r1)
     +                            + T_p2 * T_r2
                          tmp2l = tmp2l
     +                          + (STEMPh*Al(J,I)
     +                          + STEMPl*Ah(J,I))
                          T_s1 = tmp2h + tmp2l
                          tmp2l = tmp2l - (T_s1 - tmp2h)
                          tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      tmp2 = STEMP * A(J,I)
!<<<                      A(J+1,I) = tmp1 - tmp2
                          T_p1 = tmp1h - tmp2h
                          T_p2 = T_p1 - tmp1h
                          Ah(J+1,I) = T_p1
                          Al(J+1,I) = (tmp1h - (T_p1 - T_p2))
     +                             - (tmp2h + T_p2)
                          Al(J+1,I) = Al(J+1,I) + tmp1l - tmp2l
                          T_s1 = Ah(J+1,I) + Al(J+1,I)
                          Al(J+1,I) = Al(J+1,I) - (T_s1 - Ah(J+1,I))
                          Ah(J+1,I) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>                      A(J+1,I) = tmp1 - tmp2
!                        A( J, I ) = STEMP*TEMP + CTEMP*A( J, I )
!<<<                      tmp3 = STEMP * TEMP
                          tmp3h = STEMPh * TEMPh
                          T_u1 = 134217729.0 * STEMPh
                          T_p1 = T_u1
     +                             - (T_u1 - STEMPh)
                          T_p2 = STEMPh - T_p1
                          T_u1 = 134217729.0 * TEMPh
                          T_r1 = T_u1
     +                             - (T_u1 - TEMPh)
                          T_r2 = TEMPh - T_r1
                          tmp3l = ((T_p1*T_r1
     +                            - tmp3h)
     +                            + T_p1 * T_r2
     +                            + T_p2 * T_r1)
     +                            + T_p2 * T_r2
                          tmp3l = tmp3l
     +                          + (STEMPh*TEMPl
     +                          + STEMPl*TEMPh)
                          T_s1 = tmp3h + tmp3l
                          tmp3l = tmp3l - (T_s1 - tmp3h)
                          tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      tmp3 = STEMP * TEMP
!<<<                      tmp4 = CTEMP * A(J,I)
                          tmp4h = CTEMPh * Ah(J,I)
                          T_u1 = 134217729.0 * CTEMPh
                          T_p1 = T_u1
     +                             - (T_u1 - CTEMPh)
                          T_p2 = CTEMPh - T_p1
                          T_u1 = 134217729.0 * Ah(J,I)
                          T_r1 = T_u1
     +                             - (T_u1 - Ah(J,I))
                          T_r2 = Ah(J,I) - T_r1
                          tmp4l = ((T_p1*T_r1
     +                            - tmp4h)
     +                            + T_p1 * T_r2
     +                            + T_p2 * T_r1)
     +                            + T_p2 * T_r2
                          tmp4l = tmp4l
     +                          + (CTEMPh*Al(J,I)
     +                          + CTEMPl*Ah(J,I))
                          T_s1 = tmp4h + tmp4l
                          tmp4l = tmp4l - (T_s1 - tmp4h)
                          tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      tmp4 = CTEMP * A(J,I)
!<<<                      A(J,I) = tmp3 + tmp4
                          T_p1 = tmp3h + tmp4h
                          T_p2 = T_p1 - tmp3h
                          Ah(J,I) = T_p1
                          Al(J,I) = (tmp3h - (T_p1 - T_p2))
     +                          + (tmp4h - T_p2)
                          Al(J,I) = Al(J,I) + tmp3l + tmp4l
                          T_s1 = Ah(J,I) + Al(J,I)
                          Al(J,I) = Al(J,I) - (T_s1 - Ah(J,I))
                          Ah(J,I) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                      A(J,I) = tmp3 + tmp4
   10                CONTINUE
                  END IF
   20          CONTINUE
            ELSE IF( LSAME( DIRECT, 'B' ) ) THEN
               DO 40 J = M - 1, 1, -1
!                  CTEMP = C( J )
!                  STEMP = S( J )
                    CTEMPh = Ch(J)
                    CTEMPl = Cl(J)
                    STEMPh = Sh(J)
                    STEMPl = Sl(J)
                  IF((
!                    ( CTEMP.NE.ONE ) 
!                     (CTEMP .ne. ONE)
     +               ((CTEMPh .ne. ONEh) .or.
     +                (CTEMPl .ne. ONEl))
     &                ).OR.( 
!                     ( STEMP.NE.ZERO ) 
!                     (STEMP .ne. ZERO)
     +               ((STEMPh .ne. ZEROh) .or.
     +                (STEMPl .ne. ZEROl))
     &               )) THEN
                     DO 30 I = 1, N
!                        TEMP = A( J+1, I )
                    TEMPh = Ah(J+1,I)
                    TEMPl = Al(J+1,I)
!                        A( J+1, I ) = CTEMP*TEMP - STEMP*A( J, I )
!<<<                tmp1 = CTEMP * TEMP
                    tmp1h = CTEMPh * TEMPh
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp1l = ((T_p1*T_r1
     +                      - tmp1h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp1l = tmp1l
     +                    + (CTEMPh*TEMPl
     +                    + CTEMPl*TEMPh)
                    T_s1 = tmp1h + tmp1l
                    tmp1l = tmp1l - (T_s1 - tmp1h)
                    tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp1 = CTEMP * TEMP
!<<<                tmp2 = STEMP * A(J,I)
                    tmp2h = STEMPh * Ah(J,I)
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(J,I)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(J,I))
                    T_r2 = Ah(J,I) - T_r1
                    tmp2l = ((T_p1*T_r1
     +                      - tmp2h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp2l = tmp2l
     +                    + (STEMPh*Al(J,I)
     +                    + STEMPl*Ah(J,I))
                    T_s1 = tmp2h + tmp2l
                    tmp2l = tmp2l - (T_s1 - tmp2h)
                    tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp2 = STEMP * A(J,I)
!<<<                A(J+1,I) = tmp1 - tmp2
                    T_p1 = tmp1h - tmp2h
                    T_p2 = T_p1 - tmp1h
                    Ah(J+1,I) = T_p1
                    Al(J+1,I) = (tmp1h - (T_p1 - T_p2))
     +                       - (tmp2h + T_p2)
                    Al(J+1,I) = Al(J+1,I) + tmp1l - tmp2l
                    T_s1 = Ah(J+1,I) + Al(J+1,I)
                    Al(J+1,I) = Al(J+1,I) - (T_s1 - Ah(J+1,I))
                    Ah(J+1,I) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>                A(J+1,I) = tmp1 - tmp2
!                        A( J, I ) = STEMP*TEMP + CTEMP*A( J, I )
!<<<                tmp3 = STEMP * TEMP
                    tmp3h = STEMPh * TEMPh
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp3l = ((T_p1*T_r1
     +                      - tmp3h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp3l = tmp3l
     +                    + (STEMPh*TEMPl
     +                    + STEMPl*TEMPh)
                    T_s1 = tmp3h + tmp3l
                    tmp3l = tmp3l - (T_s1 - tmp3h)
                    tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp3 = STEMP * TEMP
!<<<                tmp4 = CTEMP * A(J,I)
                    tmp4h = CTEMPh * Ah(J,I)
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(J,I)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(J,I))
                    T_r2 = Ah(J,I) - T_r1
                    tmp4l = ((T_p1*T_r1
     +                      - tmp4h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp4l = tmp4l
     +                    + (CTEMPh*Al(J,I)
     +                    + CTEMPl*Ah(J,I))
                    T_s1 = tmp4h + tmp4l
                    tmp4l = tmp4l - (T_s1 - tmp4h)
                    tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp4 = CTEMP * A(J,I)
!<<<                A(J,I) = tmp3 + tmp4
                    T_p1 = tmp3h + tmp4h
                    T_p2 = T_p1 - tmp3h
                    Ah(J,I) = T_p1
                    Al(J,I) = (tmp3h - (T_p1 - T_p2))
     +                    + (tmp4h - T_p2)
                    Al(J,I) = Al(J,I) + tmp3l + tmp4l
                    T_s1 = Ah(J,I) + Al(J,I)
                    Al(J,I) = Al(J,I) - (T_s1 - Ah(J,I))
                    Ah(J,I) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                A(J,I) = tmp3 + tmp4
   30                CONTINUE
                  END IF
   40          CONTINUE
            END IF
         ELSE IF( LSAME( PIVOT, 'T' ) ) THEN
            IF( LSAME( DIRECT, 'F' ) ) THEN
               DO 60 J = 2, M
!                  CTEMP = C( J-1 )
!                  STEMP = S( J-1 )
                    CTEMPh = Ch(J-1)
                    CTEMPl = Cl(J-1)
                    STEMPh = Sh(J-1)
                    STEMPl = Sl(J-1)
                  IF( (
!                    ( CTEMP.NE.ONE ) 
!                     (CTEMP .ne. ONE)
     +               ((CTEMPh .ne. ONEh) .or.
     +                (CTEMPl .ne. ONEl))
     &              ).OR.(
!                    ( STEMP.NE.ZERO )
!                     (STEMP .ne. ZERO)
     +               ((STEMPh .ne. ZEROh) .or.
     +                (STEMPl .ne. ZEROl))
     &               )) THEN
                     DO 50 I = 1, N
!                        TEMP = A( J, I )
                    TEMPh = Ah(J,I)
                    TEMPl = Al(J,I)
!                        A( J, I ) = CTEMP*TEMP - STEMP*A( 1, I )
!<<<                tmp1 = CTEMP * TEMP
                    tmp1h = CTEMPh * TEMPh
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp1l = ((T_p1*T_r1
     +                      - tmp1h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp1l = tmp1l
     +                    + (CTEMPh*TEMPl
     +                    + CTEMPl*TEMPh)
                    T_s1 = tmp1h + tmp1l
                    tmp1l = tmp1l - (T_s1 - tmp1h)
                    tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp1 = CTEMP * TEMP
!<<<                tmp2 = STEMP * A(1,I)
                    tmp2h = STEMPh * Ah(1,I)
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(1,I)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(1,I))
                    T_r2 = Ah(1,I) - T_r1
                    tmp2l = ((T_p1*T_r1
     +                      - tmp2h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp2l = tmp2l
     +                    + (STEMPh*Al(1,I)
     +                    + STEMPl*Ah(1,I))
                    T_s1 = tmp2h + tmp2l
                    tmp2l = tmp2l - (T_s1 - tmp2h)
                    tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp2 = STEMP * A(1,I)
!<<<                A(J,I) = tmp1 - tmp2
                    T_p1 = tmp1h - tmp2h
                    T_p2 = T_p1 - tmp1h
                    Ah(J,I) = T_p1
                    Al(J,I) = (tmp1h - (T_p1 - T_p2))
     +                       - (tmp2h + T_p2)
                    Al(J,I) = Al(J,I) + tmp1l - tmp2l
                    T_s1 = Ah(J,I) + Al(J,I)
                    Al(J,I) = Al(J,I) - (T_s1 - Ah(J,I))
                    Ah(J,I) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>                A(J,I) = tmp1 - tmp2
!                        A( 1, I ) = STEMP*TEMP + CTEMP*A( 1, I )
!<<<                tmp3 = STEMP * TEMP
                    tmp3h = STEMPh * TEMPh
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp3l = ((T_p1*T_r1
     +                      - tmp3h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp3l = tmp3l
     +                    + (STEMPh*TEMPl
     +                    + STEMPl*TEMPh)
                    T_s1 = tmp3h + tmp3l
                    tmp3l = tmp3l - (T_s1 - tmp3h)
                    tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp3 = STEMP * TEMP
!<<<                tmp4 = CTEMP * A(1,I)
                    tmp4h = CTEMPh * Ah(1,I)
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(1,I)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(1,I))
                    T_r2 = Ah(1,I) - T_r1
                    tmp4l = ((T_p1*T_r1
     +                      - tmp4h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp4l = tmp4l
     +                    + (CTEMPh*Al(1,I)
     +                    + CTEMPl*Ah(1,I))
                    T_s1 = tmp4h + tmp4l
                    tmp4l = tmp4l - (T_s1 - tmp4h)
                    tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp4 = CTEMP * A(1,I)
!<<<                A(1,I) = tmp3 + tmp4
                    T_p1 = tmp3h + tmp4h
                    T_p2 = T_p1 - tmp3h
                    Ah(1,I) = T_p1
                    Al(1,I) = (tmp3h - (T_p1 - T_p2))
     +                    + (tmp4h - T_p2)
                    Al(1,I) = Al(1,I) + tmp3l + tmp4l
                    T_s1 = Ah(1,I) + Al(1,I)
                    Al(1,I) = Al(1,I) - (T_s1 - Ah(1,I))
                    Ah(1,I) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                A(1,I) = tmp3 + tmp4
   50                CONTINUE
                  END IF
   60          CONTINUE
            ELSE IF( LSAME( DIRECT, 'B' ) ) THEN
               DO 80 J = M, 2, -1
!                  CTEMP = C( J-1 )
!                  STEMP = S( J-1 )
                    CTEMPh = Ch(J-1)
                    CTEMPl = Cl(J-1)
                    STEMPh = Sh(J-1)
                    STEMPl = Sl(J-1)
                  IF( (
!                    ( CTEMP.NE.ONE ) 
!                     (CTEMP .ne. ONE)
     +               ((CTEMPh .ne. ONEh) .or.
     +                (CTEMPl .ne. ONEl))
     &               ).OR.( 
!                    ( STEMP.NE.ZERO )
!                     (STEMP .ne. ZERO)
     +               ((STEMPh .ne. ZEROh) .or.
     +                (STEMPl .ne. ZEROl))
     &               )) THEN
                     DO 70 I = 1, N
!                        TEMP = A( J, I )
                    TEMPh = Ah(J,I)
                    TEMPl = Al(J,I)
!                        A( J, I ) = CTEMP*TEMP - STEMP*A( 1, I )
!<<<                tmp1 = CTEMP * TEMP
                    tmp1h = CTEMPh * TEMPh
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp1l = ((T_p1*T_r1
     +                      - tmp1h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp1l = tmp1l
     +                    + (CTEMPh*TEMPl
     +                    + CTEMPl*TEMPh)
                    T_s1 = tmp1h + tmp1l
                    tmp1l = tmp1l - (T_s1 - tmp1h)
                    tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp1 = CTEMP * TEMP
!<<<                tmp2 = STEMP * A(1,I)
                    tmp2h = STEMPh * Ah(1,I)
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(1,I)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(1,I))
                    T_r2 = Ah(1,I) - T_r1
                    tmp2l = ((T_p1*T_r1
     +                      - tmp2h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp2l = tmp2l
     +                    + (STEMPh*Al(1,I)
     +                    + STEMPl*Ah(1,I))
                    T_s1 = tmp2h + tmp2l
                    tmp2l = tmp2l - (T_s1 - tmp2h)
                    tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp2 = STEMP * A(1,I)
!<<<                A(J,I) = tmp1 - tmp2
                    T_p1 = tmp1h - tmp2h
                    T_p2 = T_p1 - tmp1h
                    Ah(J,I) = T_p1
                    Al(J,I) = (tmp1h - (T_p1 - T_p2))
     +                       - (tmp2h + T_p2)
                    Al(J,I) = Al(J,I) + tmp1l - tmp2l
                    T_s1 = Ah(J,I) + Al(J,I)
                    Al(J,I) = Al(J,I) - (T_s1 - Ah(J,I))
                    Ah(J,I) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>                A(J,I) = tmp1 - tmp2
!                        A( 1, I ) = STEMP*TEMP + CTEMP*A( 1, I )
!<<<                tmp3 = STEMP * TEMP
                    tmp3h = STEMPh * TEMPh
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp3l = ((T_p1*T_r1
     +                      - tmp3h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp3l = tmp3l
     +                    + (STEMPh*TEMPl
     +                    + STEMPl*TEMPh)
                    T_s1 = tmp3h + tmp3l
                    tmp3l = tmp3l - (T_s1 - tmp3h)
                    tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp3 = STEMP * TEMP
!<<<                tmp4 = CTEMP * A(1,I)
                    tmp4h = CTEMPh * Ah(1,I)
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(1,I)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(1,I))
                    T_r2 = Ah(1,I) - T_r1
                    tmp4l = ((T_p1*T_r1
     +                      - tmp4h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp4l = tmp4l
     +                    + (CTEMPh*Al(1,I)
     +                    + CTEMPl*Ah(1,I))
                    T_s1 = tmp4h + tmp4l
                    tmp4l = tmp4l - (T_s1 - tmp4h)
                    tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp4 = CTEMP * A(1,I)
!<<<                A(1,I) = tmp3 + tmp4
                    T_p1 = tmp3h + tmp4h
                    T_p2 = T_p1 - tmp3h
                    Ah(1,I) = T_p1
                    Al(1,I) = (tmp3h - (T_p1 - T_p2))
     +                    + (tmp4h - T_p2)
                    Al(1,I) = Al(1,I) + tmp3l + tmp4l
                    T_s1 = Ah(1,I) + Al(1,I)
                    Al(1,I) = Al(1,I) - (T_s1 - Ah(1,I))
                    Ah(1,I) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                A(1,I) = tmp3 + tmp4
   70                CONTINUE
                  END IF
   80          CONTINUE
            END IF
         ELSE IF( LSAME( PIVOT, 'B' ) ) THEN
            IF( LSAME( DIRECT, 'F' ) ) THEN
               DO 100 J = 1, M - 1
!                  CTEMP = C( J )
!                  STEMP = S( J )
                    CTEMPh = Ch(J)
                    CTEMPl = Cl(J)
                    STEMPh = Sh(J)
                    STEMPl = Sl(J)
                  IF((
!                    ( CTEMP.NE.ONE ) 
!                     (CTEMP .ne. ONE)
     +               ((CTEMPh .ne. ONEh) .or.
     +                (CTEMPl .ne. ONEl))
     &               ).OR.(
!                    ( STEMP.NE.ZERO ) 
!                     (STEMP .ne. ZERO)
     +               ((STEMPh .ne. ZEROh) .or.
     +                (STEMPl .ne. ZEROl))
     &               )) THEN
                     DO 90 I = 1, N
!                        TEMP = A( J, I )
                    TEMPh = Ah(J,I)
                    TEMPl = Al(J,I)
!                        A( J, I ) = STEMP*A( M, I ) + CTEMP*TEMP
!<<<                tmp1 = STEMP * A(M,I)
                    tmp1h = STEMPh * Ah(M,I)
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(M,I)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(M,I))
                    T_r2 = Ah(M,I) - T_r1
                    tmp1l = ((T_p1*T_r1
     +                      - tmp1h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp1l = tmp1l
     +                    + (STEMPh*Al(M,I)
     +                    + STEMPl*Ah(M,I))
                    T_s1 = tmp1h + tmp1l
                    tmp1l = tmp1l - (T_s1 - tmp1h)
                    tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp1 = STEMP * A(M,I)
!<<<                tmp2 = CTEMP * TEMP
                    tmp2h = CTEMPh * TEMPh
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp2l = ((T_p1*T_r1
     +                      - tmp2h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp2l = tmp2l
     +                    + (CTEMPh*TEMPl
     +                    + CTEMPl*TEMPh)
                    T_s1 = tmp2h + tmp2l
                    tmp2l = tmp2l - (T_s1 - tmp2h)
                    tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp2 = CTEMP * TEMP
!<<<                A(J,I) = tmp1 + tmp2
                    T_p1 = tmp1h + tmp2h
                    T_p2 = T_p1 - tmp1h
                    Ah(J,I) = T_p1
                    Al(J,I) = (tmp1h - (T_p1 - T_p2))
     +                    + (tmp2h - T_p2)
                    Al(J,I) = Al(J,I) + tmp1l + tmp2l
                    T_s1 = Ah(J,I) + Al(J,I)
                    Al(J,I) = Al(J,I) - (T_s1 - Ah(J,I))
                    Ah(J,I) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                A(J,I) = tmp1 + tmp2
!                        A( M, I ) = CTEMP*A( M, I ) - STEMP*TEMP
!<<<                tmp3 = CTEMP * A(M,I)
                    tmp3h = CTEMPh * Ah(M,I)
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(M,I)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(M,I))
                    T_r2 = Ah(M,I) - T_r1
                    tmp3l = ((T_p1*T_r1
     +                      - tmp3h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp3l = tmp3l
     +                    + (CTEMPh*Al(M,I)
     +                    + CTEMPl*Ah(M,I))
                    T_s1 = tmp3h + tmp3l
                    tmp3l = tmp3l - (T_s1 - tmp3h)
                    tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp3 = CTEMP * A(M,I)
!<<<                tmp4 = STEMP * TEMP
                    tmp4h = STEMPh * TEMPh
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp4l = ((T_p1*T_r1
     +                      - tmp4h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp4l = tmp4l
     +                    + (STEMPh*TEMPl
     +                    + STEMPl*TEMPh)
                    T_s1 = tmp4h + tmp4l
                    tmp4l = tmp4l - (T_s1 - tmp4h)
                    tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp4 = STEMP * TEMP
!<<<                A(M,I) = tmp3 - tmp4
                    T_p1 = tmp3h - tmp4h
                    T_p2 = T_p1 - tmp3h
                    Ah(M,I) = T_p1
                    Al(M,I) = (tmp3h - (T_p1 - T_p2))
     +                       - (tmp4h + T_p2)
                    Al(M,I) = Al(M,I) + tmp3l - tmp4l
                    T_s1 = Ah(M,I) + Al(M,I)
                    Al(M,I) = Al(M,I) - (T_s1 - Ah(M,I))
                    Ah(M,I) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>                A(M,I) = tmp3 - tmp4
   90                CONTINUE
                  END IF
  100          CONTINUE
            ELSE IF( LSAME( DIRECT, 'B' ) ) THEN
               DO 120 J = M - 1, 1, -1
!                  CTEMP = C( J )
!                  STEMP = S( J )
                    CTEMPh = Ch(J)
                    CTEMPl = Cl(J)
                    STEMPh = Sh(J)
                    STEMPl = Sl(J)
                  IF(( 
!                    ( CTEMP.NE.ONE ) 
!                     (CTEMP .ne. ONE)
     +               ((CTEMPh .ne. ONEh) .or.
     +                (CTEMPl .ne. ONEl))
     &               ).OR.(
!                    ( STEMP.NE.ZERO )
!                     (STEMP .ne. ZERO)
     +               ((STEMPh .ne. ZEROh) .or.
     +                (STEMPl .ne. ZEROl))
     &               )) THEN
                     DO 110 I = 1, N
!                        TEMP = A( J, I )
                    TEMPh = Ah(J,I)
                    TEMPl = Al(J,I)
!                        A( J, I ) = STEMP*A( M, I ) + CTEMP*TEMP
!<<<                tmp1 = STEMP * A(M,I)
                    tmp1h = STEMPh * Ah(M,I)
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(M,I)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(M,I))
                    T_r2 = Ah(M,I) - T_r1
                    tmp1l = ((T_p1*T_r1
     +                      - tmp1h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp1l = tmp1l
     +                    + (STEMPh*Al(M,I)
     +                    + STEMPl*Ah(M,I))
                    T_s1 = tmp1h + tmp1l
                    tmp1l = tmp1l - (T_s1 - tmp1h)
                    tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp1 = STEMP * A(M,I)
!<<<                tmp2 = CTEMP * TEMP
                    tmp2h = CTEMPh * TEMPh
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp2l = ((T_p1*T_r1
     +                      - tmp2h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp2l = tmp2l
     +                    + (CTEMPh*TEMPl
     +                    + CTEMPl*TEMPh)
                    T_s1 = tmp2h + tmp2l
                    tmp2l = tmp2l - (T_s1 - tmp2h)
                    tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp2 = CTEMP * TEMP
!<<<                A(J,I) = tmp1 + tmp2
                    T_p1 = tmp1h + tmp2h
                    T_p2 = T_p1 - tmp1h
                    Ah(J,I) = T_p1
                    Al(J,I) = (tmp1h - (T_p1 - T_p2))
     +                    + (tmp2h - T_p2)
                    Al(J,I) = Al(J,I) + tmp1l + tmp2l
                    T_s1 = Ah(J,I) + Al(J,I)
                    Al(J,I) = Al(J,I) - (T_s1 - Ah(J,I))
                    Ah(J,I) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                A(J,I) = tmp1 + tmp2
!                        A( M, I ) = CTEMP*A( M, I ) - STEMP*TEMP
!<<<                tmp3 = CTEMP * A(M,I)
                    tmp3h = CTEMPh * Ah(M,I)
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(M,I)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(M,I))
                    T_r2 = Ah(M,I) - T_r1
                    tmp3l = ((T_p1*T_r1
     +                      - tmp3h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp3l = tmp3l
     +                    + (CTEMPh*Al(M,I)
     +                    + CTEMPl*Ah(M,I))
                    T_s1 = tmp3h + tmp3l
                    tmp3l = tmp3l - (T_s1 - tmp3h)
                    tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp3 = CTEMP * A(M,I)
!<<<                tmp4 = STEMP * TEMP
                    tmp4h = STEMPh * TEMPh
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp4l = ((T_p1*T_r1
     +                      - tmp4h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp4l = tmp4l
     +                    + (STEMPh*TEMPl
     +                    + STEMPl*TEMPh)
                    T_s1 = tmp4h + tmp4l
                    tmp4l = tmp4l - (T_s1 - tmp4h)
                    tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp4 = STEMP * TEMP
!<<<                A(M,I) = tmp3 - tmp4
                    T_p1 = tmp3h - tmp4h
                    T_p2 = T_p1 - tmp3h
                    Ah(M,I) = T_p1
                    Al(M,I) = (tmp3h - (T_p1 - T_p2))
     +                       - (tmp4h + T_p2)
                    Al(M,I) = Al(M,I) + tmp3l - tmp4l
                    T_s1 = Ah(M,I) + Al(M,I)
                    Al(M,I) = Al(M,I) - (T_s1 - Ah(M,I))
                    Ah(M,I) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>                A(M,I) = tmp3 - tmp4
  110                CONTINUE
                  END IF
  120          CONTINUE
            END IF
         END IF
      ELSE IF( LSAME( SIDE, 'R' ) ) THEN
*
*        Form A * P**T
*
         IF( LSAME( PIVOT, 'V' ) ) THEN
            IF( LSAME( DIRECT, 'F' ) ) THEN
               DO 140 J = 1, N - 1
!                  CTEMP = C( J )
!                  STEMP = S( J )
                    CTEMPh = Ch(J)
                    CTEMPl = Cl(J)
                    STEMPh = Sh(J)
                    STEMPl = Sl(J)
                  IF((
!                    ( CTEMP.NE.ONE ) 
!                     (CTEMP .ne. ONE)
     +               ((CTEMPh .ne. ONEh) .or.
     +                (CTEMPl .ne. ONEl))
     &               ).OR.( 
!                    ( STEMP.NE.ZERO )
!                     (STEMP .ne. ZERO)
     +               ((STEMPh .ne. ZEROh) .or.
     +                (STEMPl .ne. ZEROl))
     &               )) THEN
                     DO 130 I = 1, M
!                        TEMP = A( I, J+1 )
                    TEMPh = Ah(I,J+1)
                    TEMPl = Al(I,J+1)
!                        A( I, J+1 ) = CTEMP*TEMP - STEMP*A( I, J )
!<<<                tmp1 = CTEMP * TEMP
                    tmp1h = CTEMPh * TEMPh
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp1l = ((T_p1*T_r1
     +                      - tmp1h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp1l = tmp1l
     +                    + (CTEMPh*TEMPl
     +                    + CTEMPl*TEMPh)
                    T_s1 = tmp1h + tmp1l
                    tmp1l = tmp1l - (T_s1 - tmp1h)
                    tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp1 = CTEMP * TEMP
!<<<                tmp2 = STEMP * A(I,J)
                    tmp2h = STEMPh * Ah(I,J)
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(I,J)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(I,J))
                    T_r2 = Ah(I,J) - T_r1
                    tmp2l = ((T_p1*T_r1
     +                      - tmp2h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp2l = tmp2l
     +                    + (STEMPh*Al(I,J)
     +                    + STEMPl*Ah(I,J))
                    T_s1 = tmp2h + tmp2l
                    tmp2l = tmp2l - (T_s1 - tmp2h)
                    tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp2 = STEMP * A(I,J)
!<<<                A(I,J+1) = tmp1 - tmp2
                    T_p1 = tmp1h - tmp2h
                    T_p2 = T_p1 - tmp1h
                    Ah(I,J+1) = T_p1
                    Al(I,J+1) = (tmp1h - (T_p1 - T_p2))
     +                       - (tmp2h + T_p2)
                    Al(I,J+1) = Al(I,J+1) + tmp1l - tmp2l
                    T_s1 = Ah(I,J+1) + Al(I,J+1)
                    Al(I,J+1) = Al(I,J+1) - (T_s1 - Ah(I,J+1))
                    Ah(I,J+1) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>                A(I,J+1) = tmp1 - tmp2
!                        A( I, J ) = STEMP*TEMP + CTEMP*A( I, J )
!<<<                tmp3 = STEMP * TEMP
                    tmp3h = STEMPh * TEMPh
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp3l = ((T_p1*T_r1
     +                      - tmp3h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp3l = tmp3l
     +                    + (STEMPh*TEMPl
     +                    + STEMPl*TEMPh)
                    T_s1 = tmp3h + tmp3l
                    tmp3l = tmp3l - (T_s1 - tmp3h)
                    tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp3 = STEMP * TEMP
!<<<                tmp4 = CTEMP * A(I,J)
                    tmp4h = CTEMPh * Ah(I,J)
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(I,J)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(I,J))
                    T_r2 = Ah(I,J) - T_r1
                    tmp4l = ((T_p1*T_r1
     +                      - tmp4h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp4l = tmp4l
     +                    + (CTEMPh*Al(I,J)
     +                    + CTEMPl*Ah(I,J))
                    T_s1 = tmp4h + tmp4l
                    tmp4l = tmp4l - (T_s1 - tmp4h)
                    tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp4 = CTEMP * A(I,J)
!<<<                A(I,J) = tmp3 + tmp4
                    T_p1 = tmp3h + tmp4h
                    T_p2 = T_p1 - tmp3h
                    Ah(I,J) = T_p1
                    Al(I,J) = (tmp3h - (T_p1 - T_p2))
     +                    + (tmp4h - T_p2)
                    Al(I,J) = Al(I,J) + tmp3l + tmp4l
                    T_s1 = Ah(I,J) + Al(I,J)
                    Al(I,J) = Al(I,J) - (T_s1 - Ah(I,J))
                    Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                A(I,J) = tmp3 + tmp4
  130                CONTINUE
                  END IF
  140          CONTINUE
            ELSE IF( LSAME( DIRECT, 'B' ) ) THEN
               DO 160 J = N - 1, 1, -1
!                  CTEMP = C( J )
!                  STEMP = S( J )
                    CTEMPh = Ch(J)
                    CTEMPl = Cl(J)
                    STEMPh = Sh(J)
                    STEMPl = Sl(J)
                  IF(( 
!                    ( CTEMP.NE.ONE ) 
!                     (CTEMP .ne. ONE)
     +               ((CTEMPh .ne. ONEh) .or.
     +                (CTEMPl .ne. ONEl))
     &               ).OR.( 
!                    ( STEMP.NE.ZERO )
!                     (STEMP .ne. ZERO)
     +               ((STEMPh .ne. ZEROh) .or.
     +                (STEMPl .ne. ZEROl))
     &                )) THEN
                     DO 150 I = 1, M
!                        TEMP = A( I, J+1 )
                    TEMPh = Ah(I,J+1)
                    TEMPl = Al(I,J+1)
!                        A( I, J+1 ) = CTEMP*TEMP - STEMP*A( I, J )
!<<<                tmp1 = CTEMP * TEMP
                    tmp1h = CTEMPh * TEMPh
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp1l = ((T_p1*T_r1
     +                      - tmp1h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp1l = tmp1l
     +                    + (CTEMPh*TEMPl
     +                    + CTEMPl*TEMPh)
                    T_s1 = tmp1h + tmp1l
                    tmp1l = tmp1l - (T_s1 - tmp1h)
                    tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp1 = CTEMP * TEMP
!<<<                tmp2 = STEMP * A(I,J)
                    tmp2h = STEMPh * Ah(I,J)
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(I,J)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(I,J))
                    T_r2 = Ah(I,J) - T_r1
                    tmp2l = ((T_p1*T_r1
     +                      - tmp2h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp2l = tmp2l
     +                    + (STEMPh*Al(I,J)
     +                    + STEMPl*Ah(I,J))
                    T_s1 = tmp2h + tmp2l
                    tmp2l = tmp2l - (T_s1 - tmp2h)
                    tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp2 = STEMP * A(I,J)
!<<<                A(I,J+1) = tmp1 - tmp2
                    T_p1 = tmp1h - tmp2h
                    T_p2 = T_p1 - tmp1h
                    Ah(I,J+1) = T_p1
                    Al(I,J+1) = (tmp1h - (T_p1 - T_p2))
     +                       - (tmp2h + T_p2)
                    Al(I,J+1) = Al(I,J+1) + tmp1l - tmp2l
                    T_s1 = Ah(I,J+1) + Al(I,J+1)
                    Al(I,J+1) = Al(I,J+1) - (T_s1 - Ah(I,J+1))
                    Ah(I,J+1) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>                A(I,J+1) = tmp1 - tmp2
!                        A( I, J ) = STEMP*TEMP + CTEMP*A( I, J )
!<<<                tmp3 = STEMP * TEMP
                    tmp3h = STEMPh * TEMPh
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp3l = ((T_p1*T_r1
     +                      - tmp3h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp3l = tmp3l
     +                    + (STEMPh*TEMPl
     +                    + STEMPl*TEMPh)
                    T_s1 = tmp3h + tmp3l
                    tmp3l = tmp3l - (T_s1 - tmp3h)
                    tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp3 = STEMP * TEMP
!<<<                tmp4 = CTEMP * A(I,J)
                    tmp4h = CTEMPh * Ah(I,J)
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(I,J)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(I,J))
                    T_r2 = Ah(I,J) - T_r1
                    tmp4l = ((T_p1*T_r1
     +                      - tmp4h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp4l = tmp4l
     +                    + (CTEMPh*Al(I,J)
     +                    + CTEMPl*Ah(I,J))
                    T_s1 = tmp4h + tmp4l
                    tmp4l = tmp4l - (T_s1 - tmp4h)
                    tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp4 = CTEMP * A(I,J)
!<<<                A(I,J) = tmp3 + tmp4
                    T_p1 = tmp3h + tmp4h
                    T_p2 = T_p1 - tmp3h
                    Ah(I,J) = T_p1
                    Al(I,J) = (tmp3h - (T_p1 - T_p2))
     +                    + (tmp4h - T_p2)
                    Al(I,J) = Al(I,J) + tmp3l + tmp4l
                    T_s1 = Ah(I,J) + Al(I,J)
                    Al(I,J) = Al(I,J) - (T_s1 - Ah(I,J))
                    Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                A(I,J) = tmp3 + tmp4
  150                CONTINUE
                  END IF
  160          CONTINUE
            END IF
         ELSE IF( LSAME( PIVOT, 'T' ) ) THEN
            IF( LSAME( DIRECT, 'F' ) ) THEN
               DO 180 J = 2, N
!                  CTEMP = C( J-1 )
!                  STEMP = S( J-1 )
                    CTEMPh = Ch(J-1)
                    CTEMPl = Cl(J-1)
                    STEMPh = Sh(J-1)
                    STEMPl = Sl(J-1)
                  IF(( 
!                    ( CTEMP.NE.ONE ) 
!                     (CTEMP .ne. ONE)
     +               ((CTEMPh .ne. ONEh) .or.
     +                (CTEMPl .ne. ONEl))
     &                ).OR.(
!                    ( STEMP.NE.ZERO )
!                     (STEMP .ne. ZERO)
     +               ((STEMPh .ne. ZEROh) .or.
     +                (STEMPl .ne. ZEROl))
     &                )) THEN
                     DO 170 I = 1, M
!                        TEMP = A( I, J )
                    TEMPh = Ah(I,J)
                    TEMPl = Al(I,J)
!                        A( I, J ) = CTEMP*TEMP - STEMP*A( I, 1 )
!<<<                tmp1 = CTEMP * TEMP
                    tmp1h = CTEMPh * TEMPh
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp1l = ((T_p1*T_r1
     +                      - tmp1h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp1l = tmp1l
     +                    + (CTEMPh*TEMPl
     +                    + CTEMPl*TEMPh)
                    T_s1 = tmp1h + tmp1l
                    tmp1l = tmp1l - (T_s1 - tmp1h)
                    tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp1 = CTEMP * TEMP
!<<<                tmp2 = STEMP * A(I,1)
                    tmp2h = STEMPh * Ah(I,1)
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(I,1)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(I,1))
                    T_r2 = Ah(I,1) - T_r1
                    tmp2l = ((T_p1*T_r1
     +                      - tmp2h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp2l = tmp2l
     +                    + (STEMPh*Al(I,1)
     +                    + STEMPl*Ah(I,1))
                    T_s1 = tmp2h + tmp2l
                    tmp2l = tmp2l - (T_s1 - tmp2h)
                    tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp2 = STEMP * A(I,1)
!<<<                A(I,J) = tmp1 - tmp2
                    T_p1 = tmp1h - tmp2h
                    T_p2 = T_p1 - tmp1h
                    Ah(I,J) = T_p1
                    Al(I,J) = (tmp1h - (T_p1 - T_p2))
     +                       - (tmp2h + T_p2)
                    Al(I,J) = Al(I,J) + tmp1l - tmp2l
                    T_s1 = Ah(I,J) + Al(I,J)
                    Al(I,J) = Al(I,J) - (T_s1 - Ah(I,J))
                    Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>                A(I,J) = tmp1 - tmp2
!                        A( I, 1 ) = STEMP*TEMP + CTEMP*A( I, 1 )
!<<<                tmp3 = STEMP * TEMP
                    tmp3h = STEMPh * TEMPh
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp3l = ((T_p1*T_r1
     +                      - tmp3h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp3l = tmp3l
     +                    + (STEMPh*TEMPl
     +                    + STEMPl*TEMPh)
                    T_s1 = tmp3h + tmp3l
                    tmp3l = tmp3l - (T_s1 - tmp3h)
                    tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp3 = STEMP * TEMP
!<<<                tmp4 = CTEMP * A(I,1)
                    tmp4h = CTEMPh * Ah(I,1)
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(I,1)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(I,1))
                    T_r2 = Ah(I,1) - T_r1
                    tmp4l = ((T_p1*T_r1
     +                      - tmp4h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp4l = tmp4l
     +                    + (CTEMPh*Al(I,1)
     +                    + CTEMPl*Ah(I,1))
                    T_s1 = tmp4h + tmp4l
                    tmp4l = tmp4l - (T_s1 - tmp4h)
                    tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp4 = CTEMP * A(I,1)
!<<<                A(I,1) = tmp3 + tmp4
                    T_p1 = tmp3h + tmp4h
                    T_p2 = T_p1 - tmp3h
                    Ah(I,1) = T_p1
                    Al(I,1) = (tmp3h - (T_p1 - T_p2))
     +                    + (tmp4h - T_p2)
                    Al(I,1) = Al(I,1) + tmp3l + tmp4l
                    T_s1 = Ah(I,1) + Al(I,1)
                    Al(I,1) = Al(I,1) - (T_s1 - Ah(I,1))
                    Ah(I,1) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                A(I,1) = tmp3 + tmp4
  170                CONTINUE
                  END IF
  180          CONTINUE
            ELSE IF( LSAME( DIRECT, 'B' ) ) THEN
               DO 200 J = N, 2, -1
!                  CTEMP = C( J-1 )
!                  STEMP = S( J-1 )
                    CTEMPh = Ch(J-1)
                    CTEMPl = Cl(J-1)
                    STEMPh = Sh(J-1)
                    STEMPl = Sl(J-1)
                  IF((
!                    ( CTEMP.NE.ONE ) 
!                     (CTEMP .ne. ONE)
     +               ((CTEMPh .ne. ONEh) .or.
     +                (CTEMPl .ne. ONEl))
     &               ).OR.(
!                    ( STEMP.NE.ZERO )
!                     (STEMP .ne. ZERO)
     +               ((STEMPh .ne. ZEROh) .or.
     +                (STEMPl .ne. ZEROl))
     &               )) THEN
                     DO 190 I = 1, M
!                        TEMP = A( I, J )
                    TEMPh = Ah(I,J)
                    TEMPl = Al(I,J)
!                        A( I, J ) = CTEMP*TEMP - STEMP*A( I, 1 )
!<<<                tmp1 = CTEMP * TEMP
                    tmp1h = CTEMPh * TEMPh
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp1l = ((T_p1*T_r1
     +                      - tmp1h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp1l = tmp1l
     +                    + (CTEMPh*TEMPl
     +                    + CTEMPl*TEMPh)
                    T_s1 = tmp1h + tmp1l
                    tmp1l = tmp1l - (T_s1 - tmp1h)
                    tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp1 = CTEMP * TEMP
!<<<                tmp2 = STEMP * A(I,1)
                    tmp2h = STEMPh * Ah(I,1)
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(I,1)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(I,1))
                    T_r2 = Ah(I,1) - T_r1
                    tmp2l = ((T_p1*T_r1
     +                      - tmp2h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp2l = tmp2l
     +                    + (STEMPh*Al(I,1)
     +                    + STEMPl*Ah(I,1))
                    T_s1 = tmp2h + tmp2l
                    tmp2l = tmp2l - (T_s1 - tmp2h)
                    tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp2 = STEMP * A(I,1)
!<<<                A(I,J) = tmp1 - tmp2
                    T_p1 = tmp1h - tmp2h
                    T_p2 = T_p1 - tmp1h
                    Ah(I,J) = T_p1
                    Al(I,J) = (tmp1h - (T_p1 - T_p2))
     +                       - (tmp2h + T_p2)
                    Al(I,J) = Al(I,J) + tmp1l - tmp2l
                    T_s1 = Ah(I,J) + Al(I,J)
                    Al(I,J) = Al(I,J) - (T_s1 - Ah(I,J))
                    Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>                A(I,J) = tmp1 - tmp2
!                        A( I, 1 ) = STEMP*TEMP + CTEMP*A( I, 1 )
!<<<                tmp3 = STEMP * TEMP
                    tmp3h = STEMPh * TEMPh
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp3l = ((T_p1*T_r1
     +                      - tmp3h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp3l = tmp3l
     +                    + (STEMPh*TEMPl
     +                    + STEMPl*TEMPh)
                    T_s1 = tmp3h + tmp3l
                    tmp3l = tmp3l - (T_s1 - tmp3h)
                    tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp3 = STEMP * TEMP
!<<<                tmp4 = CTEMP * A(I,1)
                    tmp4h = CTEMPh * Ah(I,1)
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(I,1)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(I,1))
                    T_r2 = Ah(I,1) - T_r1
                    tmp4l = ((T_p1*T_r1
     +                      - tmp4h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp4l = tmp4l
     +                    + (CTEMPh*Al(I,1)
     +                    + CTEMPl*Ah(I,1))
                    T_s1 = tmp4h + tmp4l
                    tmp4l = tmp4l - (T_s1 - tmp4h)
                    tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp4 = CTEMP * A(I,1)
!<<<                A(I,1) = tmp3 + tmp4
                    T_p1 = tmp3h + tmp4h
                    T_p2 = T_p1 - tmp3h
                    Ah(I,1) = T_p1
                    Al(I,1) = (tmp3h - (T_p1 - T_p2))
     +                    + (tmp4h - T_p2)
                    Al(I,1) = Al(I,1) + tmp3l + tmp4l
                    T_s1 = Ah(I,1) + Al(I,1)
                    Al(I,1) = Al(I,1) - (T_s1 - Ah(I,1))
                    Ah(I,1) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                A(I,1) = tmp3 + tmp4
  190                CONTINUE
                  END IF
  200          CONTINUE
            END IF
         ELSE IF( LSAME( PIVOT, 'B' ) ) THEN
            IF( LSAME( DIRECT, 'F' ) ) THEN
               DO 220 J = 1, N - 1
!                  CTEMP = C( J )
!                  STEMP = S( J )
                    CTEMPh = Ch(J)
                    CTEMPl = Cl(J)
                    STEMPh = Sh(J)
                    STEMPl = Sl(J)
                  IF(( 
!                     ( CTEMP.NE.ONE ) 
!                     (CTEMP .ne. ONE)
     +               ((CTEMPh .ne. ONEh) .or.
     +                (CTEMPl .ne. ONEl))
     &               ).OR.(
!                     ( STEMP.NE.ZERO )
!                     (STEMP .ne. ZERO)
     +               ((STEMPh .ne. ZEROh) .or.
     +                (STEMPl .ne. ZEROl))
     &                )) THEN
                     DO 210 I = 1, M
!                        TEMP = A( I, J )
                    TEMPh = Ah(I,J)
                    TEMPl = Al(I,J)
!                        A( I, J ) = STEMP*A( I, N ) + CTEMP*TEMP
!<<<                tmp1 = STEMP * A(I,N)
                    tmp1h = STEMPh * Ah(I,N)
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(I,N)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(I,N))
                    T_r2 = Ah(I,N) - T_r1
                    tmp1l = ((T_p1*T_r1
     +                      - tmp1h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp1l = tmp1l
     +                    + (STEMPh*Al(I,N)
     +                    + STEMPl*Ah(I,N))
                    T_s1 = tmp1h + tmp1l
                    tmp1l = tmp1l - (T_s1 - tmp1h)
                    tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp1 = STEMP * A(I,N)
!<<<                tmp2 = CTEMP * TEMP
                    tmp2h = CTEMPh * TEMPh
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp2l = ((T_p1*T_r1
     +                      - tmp2h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp2l = tmp2l
     +                    + (CTEMPh*TEMPl
     +                    + CTEMPl*TEMPh)
                    T_s1 = tmp2h + tmp2l
                    tmp2l = tmp2l - (T_s1 - tmp2h)
                    tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp2 = CTEMP * TEMP
!<<<                A(I,J) = tmp1 + tmp2
                    T_p1 = tmp1h + tmp2h
                    T_p2 = T_p1 - tmp1h
                    Ah(I,J) = T_p1
                    Al(I,J) = (tmp1h - (T_p1 - T_p2))
     +                    + (tmp2h - T_p2)
                    Al(I,J) = Al(I,J) + tmp1l + tmp2l
                    T_s1 = Ah(I,J) + Al(I,J)
                    Al(I,J) = Al(I,J) - (T_s1 - Ah(I,J))
                    Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                A(I,J) = tmp1 + tmp2
!                        A( I, N ) = CTEMP*A( I, N ) - STEMP*TEMP
!<<<                tmp3 = CTEMP * A(I,N)
                    tmp3h = CTEMPh * Ah(I,N)
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(I,N)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(I,N))
                    T_r2 = Ah(I,N) - T_r1
                    tmp3l = ((T_p1*T_r1
     +                      - tmp3h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp3l = tmp3l
     +                    + (CTEMPh*Al(I,N)
     +                    + CTEMPl*Ah(I,N))
                    T_s1 = tmp3h + tmp3l
                    tmp3l = tmp3l - (T_s1 - tmp3h)
                    tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp3 = CTEMP * A(I,N)
!<<<                tmp4 = STEMP * TEMP
                    tmp4h = STEMPh * TEMPh
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp4l = ((T_p1*T_r1
     +                      - tmp4h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp4l = tmp4l
     +                    + (STEMPh*TEMPl
     +                    + STEMPl*TEMPh)
                    T_s1 = tmp4h + tmp4l
                    tmp4l = tmp4l - (T_s1 - tmp4h)
                    tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp4 = STEMP * TEMP
!<<<                A(I,N) = tmp3 - tmp4
                    T_p1 = tmp3h - tmp4h
                    T_p2 = T_p1 - tmp3h
                    Ah(I,N) = T_p1
                    Al(I,N) = (tmp3h - (T_p1 - T_p2))
     +                       - (tmp4h + T_p2)
                    Al(I,N) = Al(I,N) + tmp3l - tmp4l
                    T_s1 = Ah(I,N) + Al(I,N)
                    Al(I,N) = Al(I,N) - (T_s1 - Ah(I,N))
                    Ah(I,N) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>                A(I,N) = tmp3 - tmp4

  210                CONTINUE
                  END IF
  220          CONTINUE
            ELSE IF( LSAME( DIRECT, 'B' ) ) THEN
               DO 240 J = N - 1, 1, -1
!                  CTEMP = C( J )
!                  STEMP = S( J )
                    CTEMPh = Ch(J)
                    CTEMPl = Cl(J)
                    STEMPh = Sh(J)
                    STEMPl = Sl(J)
                  IF(( 
!                    ( CTEMP.NE.ONE ) 
!                     (CTEMP .ne. ONE)
     +               ((CTEMPh .ne. ONEh) .or.
     +                (CTEMPl .ne. ONEl))
     &               ).OR.( 
!                    ( STEMP.NE.ZERO )
!                     (STEMP .ne. ZERO)
     +               ((STEMPh .ne. ZEROh) .or.
     +                (STEMPl .ne. ZEROl))
     &               )) THEN
                     DO 230 I = 1, M
!                        TEMP = A( I, J )
                    TEMPh = Ah(I,J)
                    TEMPl = Al(I,J)
!                        A( I, J ) = STEMP*A( I, N ) + CTEMP*TEMP
!<<<                tmp1 = STEMP * A(I,N)
                    tmp1h = STEMPh * Ah(I,N)
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(I,N)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(I,N))
                    T_r2 = Ah(I,N) - T_r1
                    tmp1l = ((T_p1*T_r1
     +                      - tmp1h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp1l = tmp1l
     +                    + (STEMPh*Al(I,N)
     +                    + STEMPl*Ah(I,N))
                    T_s1 = tmp1h + tmp1l
                    tmp1l = tmp1l - (T_s1 - tmp1h)
                    tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp1 = STEMP * A(I,N)
!<<<                tmp2 = CTEMP * TEMP
                    tmp2h = CTEMPh * TEMPh
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp2l = ((T_p1*T_r1
     +                      - tmp2h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp2l = tmp2l
     +                    + (CTEMPh*TEMPl
     +                    + CTEMPl*TEMPh)
                    T_s1 = tmp2h + tmp2l
                    tmp2l = tmp2l - (T_s1 - tmp2h)
                    tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp2 = CTEMP * TEMP
!<<<                A(I,J) = tmp1 + tmp2
                    T_p1 = tmp1h + tmp2h
                    T_p2 = T_p1 - tmp1h
                    Ah(I,J) = T_p1
                    Al(I,J) = (tmp1h - (T_p1 - T_p2))
     +                    + (tmp2h - T_p2)
                    Al(I,J) = Al(I,J) + tmp1l + tmp2l
                    T_s1 = Ah(I,J) + Al(I,J)
                    Al(I,J) = Al(I,J) - (T_s1 - Ah(I,J))
                    Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                A(I,J) = tmp1 + tmp2
!                        A( I, N ) = CTEMP*A( I, N ) - STEMP*TEMP
!<<<                tmp3 = CTEMP * A(I,N)
                    tmp3h = CTEMPh * Ah(I,N)
                    T_u1 = 134217729.0 * CTEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - CTEMPh)
                    T_p2 = CTEMPh - T_p1
                    T_u1 = 134217729.0 * Ah(I,N)
                    T_r1 = T_u1
     +                       - (T_u1 - Ah(I,N))
                    T_r2 = Ah(I,N) - T_r1
                    tmp3l = ((T_p1*T_r1
     +                      - tmp3h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp3l = tmp3l
     +                    + (CTEMPh*Al(I,N)
     +                    + CTEMPl*Ah(I,N))
                    T_s1 = tmp3h + tmp3l
                    tmp3l = tmp3l - (T_s1 - tmp3h)
                    tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp3 = CTEMP * A(I,N)
!<<<                tmp4 = STEMP * TEMP
                    tmp4h = STEMPh * TEMPh
                    T_u1 = 134217729.0 * STEMPh
                    T_p1 = T_u1
     +                       - (T_u1 - STEMPh)
                    T_p2 = STEMPh - T_p1
                    T_u1 = 134217729.0 * TEMPh
                    T_r1 = T_u1
     +                       - (T_u1 - TEMPh)
                    T_r2 = TEMPh - T_r1
                    tmp4l = ((T_p1*T_r1
     +                      - tmp4h)
     +                      + T_p1 * T_r2
     +                      + T_p2 * T_r1)
     +                      + T_p2 * T_r2
                    tmp4l = tmp4l
     +                    + (STEMPh*TEMPl
     +                    + STEMPl*TEMPh)
                    T_s1 = tmp4h + tmp4l
                    tmp4l = tmp4l - (T_s1 - tmp4h)
                    tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                tmp4 = STEMP * TEMP
!<<<                A(I,N) = tmp3 - tmp4
                    T_p1 = tmp3h - tmp4h
                    T_p2 = T_p1 - tmp3h
                    Ah(I,N) = T_p1
                    Al(I,N) = (tmp3h - (T_p1 - T_p2))
     +                       - (tmp4h + T_p2)
                    Al(I,N) = Al(I,N) + tmp3l - tmp4l
                    T_s1 = Ah(I,N) + Al(I,N)
                    Al(I,N) = Al(I,N) - (T_s1 - Ah(I,N))
                    Ah(I,N) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>                A(I,N) = tmp3 - tmp4
  230                CONTINUE
                  END IF
  240          CONTINUE
            END IF
         END IF
      END IF
*
      RETURN
*
*     End of DLASR
*
      END
